package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

public class BoolProp extends Property {

  public boolean value;
  public boolean values[] = new boolean[0];;

  public BoolProp() {
    super();
    mType = BOOLEAN;
  }

  public void copyValue(Property property) {
    if (isArray) {
      values = (boolean[])((BoolProp)property).values.clone();
    }
    else {
      value = ((BoolProp)property).value;
    }
  }

  public String valueAsString() {
    validate();
    return String.valueOf(value);
  }

  public int arrayLength() {
    return values.length;
  }

  public void setArrayLength(int length) {
    values = new boolean[length];
  }

  public void setValueByName(String fieldName, String aValue) {  // used for compound types
    value = Util.parseBoolean(aValue);
  }

  public void setValueByName(String fieldName, int index, String aValue) {  // used for compound types
    values[index] = Util.parseBoolean(aValue);
  }

  public void insertAt(int index) {
    values = Util.insertBoolean(values, false, index);
  }

  public void deleteItem(int index) {
    values = Util.removeBoolean(values, index);
  }

  public boolean serializeProperty(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("property");

      super.serializeProperty(xmlOut);
      if (isArray) {
        xmlOut.startElement("value");
        Util.serializeElement(xmlOut, "length", values.length);
        for (int i = 0; i < values.length; i++) {
          Util.serializeElement(xmlOut, "index", values[i]);
        }
        xmlOut.endElement("value");
      }
      else {
        Util.serializeElement(xmlOut, "value", value);
      }

      xmlOut.endElement("property");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }
}