package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.geom.*;
import java.awt.event.*;

public class Button extends GraphicEntity {

  public JToggleButtonNoAntialias mButton = new JToggleButtonNoAntialias();

  private Point2D mPreDragTopLeft = new Point2D.Double();
  private Point2D mPreDragBottomRight = new Point2D.Double();
  private Point2D mPostDragTopLeft = new Point2D.Double();
  private Point2D mPostDragBottomRight = new Point2D.Double();

  private LocationProp mTopLeftProperty;
  private Point2D.Double mTopLeft = new Point2D.Double();
  private LocationProp mBottomRightProperty;
  private Point2D.Double mBottomRight = new Point2D.Double();
  private StringProp mTextProperty;
  private String mText;
  private BoolProp mSelectedProperty;
  private boolean mSelected = false;
  private BoolProp mVisibleProperty;
  private BoolProp mSelectableProperty;
  private NumProp mLayerProperty;
  private Point2D.Double mPanelTopLeft = new Point2D.Double();
  private Point2D.Double mPanelBottomRight = new Point2D.Double();

  public Button() {
    super();
    mEntityType = "button";
    addProperties();
  }

  public Button(String name, Enclosure enclosure) {
    super(name, enclosure);
    mEntityType = "button";
    addProperties();
    g.scenePanel.add(mButton);
    mButton.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        buttonSelected();
      }
    });
  }

  protected void addProperties() {
    mTextProperty = new StringProp();
    mTextProperty.mName = "Text";
    mTextProperty.value = "Button";
    addProperty(mTextProperty);

    mTopLeftProperty = new LocationProp();
    mTopLeftProperty.mName = "TopLeft";
    addProperty(mTopLeftProperty);

    mBottomRightProperty = new LocationProp();
    mBottomRightProperty.mName = "BottomRight";
    addProperty(mBottomRightProperty);

    mSelectedProperty = new BoolProp();
    mSelectedProperty.mName = "Selected";
//    mSelectedProperty.setIsInput(false);
    addProperty(mSelectedProperty);

    mLayerProperty = new NumProp();
    mLayerProperty.mName = "Layer";
    mLayerProperty.value = 0;
    addProperty(mLayerProperty);

    mVisibleProperty = new BoolProp();
    mVisibleProperty.mName = "Visible";
    mVisibleProperty.value = true;
    addProperty(mVisibleProperty);

    mSelectableProperty = new BoolProp();
    mSelectableProperty.mName = "Selectable";
    mSelectableProperty.value = true;
    addProperty(mSelectableProperty);
  }

    public void setTopLeft(Point2D point) {
        mTopLeftProperty.x = point.getX();
        mTopLeftProperty.y = point.getY();
        mTopLeftProperty.valueSet();
    }

    public void setBottomRight(Point2D point) {
        mBottomRightProperty.x = point.getX();
        mBottomRightProperty.y = point.getY();
        mBottomRightProperty.valueSet();
    }

    public void setText(String text) {
        mTextProperty.value = text;
        mTextProperty.valueSet();
    }

    protected void readInputs() {


        mTextProperty.validate();
        mText = mTextProperty.value;

        mVisibleProperty.validate();
        mVisible = mVisibleProperty.value;

        mSelectableProperty.validate();
        mSelectable = mSelectableProperty.value;

    mLayerProperty.validate();
    mLayer = (int)mLayerProperty.value;

        mTopLeftProperty.validate();
        mTopLeft.setLocation(mTopLeftProperty.x, mTopLeftProperty.y);


        mBottomRightProperty.validate();
        mBottomRight.setLocation(mBottomRightProperty.x, mBottomRightProperty.y);

        mSelectedProperty.validate();
        mSelected = mSelectedProperty.value;
        if (mButton.isSelected() != mSelected) {
          mButton.setSelected(mSelected);
        }
    }

    protected void writeOutputs() {
        mSelectedProperty.value = mButton.isSelected();
    }

    public void run() {
      mButton.setText(mText);
      mPanelTopLeft.setLocation(g.transform.toPanelLocation(mTopLeft));
      mPanelBottomRight.setLocation(g.transform.toPanelLocation(mBottomRight));
      mButton.setLocation((int)mPanelTopLeft.getX(), (int)mPanelTopLeft.getY());
      mButton.setSize((int)(mPanelBottomRight.getX() - mPanelTopLeft.getX()), (int)(mPanelBottomRight.getY() - mPanelTopLeft.getY()));
      mButton.setVisible(mVisible);
      mButton.setEnabled(mSelectable);

      g.scenePanel.validate();
    }

    public void move(Point2D mouseDelta, ScenePanelTransform transform) {
        mPostDragTopLeft.setLocation(mPreDragTopLeft.getX() + mouseDelta.getX(),
                                       mPreDragTopLeft.getY() + mouseDelta.getY());
        mPostDragBottomRight.setLocation(mPreDragBottomRight.getX() + mouseDelta.getX(),
                                       mPreDragBottomRight.getY() + mouseDelta.getY());

        mTopLeftProperty.x = g.transform.toGeomLocation(mPostDragTopLeft).getX();
        mTopLeftProperty.y = g.transform.toGeomLocation(mPostDragTopLeft).getY();
        mTopLeftProperty.valueSet();

        mBottomRightProperty.x = g.transform.toGeomLocation(mPostDragBottomRight).getX();
        mBottomRightProperty.y = g.transform.toGeomLocation(mPostDragBottomRight).getY();
        mBottomRightProperty.valueSet();
    }

    public void savePreDragLocation() {
        mPreDragTopLeft.setLocation(mPanelTopLeft);
        mPreDragBottomRight.setLocation(mPanelBottomRight);
    }

    public void buttonSelected() {
        mSelected = true;
        mSelectedProperty.value = mButton.isSelected();
        mSelectedProperty.valueSet();
        g.scene.refresh();

/*
        Property[] children = mHitOrSelectedProperty.getChildren();
        for (int i = 0; i < children.length; i++) {
            children[i].getOwner().needsRefresh();
            children[i].getOwner().update();
        }
        getScene().refresh();
        mHitOrSelected = false;
        mHitOrSelectedProperty.setBooleanValue(false);
        getScene().refresh();*/
    }

    public void delete() {
        super.delete();
        g.scenePanel.remove(mButton);
        g.scenePanel.validate();
    }
}


/*
    public Button() {
    }

    public Button(String name, Enclosure context) {
    }

    public Button(String name, Enclosure context, boolean addProperties) {
        super(name, context, addProperties);
        mScenePanelTransform = getScene().getTransform();
        mSceneManager.getScenePanel().add(mButton);
        mButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                buttonHit();
            }
        });
        mToggleButton.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                toggleButtonHit();
            }
        });
    }

    protected String getCategory() {
        return "BUTTON";
    }



    public void initializeProperties() {        // To be used by loading functions

    }


    public void setIsToggle(boolean isToggle) {
        mIsToggleProperty.setBooleanValue(isToggle);
        if (!isToggle) {
            mHitOrSelected = false;
        }
    }




    public void draw(Graphics2D g, ScenePanelTransform transform) {
        update();
    }


    public void toggleButtonHit() {
        mHitOrSelected = !mHitOrSelected;
        mHitOrSelectedProperty.setBooleanValue(mHitOrSelected);
        mToggleButton.setSelected(mHitOrSelected);
        mSceneManager.getScene().refresh();
//        mSceneManager.getSceneGraph().update();
    }

    public  boolean hit(Point2D hitLocation)  {   // Not good enough
//        return mPaintLine.intersects(hitLocation.getX() - 2, hitLocation.getY() - 2, 4, 4);
        return false;
    }




    protected boolean serializeEntity(XMLSerializer xmlOut) {
        try {
            xmlOut.startElement("entity", null);
            Util.serializeElement(xmlOut, "type", "Button");
            if (! super.serializeEntity(xmlOut))
                return false;

            xmlOut.endElement("entity");
        }
        catch (Exception e) {
            return false;
        }
        return true;
    }
}*/