package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.*;

public class ColorProp extends Property {

  public double r;
  public double g;
  public double b;
  public double a;

  private double rs[] = new double[0];
  private double gs[] = new double[0];
  private double bs[] = new double[0];
  private double as[] = new double[0];

  public ColorProp() {
    super();
    mType = COLOR;
    isCompound = true;
  }

  public Color getColorValue() {
    return new Color((int)r, (int)g, (int)b, (int)a);
  }

  public String getValueByName(String fieldName) {
    if (fieldName.equals("r")) {
      return String.valueOf(r);
    }
    else if (fieldName.equals("g")) {
      return String.valueOf(g);
    }
    else if (fieldName.equals("b")) {
      return String.valueOf(b);
    }
    else if (fieldName.equals("a")) {
      return String.valueOf(a);
    }
    else
      return "";
  }

  public String getValueByName(String fieldName, int index) {
    if (fieldName.equals("r")) {
      return String.valueOf(rs[index]);
    }
    else if (fieldName.equals("g")) {
      return String.valueOf(gs[index]);
    }
    else if (fieldName.equals("b")) {
      return String.valueOf(bs[index]);
    }
    else if (fieldName.equals("a")) {
      return String.valueOf(as[index]);
    }
    else
      return "";
  }

  public void setValueByName(String fieldName, String aValue) {  // used for compound types
    if (fieldName.equals("r")) {
      r = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("g")) {
      g = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("b")) {
      b = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("a")) {
      a = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
  }

  public void setValueByName(String fieldName, int index, String aValue) {  // used for compound types
    if (fieldName.equals("r")) {
      rs[index] = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("g")) {
      gs[index] = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("b")) {
      bs[index] = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
    else if (fieldName.equals("a")) {
      as[index] = Util.clamp(Double.valueOf(aValue).doubleValue(), 0.0, 255.0);
    }
  }

  public void copyValue(Property property) {
    if (isArray) {
      rs = (double[])((ColorProp)property).rs.clone();
      gs = (double[])((ColorProp)property).gs.clone();
      bs = (double[])((ColorProp)property).bs.clone();
      as = (double[])((ColorProp)property).as.clone();
    }
    else {
      r = ((ColorProp)property).r;
      g = ((ColorProp)property).g;
      b = ((ColorProp)property).b;
      a = ((ColorProp)property).a;
    }
  }

  public int arrayLength() {
    return rs.length;
  }

  public void setArrayLength(int length) {
    rs = new double[length];
    gs = new double[length];
    bs = new double[length];
    as = new double[length];
  }

  public void insertAt(int index) {
    rs = Util.insertDouble(rs, 0, index);
    gs = Util.insertDouble(gs, 0, index);
    bs = Util.insertDouble(bs, 0, index);
    as = Util.insertDouble(as, 0, index);
  }

  public void deleteItem(int index) {
    rs = Util.removeDouble(rs, index);
    gs = Util.removeDouble(gs, index);
    bs = Util.removeDouble(bs, index);
    as = Util.removeDouble(as, index);
  }

  public boolean serializeProperty(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("property");

      super.serializeProperty(xmlOut);
      if (isArray) {

        Util.serializeElement(xmlOut, "valueName", "r");
        xmlOut.startElement("value");
        Util.serializeElement(xmlOut, "length", rs.length);
        for (int i = 0; i < rs.length; i++) {
          Util.serializeElement(xmlOut, "index", rs[i]);
        }
        xmlOut.endElement("value");

        Util.serializeElement(xmlOut, "valueName", "g");
        xmlOut.startElement("value");
        for (int i = 0; i < gs.length; i++) {
          Util.serializeElement(xmlOut, "index", gs[i]);
        }
        xmlOut.endElement("value");

        Util.serializeElement(xmlOut, "valueName", "b");
        xmlOut.startElement("value");
        for (int i = 0; i < bs.length; i++) {
          Util.serializeElement(xmlOut, "index", bs[i]);
        }
        xmlOut.endElement("value");

        Util.serializeElement(xmlOut, "valueName", "a");
        xmlOut.startElement("value");
        for (int i = 0; i < as.length; i++) {
          Util.serializeElement(xmlOut, "index", as[i]);
        }
        xmlOut.endElement("value");
      }
      else {
        Util.serializeElement(xmlOut, "valueName", "r");
        Util.serializeElement(xmlOut, "value", r);
        Util.serializeElement(xmlOut, "valueName", "g");
        Util.serializeElement(xmlOut, "value", g);
        Util.serializeElement(xmlOut, "valueName", "b");
        Util.serializeElement(xmlOut, "value", b);
        Util.serializeElement(xmlOut, "valueName", "a");
        Util.serializeElement(xmlOut, "value", a);
      }

      xmlOut.endElement("property");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }

  public void valueSet() {
    Util.clamp(r, 0.0, 255.0);
    Util.clamp(g, 0.0, 255.0);
    Util.clamp(b, 0.0, 255.0);
    Util.clamp(a, 0.0, 255.0);
  }

}