package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

import at.dms.kjc.*;
import java.io.*;
import java.net.*;

public class Compiler {
  static private String REPLACE_CLASS_NAME = "$$$CLASS_NAME$$$";
  static private String REPLACE_TYPE_NAME = "$$$TYPE_NAME$$$";
  static private String REPLACE_FUNCTION_CODE = "$$$FUNCTION_CODE$$$";
  static private String REPLACE_RUN = "$$$RUN$$$";
  static private String REPLACE_PROPERTY_DECLARATIONS = "$$$PROPERTY_DECLARATIONS$$$";
  static private String REPLACE_INITIALIZE_PROPERTIES = "$$$INITIALIZE_PROPERTIES$$$";
  static private String REPLACE_READ_INPUTS = "$$$READ_INPUTS$$$";
  static private String REPLACE_WRITE_OUTPUTS = "$$$WRITE_OUTPUTS$$$";
  static private String REPLACE_CLASS_LEVEL_CODE = "$$$CLASS_LEVEL_CODE$$$";
  static private String REPLACE_CLASS_LEVEL_FUNCTION_CODE = "$$$CLASS_LEVEL_FUNCTION_CODE$$$";

  static private PrintWriter errorWriter;
  static private ByteArrayOutputStream byteArrayStream;
  static public String errorString;

  public Compiler() {
  }

  public static boolean compile(String[] arguments) {
    Main kjcCompiler = new Main();

    byteArrayStream = new ByteArrayOutputStream();
    errorWriter = new PrintWriter(byteArrayStream, true);

    if (kjcCompiler.run(g.USER_DIR, errorWriter, arguments)) {
//    if (kjcCompiler.run(arguments)) {

      errorWriter.flush();
      errorString = byteArrayStream.toString();
      return true;
    }
    else {
      errorWriter.flush();
      errorString = byteArrayStream.toString();
      return false;
    }
  }

  public static boolean compileFunction(String className) {
    String sourceName = g.sourceDirectory.getPath() + g.FS + className + ".java";

    if (! compile(new String[] {sourceName, "-d" + g.tempDirectory.getPath(), "-l1.2"})) {
      return false;
    }
    return true;
  }

  public static String getError() {
    String returnString;

    int startPos = errorString.indexOf("error:");
    if (startPos < 0)
      return errorString;
    startPos += 6;

    int endPos = errorString.indexOf("\n");
    if (endPos < 0)
      endPos = errorString.length();

    return errorString.substring(startPos, endPos);
  }

/*
  public static Class compileAsNewFunctionType(String typeName, Property[] properties, String code,
                                               String classLevelCode) {
    StringBuffer classCode = new StringBuffer();
    FileManager.readFileToStringBuffer(FileManager.getTemplateDirectory(), "function.txt", classCode);

    String className = "Function" + typeName + Util.generateGUID();
    replaceText(classCode, className, typeName, properties, code, classLevelCode, constraintCode);

    FileManager.writeStringToFile(FileManager.getSourceDirectory(), className + ".java", classCode.toString());
    if (! paramour.util.Compiler.compileFunction(className, scene)) {
      System.out.println("Compile failed!");
      FileManager.deleteFile(FileManager.getSourceDirectory(), className + ".java");
      return null;
    }
    FileManager.deleteFile(FileManager.getSourceDirectory(), className + ".java");
    Class functionClass = loadFunction(className, scene);
    if (functionClass != null) {
      try {
          ((Function)functionClass.newInstance()).setFile(new File(scene.getFileDirectory().getPath() +
                                                               FileManager.FILE_SEPARATOR +
                                                               "paramour" + FileManager.FILE_SEPARATOR +
                                                               "entity" + FileManager.FILE_SEPARATOR +
                                                               className + ".class"));
      }
      catch (Exception e) {
      }
    }
    return functionClass;
  }
*/
  public static Class makeNewFunctionType(Entity entity, String typeName, Property[] properties, String code,
                                          String classLevelCode) {
    StringBuffer classCode = new StringBuffer();
    FileUtil.readToStringBuffer(g.templateDirectory, "function.txt", classCode);

    String className = typeName + Util.generateGUID(); //FunctionsFrame.uniqueFunctionClassName(typeName);
    replaceText(entity, classCode, className, typeName, properties, code, classLevelCode);

//    System.out.println(classCode);

    FileUtil.writeStringToFile(g.sourceDirectory, className + ".java", classCode.toString());
    if (! compileFunction(className)) {
      System.err.println("Compile failed!");
      FileUtil.deleteFile(g.sourceDirectory, className + ".java");
      return null;
    }
    FileUtil.deleteFile(g.sourceDirectory, className + ".java");
    Class functionClass = loadFunction(className);
    if (functionClass != null) {
/*      try {
        ((Function)functionClass.newInstance()).setFile(new File(scene.getFileDirectory().getPath() +
                                                             FileManager.FILE_SEPARATOR +
                                                             "paramour" + FileManager.FILE_SEPARATOR +
                                                             "entity" + FileManager.FILE_SEPARATOR +
                                                             className + ".class"));
      }
      catch (Exception e) {
        System.err.println("Error instantiating function class.");
        e.printStackTrace();
      }*/
    }
    return functionClass;
  }

  public static Class loadFunction(String className) {
    return loadFunction(g.tempDirectory.getPath(), className);
  }

  public static Class loadFunction(String directory, String className) {
    URL classURL;
    try {
      classURL = new URL("file:\\\\" + directory + g.FS);
    }
    catch (Exception e) {
      System.err.println("Bad URL");
      return null;
    }
    URLClassLoader classLoader = new URLClassLoader(new URL[] {classURL});
    try {
      Class returnClass = classLoader.loadClass("roebling." + className);
      return returnClass;
    }
    catch (Exception e) {
      System.err.println("Class " + className + " could not be loaded.");
      return null;
    }
  }

  private static void replaceText(Entity entity, StringBuffer classCode, String className, String typeName,
                       Property[] properties, String functionCode, String classLevelCode) {
    Util.replaceString(classCode, REPLACE_CLASS_NAME, className);
    Util.replaceString(classCode, REPLACE_TYPE_NAME, typeName);
    Util.replaceString(classCode, REPLACE_FUNCTION_CODE, textToJavaString(functionCode));
    Util.replaceString(classCode, REPLACE_RUN, functionCode);
    Util.replaceString(classCode, REPLACE_CLASS_LEVEL_FUNCTION_CODE,
                       textToJavaString(classLevelCode));
    Util.replaceString(classCode, REPLACE_CLASS_LEVEL_CODE, classLevelCode);
    Util.replaceString(classCode, REPLACE_PROPERTY_DECLARATIONS,
                       makePropertyDeclarations(properties));
    Util.replaceString(classCode, REPLACE_READ_INPUTS, makeReadInputs(properties));
    Util.replaceString(classCode, REPLACE_WRITE_OUTPUTS, makeWriteOutputs(properties));
    Util.replaceString(classCode, REPLACE_INITIALIZE_PROPERTIES,
                       makeInitializeProperties(entity, properties));
  }

  private static String makePropertyDeclarations(Property[] properties) {
    String returnString = "";
    String lineSeparator = "";
    Property prop;
    for (int i = 0; i < properties.length; i++) {
      prop = properties[i];
      returnString += lineSeparator;
      lineSeparator = "\n";

      if (prop.isCompound) {    // Arrays are taken care of
        returnString += "    private " + typeString(prop.mType) + " " + prop.mName +
                        " = new " + typeString(prop.mType) + "();\n";
      }
      else {    // Must deal with arrays here too.
        returnString += "    private " + typeString(prop.mType) + " " + prop.mName + "Property" +
                        " = new " + typeString(prop.mType) + "();\n";
        if (! prop.isArray) {
          returnString += "    private " + javaTypeString(prop.mType) + " " + prop.mName + ";\n";
        }
        else {
          returnString += "    private " + javaTypeString(prop.mType) + "[] " + prop.mName + ";\n";
        }
      }
    }
    return returnString;
  }

  private static String makeInitCompoundValue(Property prop) {
    String returnString = "";
    String propName = prop.mName;

    CompoundType propType = (CompoundType)prop.getType();
    for (int j = 0; j < propType.mFieldOrder.size(); j++) {
      String field = (String)propType.mFieldOrder.get(j);
      Integer fieldType = (Integer)propType.mFields.get(field);
      if (fieldType.intValue() == Property.NUMBER) {
        returnString += "    " + propName + "." + field + " = " + prop.getValueByName(field) + ";\n";
      }
      else if (fieldType.intValue() == Property.BOOLEAN) {
        returnString += "    " + propName + "." + field + " = " + prop.getValueByName(field) + ";\n";
      }
      else if (fieldType.intValue() == Property.STRING) {
        returnString += "    " + propName + "." + field + " = \"" + prop.getValueByName(field) + "\";\n";
      }
    }
    return returnString;
  }

  private static String makeInitCompoundArrayValues(Property prop) {
    String returnString = "";
    String propName = prop.mName;

    CompoundType propType = (CompoundType)prop.getType();
    for (int index = 0; index < prop.arrayLength(); index++) {
      for (int j = 0; j < propType.mFieldOrder.size(); j++) {
        String field = (String)propType.mFieldOrder.get(j);
        Integer fieldType = (Integer)propType.mFields.get(field);
        if (fieldType.intValue() == Property.NUMBER) {
          returnString += "    " + propName + "." + field + "s["+index+"] = " + prop.getValueByName(field, index) + ";\n";
        }
        else if (fieldType.intValue() == Property.BOOLEAN) {
          returnString += "    " + propName + "." + field + "s["+index+"] = " + prop.getValueByName(field, index) + ";\n";
        }
        else if (fieldType.intValue() == Property.STRING) {
          returnString += "    " + propName + "." + field + "s["+index+"] = \"" + prop.getValueByName(field, index) + "\";\n";
        }
      }
    }
    return returnString;
  }

  private static String makeInitAtomicValue(Property prop) {
    String returnString = "";
    String propName = prop.mName + "Property";

    if (prop.mType == Property.NUMBER)
      returnString += "    " + propName + ".value = " + ((NumProp)prop).value + ";\n";
    else if (prop.mType == Property.BOOLEAN)
      returnString += "    " + propName + ".value = " + ((BoolProp)prop).value + ";\n";
    else if (prop.mType == Property.STRING)
      returnString += "    " + propName + ".value = \"" + Compiler.textToJavaString(((StringProp)prop).value) + "\";\n";

    return returnString;
  }

  private static String makeInitAtomicArrayValues(Property prop) {
    String returnString = "";
    String propName = prop.mName + "Property";

    if (prop.mType == Property.NUMBER) {
      for (int index = 0; index < prop.arrayLength(); index++) {
        returnString += "    " + propName + ".values["+index+"] = " + ((NumProp)prop).values[index] + ";\n";
      }
    }
    else if (prop.mType == Property.BOOLEAN) {
      for (int index = 0; index < prop.arrayLength(); index++) {
        returnString += "    " + propName + ".values["+index+"] = " + ((BoolProp)prop).values[index] + ";\n";
      }
    }
    else if (prop.mType == Property.STRING) {
      for (int index = 0; index < prop.arrayLength(); index++) {
        returnString += "    " + propName + ".values["+index+"] = \"" + Compiler.textToJavaString(((StringProp)prop).values[index]) + "\";\n";
      }
    }
    return returnString;
  }

  private static String makeInitializeProperties(Entity entity, Property[] properties) {
    String returnString = "";
    String lineSeparator = "";
    Property prop;
    if (entity.isFunction()) {
      returnString += "    mTriggerType = " + ((Function)entity).mTriggerType + ";\n";
      returnString += "    mTimerInterval = " + ((Function)entity).mTimerInterval + ";\n";
    }
    for (int i = 0; i < properties.length; i++) {
      prop = properties[i];
      returnString += lineSeparator;
      lineSeparator = "\n";
      String propName;

      if (prop.isCompound) {
        propName = prop.mName;
        if (! prop.isArray) {
          returnString += makeInitCompoundValue(prop);
        }
        else {      // Compound array
          returnString += "    " + propName + ".setArrayLength(" + prop.arrayLength() + ");\n";
          returnString += makeInitCompoundArrayValues(prop);
        }
      }
      else {    // Atomic
        propName = prop.mName + "Property";

        if (! prop.isArray) {
          returnString += makeInitAtomicValue(prop);
        }
        else {    // Atomic array
          returnString += "    " + propName + ".setArrayLength(" + prop.arrayLength() + ");\n";
          returnString += makeInitAtomicArrayValues(prop);
        }
      }

      returnString += "    " + propName + ".mName = \"" + prop.mName + "\";\n";
      returnString += "    " + propName + ".isArray = " + prop.isArray + ";\n";
      returnString += "    " + propName + ".setIsOutput(" +
                      prop.isOutput() + ");\n";
      returnString += "    " + propName + ".setIsInput(" +
                      prop.isInput() + ");\n";
      returnString += "    " + propName + ".valueSet();\n";
      returnString += "    addProperty(" + propName + ");\n";
    }

    return returnString;
  }

  private static String makeReadInputs(Property[] properties) {
    String returnString = "";
    String lineSeparator = "";
    for (int i = 0; i < properties.length; i++) {
      returnString += lineSeparator;
      lineSeparator = "\n";
      Property prop = properties[i];
      String propName;

      if (prop.isInput()) {
        if (prop.isCompound) {
          propName = prop.mName;
          returnString += "    " + propName + ".validate();\n";
        }
        else {
          propName = prop.mName + "Property";
          returnString += "    " + propName + ".validate();\n";
          if (! prop.isArray) {
            returnString += "    " + prop.mName + " = " + propName + ".value;\n";
          }
        }
      }
    }
    return returnString;
  }

  private static String makeWriteOutputs(Property[] properties) {
    String returnString = "";
    String lineSeparator = "";
    for (int i = 0; i < properties.length; i++) {
      returnString += lineSeparator;
      lineSeparator = "\n";
      String propName;
      Property prop = properties[i];

      if ((! prop.isInput()) && prop.isOutput()) {
        if (prop.isCompound) {
          propName = prop.mName;
          returnString += "    " + propName + ".valueSet();\n";
        }
        else {
          propName = prop.mName + "Property";
          returnString += "    " + propName + ".validate();\n";
          if (! prop.isArray) {
            returnString += "    " + propName + ".value = " + prop.mName + ";\n";
          }
        }
      }
    }
    return returnString;
  }

  public static String typeString(int type) {
      return g.propertyTypes[type].mClass.getName();
  }

  private static String javaTypeString(int type) {
      if (type == Property.BOOLEAN)
          return "boolean";
      if (type == Property.NUMBER)
          return "double";
      if (type == Property.STRING)
          return "String";
      return "";
  }

  public static String textToJavaString(String text) {
    StringBuffer returnBuffer = new StringBuffer();
    char c;
    for (int i = 0; i < text.length(); i++) {
      c = text.charAt(i);
      if (c == '"')
        returnBuffer.append("\\\"");
      else if (c == '\\')
        returnBuffer.append("\\\\");
      else if (c == '\n')
        returnBuffer.append("\\n");
      else
        returnBuffer.append(c);
    }
    return returnBuffer.toString();
  }
}