package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric design tool.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

public class ConnectionNode extends GraphNode {

    private PropertyBound mParentNode;
    private PropertyBound mChildNode;
    private boolean mInDelete = false;

//    private boolean mIsInteriorConnection = false;  // Is this a connection to an Enclosure exterior

    private static BasicStroke mConnectionStroke = new BasicStroke(2.0f,
                                                  BasicStroke.CAP_BUTT,
                                                  BasicStroke.JOIN_MITER);
    private static Color mColor = new Color(0, 0, 255, 150);
    private Line2D mPaintLine = new Line2D.Double();

    public ConnectionNode() {
        super();

        // Added to make it easy to add intermediate floating props, but doean't work
        // Connection nodes not normally selectable
        /*
        addMouseListener(new MouseAdapter() {
          public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
              mEnclosureNode.makeNewProperty(new Point2D.Double(e.getX(), e.getY()), mParentNode.mProperty.mType);  //addFreeFloatingProperty(Property property, Point2D releasePoint)
            }
          }
        });*/

    }

    public ConnectionNode(EnclosureNode enclosureNode, PropertyBound parent, PropertyBound child) {
        super();
        mParentNode = parent;
        mChildNode = child;
        mEnclosureNode = enclosureNode;
        setOpaque(false);
    }

    public void connectionsMoved() {
        int top;
        int left;
        int right;
        int bottom;

        if (mParentNode.isEnclosureIn() && !((EnclosureNode)mParentNode.getEntityNode()).isRevealed())
            return;
        if (mChildNode.isEnclosureOut() && !((EnclosureNode)mChildNode.getEntityNode()).isRevealed())
            return;

        Point2D center1 = mParentNode.getCenter(mEnclosureNode.getInterior());
        Point2D center2 = mChildNode.getCenter(mEnclosureNode.getInterior());

        top = (int)java.lang.Math.min(center1.getY(), center2.getY());
        bottom = (int)java.lang.Math.max(center1.getY(), center2.getY());
        left = (int)java.lang.Math.min(center1.getX(), center2.getX());
        right = (int)java.lang.Math.max(center1.getX(), center2.getX());

        mPaintLine.setLine(center1.getX() + 2 - left, center1.getY() + 2 - top,
                           center2.getX() + 2 - left, center2.getY() + 2 - top);
        setLocation(left - 2, top - 2);
        setSize(right - left + 4, bottom - top + 4);
    }

    // Possibly problematic with internal enclosure connections?
    public void connectionsChangedLayer() {
        int parentLayer = mParentNode.getOwnerLayer(mEnclosureNode);
        int childLayer = mChildNode.getOwnerLayer(mEnclosureNode);

        if (parentLayer < 1)
            parentLayer = Integer.MAX_VALUE;
        if (childLayer < 1)
            childLayer = Integer.MAX_VALUE;

        int newLayer = java.lang.Math.min(parentLayer, childLayer);
        if (newLayer == Integer.MAX_VALUE)
            return;
        mEnclosureNode.setLayer(this, newLayer - 1);
    }

    public boolean contains(int x, int y) {
        if (!g.sceneGraphPanel.connectionsSelectable())
            return false;
        return mPaintLine.intersects(x - 2, y - 2, 4, 4);
    }

    public boolean contains(java.awt.Point p) {
        if (!g.sceneGraphPanel.connectionsSelectable())
            return false;
        return contains((int)p.getX(), (int)p.getY());
    }

    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setStroke(mConnectionStroke);
        g2.setPaint(mColor);
        g2.draw(mPaintLine);
    }

    public PropertyBound getChildNode() {
        return mChildNode;
    }

    public PropertyBound getParentNode() {
        return mParentNode;
    }

    public void delete() {
        if (mInDelete)
            return;
        mInDelete = true;
        mEnclosureNode.removeConnectionNode(this);
        mParentNode.getProperty().removeChild(mChildNode.getProperty());
        mChildNode.getProperty().setParent(null);
        mEnclosureNode.repaint();
    }
}