package roebling;

import javax.swing.JPanel;

/**
 * Title:        ParaMour
 * Description:  Parametric design tool.
 * Copyright:    Copyright (c) 2001
 * Company:      NewBricks
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.net.*;
import java.awt.geom.*;
import java.io.*;


public class EnclosureNode extends EntityNode {

    private Enclosure mEntityAsEnclosure;

    private boolean mIsRevealed = false;
    private boolean mIsMaximized = false;

    private static final int ENCLOSURE_TITLE_BAR_HEIGHT = 16;
    private static final int OFFSET_X_START = 20;
    private static final int ADD_OFFSET_X = 20;
    private static final int ADD_OFFSET_Y = 20;
    private static final int TRANS_ADD_OFFSET_X = 100;
    private static final int TRANS_ADD_OFFSET_Y = 80;

    private int mNextAddX = OFFSET_X_START;
    private int mNextAddY = ADD_OFFSET_Y;
    private int mNextTransAddX = ADD_OFFSET_X - 10;
    private int mNextTransAddY = ADD_OFFSET_Y;

    private static final int MIN_LAYER = 401;
    private static final int MAX_LAYER = 900000;
    private static final int BUS_LAYER = MAX_LAYER + 1;
    private static final int CONNECTION_BAND_LAYER = MAX_LAYER + 2;
    private int mLowestLayer = MIN_LAYER;
    private int mTopLayer = MIN_LAYER;

    private static final double SCALE_MOVEMENT_DIVISOR = 70.0;
    private static final int INTERIOR_X_OFFSET = 8;
    private static final int INTERIOR_TOP_OFFSET = 22;
    private static final int INTERIOR_BOTTOM_OFFSET = 17;
    protected static final int ENCLOSURE_MINIMUM_WIDTH = 135;

    private static final Color ENCLOSURE_NODE_COLOR = new Color(200, 200, 200, 150);
    private static final Color ENCLOSURE_NODE_INSTANCE_COLOR = new Color(150, 150, 150, 150);
    private static final Color ENCLOSURE_NODE_COLOR_OPAQUE = new Color(200, 200, 200, 255);

    private EnclosureInteriorPanel mInterior;
    private JToggleButton mAbstractRevealButton;
    private JToggleButton mMaximizeButton;
    private Rectangle mUnMaximizedBounds;
    private boolean mWasRevealedBeforeMaximize;
    private JToggleButton mScaleButton;
    private JToggleButton mPanButton;
    private JButton mStoreButton;
    private boolean mWasRevealedBeforePropEdit;

    public EntityNode[] mDirectory = new EntityNode[0];

    private boolean mIsScaling = false;
    private Point2D mScaleCenter = new Point2D.Double();
    private double mLastScaleX;
    private double mLastPanX;
    private double mLastPanY;
    private boolean mIsPanning;

    private ConnectionBand mConnectionBand = new ConnectionBand();

    public EnclosureNode() {
        super();
        addButtons();
        setIsRevealed(true);
    }

    protected int getTitleBarHeight() {
        return ENCLOSURE_TITLE_BAR_HEIGHT;
    }

  protected PropertyNode addFreeFloatingProperty(Property property, Point2D releasePoint) {
    PropertyNode newPropNode = null;
    JLabelNoAntialias newLabel;

    if (mPropertyNodes == null)
      mPropertyNodes = new PropertyNode[0];

    newLabel = new JLabelNoAntialias(property.mName);
    newPropNode = new PropertyNode(property, this, newLabel);
    g.sceneGraphPanel.setInputPropertyNode(property, newPropNode);
    g.sceneGraphPanel.setOutputPropertyNode(property, newPropNode);
    mPropertyNodes = (PropertyNode[])Util.append(mPropertyNodes, Types.PROPERTY_NODE, newPropNode);
    newPropNode.mFreeFloatingLocation = releasePoint;

    add(newLabel);
    add(newPropNode);
    newPropNode.mEnclosureNode = this;
    newPropNode.setLocation((int)releasePoint.getX(),(int)releasePoint.getY());

    newPropNode.setIsInput(true);
    newPropNode.setIsOutput(true);
    newPropNode.setIsEnclosureIn(true);

    newLabel.setFont(mPropertyFont);
    newLabel.setForeground(Color.black);

    return newPropNode;
  }

  public void transformationListChanged() {
    for (int i = 0; i < mDirectory.length; i++) {
      if (mDirectory[i].isTransformationNode()) {
        ((TransformationNode)mDirectory[i]).transformationListChanged();
        ((TransformationNode)mDirectory[i]).makeConnector();
      }
    }
  }

    public void addGraphNode(EntityNode entityNode) {
        entityNode.setEnclosureNode(this);
        mDirectory = (EntityNode[])Util.append(mDirectory, Types.ENTITY_NODE, entityNode);
        if (isRevealed()) {
            mInterior.add(entityNode, new Integer(mTopLayer));
            mTopLayer += 2;

            if (entityNode.isTransformationNode()) {
              if (mNextTransAddY >= mInterior.getHeight()) {
                mNextTransAddX += TRANS_ADD_OFFSET_X;
                mNextTransAddY = ADD_OFFSET_Y;
              }

              entityNode.setLocation(mNextTransAddX, mNextTransAddY);
              //mNextTransAddX += TRANS_ADD_OFFSET_X;
              mNextTransAddY += TRANS_ADD_OFFSET_Y;
            }
            else {
              if (mNextAddX >= mInterior.getWidth() || mNextAddY >= mInterior.getHeight()) {
                mNextAddX = OFFSET_X_START;
                mNextAddY = ADD_OFFSET_Y;
              }

              entityNode.setLocation(mNextAddX, mNextAddY);
              mNextAddX += ADD_OFFSET_X;
              mNextAddY += ADD_OFFSET_Y;
            }

            if (mTopLayer > MAX_LAYER)
                compressLayers();
        }
    }

    protected void addPropertyNodes() {
      Property curProp;
      for (int i = 0; i < mEntity.mProperties.length; i++) {
        curProp = mEntity.mProperties[i];
        if (curProp.isFreeFloating()) {
          addFreeFloatingProperty(curProp, curProp.mFreeFloatingLocationHint);
        }
        else {
          addProperty(curProp, i);
        }
      }
    }

    public void bringToFront(EntityNode entityNode) {
        mInterior.setLayer(entityNode, mTopLayer);
        mTopLayer += 2;
        if (mTopLayer > MAX_LAYER)
            compressLayers();
    }

    public void compressLayers() {
        SortedSet oldLayers = new TreeSet();
        Hashtable oldToNew = new Hashtable();
        Component[] components = mInterior.getComponents();
        for (int i = 0; i < components.length; i++) {
            if (components[i] != mConnectionBand)
                oldLayers.add(new Integer(mInterior.getLayer(components[i])));
        }
        int index = MIN_LAYER;
        for (Iterator i = oldLayers.iterator(); i.hasNext();) {
            oldToNew.put(i.next(), new Integer(index));
            mTopLayer = index;
            index++;
        }
        for (int i = 0; i < components.length; i++) {
            if (components[i] != mConnectionBand)
                mInterior.setLayer(components[i], ((Integer)oldToNew.get(new Integer(mInterior.getLayer(components[i])))).intValue());
        }
        if (mTopLayer > MAX_LAYER) {
            System.out.println("Too many objects in this enclosure!");
        }
    }

    protected void drag(MouseEvent e) {
        if (isMaximized())
            return;
        super.drag(e);
    }

    public boolean isEnclosureNode() {
        return true;
    }

    public boolean isRevealed() {
        return mIsRevealed;
    }

    public int getMinimumWidth() {
        return ENCLOSURE_MINIMUM_WIDTH;
    }

    public void editPropNode(PropertyNode propNode) {
      if (mEditingProperty) {
        if(propNode == mPropertyEditPanel.mPropertyNode) {
          setIsRevealed(mWasRevealedBeforePropEdit);
        }
      }
      else {
        mWasRevealedBeforePropEdit = mIsRevealed;
        setIsRevealed(false);
      }
      super.editPropNode(propNode);
    }

    private void makeEntityNodes() {
      if (mEntityAsEnclosure == null)
        return;
      Entity directory[] = mEntityAsEnclosure.mDirectory;
      EntityNode node;
      Vector newNodes = new Vector();

      for (int i = 0; i < directory.length; i++) {
        node = g.sceneGraphPanel.getEntityNode(directory[i]);
        if (node == null) {
          node = g.sceneGraphPanel.addNewEntityNode(directory[i], mEntityAsEnclosure);
          newNodes.add(node);
          if (node.isEnclosureNode()) {
            if (((EnclosureNode)node).isRevealed()) {
              ((EnclosureNode)node).makeEntityNodes();
            }
          }
        }
      }

      for (Iterator newNode = newNodes.iterator(); newNode.hasNext();) {
        node = (EntityNode)newNode.next();
        node.addPropertyConnections(this);
      }
      if (newNodes.size() > 0) {    // Made new nodes. Connect to them if necessary
        Property prop;
        PropertyNode parentNode;
        PropertyNode childNode;
        for (int i = 0; i < mEntity.mProperties.length; i++) {
          prop = mEntity.mProperties[i];
          if (prop.isEnclosureOut() && prop.mParent != null) {
            parentNode = g.sceneGraphPanel.getOutputNode(prop.mParent);
            childNode = g.sceneGraphPanel.getInputNode(prop);
            if (parentNode != null && childNode != null)
              newConnection(parentNode, childNode);
          }
        }
      }
    }

    public void showFreeFloatingProps(boolean show) {
      for (int i = 0; i < mPropertyNodes.length; i++) {
        if (mPropertyNodes[i].mProperty.isFreeFloating()) {
          mPropertyNodes[i].setVisible(show);
          mPropertyNodes[i].mPropertyLabel.setVisible(show);
        }
      }
    }

    public void setIsRevealed(boolean revealed) {
        if (mIsRevealed == revealed)
            return;
        mIsRevealed = revealed;
        showFreeFloatingProps(mIsRevealed);
        if (mIsRevealed) {
            if (mEntity != null) {
              mEntity.mIsInstance = false;
              mEntity.mRRL = null;
              setIsInstance(false);
            }

            if (mInterior == null) {
                mInterior = new EnclosureInteriorPanel();
            }
            add(mInterior);

            mInterior.setBounds(INTERIOR_X_OFFSET, getInteriorTopOffset(),
                                getWidth() - 2 * INTERIOR_X_OFFSET, getHeight() - (getInteriorTopOffset() + INTERIOR_BOTTOM_OFFSET));

            makeEntityNodes();

            moveAllConnections();
            mScaleButton.setVisible(true);
            mPanButton.setVisible(true);
            mAbstractRevealButton.setSelected(false);
            mAbstractRevealButton.setText("Abstract");

        }
        else {
            remove(mInterior);
            mAbstractRevealButton.setSelected(true);
            mAbstractRevealButton.setText("Reveal");
            mScaleButton.setVisible(false);
            mPanButton.setVisible(false);
        }
        repaint();
    }

    private void moveAllConnections() {
        for (int i = 0; i < mPropertyNodes.length; i++) {
                mPropertyNodes[i].moveConnections();
        }
    }

    public int getLayer(GraphNode node) {
        return mInterior.getLayer(node);
    }

    public void setLayer(GraphNode node, int layer) {
        mInterior.setLayer(node, layer);
    }

    public EnclosureInteriorPanel getInterior() {
        return mInterior;
    }

    public boolean isMaximized() {
        return mIsMaximized;
    }

    public void setEntity(Entity entity) {
        super.setEntity(entity);
        mEntityAsEnclosure = (Enclosure)entity;
        if (mEntityAsEnclosure.isScene())
            mMaximizeButton.setVisible(false);
    }

    public void setEnclosureNode(EnclosureNode enclosureNode) {
        super.setEnclosureNode(enclosureNode);
    }

    // This is to be called by SceneGraphPanel, not the other way
    public void setIsMaximized(boolean maximized) {
        mIsMaximized = maximized;
        if (mIsMaximized) {
            mUnMaximizedBounds = getBounds();
        }
        else {
            if (mIsRevealed) {
                if (! mWasRevealedBeforeMaximize) {
                    setIsRevealed(false);
                }
                else
                    mInterior.setSize(getWidth() - 2 * INTERIOR_X_OFFSET, getHeight() - (getInteriorTopOffset() + INTERIOR_BOTTOM_OFFSET));
            }
            EnclosureNode encNode = this;
            while (encNode != null) {
                encNode.moveAllConnections();
                encNode = encNode.getEnclosureNode();
            }
        }
    }

    public Rectangle getUnMaximizedBounds() {
        return mUnMaximizedBounds;
    }

    public Color getFillColor() {
        if (mIsRevealed)
            return ENCLOSURE_NODE_COLOR_OPAQUE;

        if (! mEntity.mIsInstance)
          return ENCLOSURE_NODE_COLOR;
        return ENCLOSURE_NODE_INSTANCE_COLOR;
    }

    // This enclosure should be the enclosure of at least one of the two properties
    public void newConnection(PropertyBound parent, PropertyBound child) {
        ConnectionNode connection = new ConnectionNode(this, parent, child);
        mInterior.add(connection);
        parent.addConnectionNode(connection);
        child.addConnectionNode(connection);
        connection.connectionsChangedLayer();
        connection.connectionsMoved();
    }

    public void removeConnectionNode(ConnectionNode connectionNode) {
        connectionNode.getChildNode().removeConnectionNode(connectionNode);
        connectionNode.getParentNode().removeConnectionNode(connectionNode);
        mInterior.remove(connectionNode);
    }

    public void resized() {
        super.resized();
        if (isRevealed()) {
            mInterior.reshape(INTERIOR_X_OFFSET, getInteriorTopOffset(), getWidth() - 2 * INTERIOR_X_OFFSET, getHeight() - (getInteriorTopOffset() + INTERIOR_BOTTOM_OFFSET));
        }
        mName.setSize(getWidth() - (2 * (X_INSET + BORDER_THICKNESS) + TITLE_X_OFFSET) - 18, 16);
        mNameEdit.setSize(getWidth() - (2 * (X_INSET + BORDER_THICKNESS) + TITLE_X_OFFSET) - 18, 16);

        mPanButton.setBounds(20, getHeight() - 17, 16, 12);
        mScaleButton.setBounds(36, getHeight() - 17, 16, 12);
        mAbstractRevealButton.setBounds(52, getHeight() - 17, 43, 12);
        mStoreButton.setBounds(96, getHeight() - 17, 34, 12);
        mMaximizeButton.setBounds(getWidth() - 24, Y_INSET + BORDER_THICKNESS, 17, 17);
    }

    public int getInteriorTopOffset() {
        //return INTERIOR_TOP_OFFSET + mBusHeights;
        return INTERIOR_TOP_OFFSET;
    }

    public void rescale(Point2D scaleCenter, double scaleFactor) {
        EntityNode currentNode;
        Point2D topLeft = new Point2D.Double();
        Point2D bottomRight = new Point2D.Double();
        Point2D newTopLeft = new Point2D.Double();
        Point2D newBottomRight = new Point2D.Double();
        int newWidth;
        int newHeight;

        for (int i = 0; i < mDirectory.length; i++) {
            currentNode = mDirectory[i];
            topLeft.setLocation(currentNode.getLocation());
            bottomRight.setLocation(topLeft.getX() + currentNode.getWidth(),
                                    topLeft.getY() + currentNode.getHeight());
            Util.scalePoint(scaleCenter, topLeft, scaleFactor, newTopLeft);
            Util.scalePoint(scaleCenter, bottomRight, scaleFactor, newBottomRight);
            newWidth = (int)(newBottomRight.getX() - newTopLeft.getX());
            newWidth = Math.max(newWidth, currentNode.getMinimumWidth());
            newHeight = (int)(newBottomRight.getY() - newTopLeft.getY());
            newHeight = Math.max(newWidth, currentNode.getMinimumHeight());
            currentNode.reshape((int)newTopLeft.getX(), (int)newTopLeft.getY(),
                                newWidth,
                                newHeight);
            currentNode.resized();
        }
    }

    public void pan(int dx, int dy) {
        Entity[] directory = mEntityAsEnclosure.mDirectory;
        EntityNode currentNode;
        Point2D topLeft = new Point2D.Double();

        for (int i = 0; i < mDirectory.length; i++) {
            currentNode = mDirectory[i];
            topLeft.setLocation(currentNode.getLocation());

            currentNode.setLocation((int)topLeft.getX() + dx, (int)topLeft.getY() + dy);
        }
    }

    private void addButtons() {
        String jarPath = "";
        String tempStr;
        StringTokenizer pathTokens;
        try {
           pathTokens = new StringTokenizer(System.getProperty("java.class.path"), ";");
            while (pathTokens.hasMoreTokens()) {
                tempStr = pathTokens.nextToken();
                if (tempStr.indexOf("paramour.jar") != -1) jarPath = tempStr;
            }
        }
        catch (Exception e) {
        }


        if (!jarPath.equals("")) {
            jarPath = "jar:file:" + jarPath + "!/";
        }
        else
            jarPath = "file:";

        try {

            mAbstractRevealButton = new JToggleButtonNoAntialias();
            if (mButtonFont != null)
                mAbstractRevealButton.setFont(mButtonFont);
            mAbstractRevealButton.setMargin(new Insets(0,0,0,0));
            mAbstractRevealButton.setText("Abstract");
            mAbstractRevealButton.addActionListener(new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    setIsRevealed(!mAbstractRevealButton.isSelected());
                    g.sceneGraphPanel.setSelected(mSelf);
                }
            });
            mAbstractRevealButton.setSelected(mIsRevealed);
            add(mAbstractRevealButton);

            mStoreButton = new JButtonNoAntialias();
            if (mButtonFont != null)
                mStoreButton.setFont(mButtonFont);
            mStoreButton.setMargin(new Insets(0,0,0,0));
            mStoreButton.setText("Store");
            mStoreButton.addActionListener(new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    store();
                }
            });
            add(mStoreButton);

            mMaximizeButton = new JToggleButton(new ImageIcon(new URL(jarPath + "images/unmaximize.gif")));
            mMaximizeButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/maximize.gif")));
            mMaximizeButton.addActionListener(new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    if (mMaximizeButton.isSelected()) {
                        g.sceneGraphPanel.unMaximizeTopNode();
                    }
                    else {
                        mWasRevealedBeforeMaximize = mIsRevealed;
                        if (!isRevealed()) {
                            setIsRevealed(true);
                            mAbstractRevealButton.setSelected(false);
                        }
                        g.sceneGraphPanel.maximizeEnclosureNode((EnclosureNode)mSelf);
                    }
                }
            });
            mMaximizeButton.setSelected(!mIsMaximized);
            add(mMaximizeButton);


            mScaleButton = new JToggleButton(new ImageIcon(new URL(jarPath + "images/scale.gif")));
            mScaleButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/scaleHilite.gif")));
            mScaleButton.addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    mScaleButton.setSelected(true);
                    mIsScaling = true;
                    mScaleCenter.setLocation(mInterior.getWidth() / 2.0, mInterior.getHeight() / 2.0);
                    mLastScaleX = e.getX();
                }
                public void mouseReleased(MouseEvent e) {
                    mScaleButton.setSelected(false);
                    mIsScaling = false;
                }
            });
            mScaleButton.addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseDragged(MouseEvent e) {
                    if (mIsScaling) {
                        double factor = 1.0 + ((e.getX() - mLastScaleX) / SCALE_MOVEMENT_DIVISOR);
                        rescale(mScaleCenter, factor);
                        mLastScaleX = e.getX();
                    }
                }
            });
            mScaleButton.setSelected(false);
            add(mScaleButton);

            mPanButton = new JToggleButton(new ImageIcon(new URL(jarPath + "images/pan.gif")));
            mPanButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/panHilite.gif")));
            mPanButton.addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent e) {
                    mPanButton.setSelected(true);
                    mIsPanning = true;
                    mLastPanX = e.getX();
                    mLastPanY = e.getY();
                }
                public void mouseReleased(MouseEvent e) {
                    mPanButton.setSelected(false);
                    mIsPanning = false;
                }
            });
            mPanButton.addMouseMotionListener(new MouseMotionAdapter() {
                public void mouseDragged(MouseEvent e) {
                    if (mIsPanning) {
                        int dx = (int)(e.getX() - mLastPanX);
                        int dy = (int)(e.getY() - mLastPanY);
                        pan(dx, dy);
                        mLastPanX = e.getX();
                        mLastPanY = e.getY();
                    }
                }
            });
            mPanButton.setSelected(false);
            add(mPanButton);
        }
        catch (MalformedURLException e) {
            System.out.println("Cannot locate button images in file \"Roebling.jar\"");
        }
    }

  public void moveProperties() {
    super.moveProperties();
    if (mIsRevealed)
      try {
        moveToBack(mInterior);
      }
      catch (Exception e) {
      }
  }

  public boolean store() {
    File chosenFile = storeAs();
    if (chosenFile == null)
      return false;

    setIsRevealed(false);
    mEntity.mIsInstance = false;
    mEntity.mRRL = null;
    setIsInstance(false);

    String fileName = chosenFile.getParent() + g.FS + FileUtil.stripPostfix(chosenFile.getName()) + ".enc";
    File resourceFile = new File(fileName);

    mEntityAsEnclosure.store();

/*
    File graphFile = new File(g.tempDirectory.getPath() + g.FS + "graph.xml");
    if (graphFile.exists()) {
      graphFile.delete();
      try {
        graphFile.createNewFile();
      }
      catch (IOException e) {
        return false;
      }
    }

    XMLSerializer xmlOut;
    try {
      xmlOut = new XMLSerializer(new BufferedWriter(new FileWriter(graphFile.getPath())),
                                 new OutputFormat());
      xmlOut.startDocument();
      serializeNode(xmlOut);
      xmlOut.endDocument();
    }
    catch (Exception e) {
      return false;
    }
*/
    try {
      File classesDir = new File(g.tempDirectory.getPath() + g.FS + "classes" + g.FS + "roebling");
      String[] classes = classesDir.list();
      String[] files = new String[classes.length + 1];
//      files[0] = graphFile.getPath();
      files[0] = g.tempDirectory.getPath() + g.FS + "enclosure.xml";
      for (int i = 0; i < classes.length; i++) {
        files[i + 1] = g.tempDirectory.getPath() + g.FS + "classes" + g.FS + "roebling" +
                         g.FS + classes[i];
      }

      FileUtil.zipUp(files, g.tempDirectory.getPath(), fileName);
    }
    catch (Exception e) {
      return false;
    }

    g.resourcesFrame.reloadResourceTree();

    mEntity.mIsInstance = true;
    mEntity.mRRL = g.resourcesFrame.getRRL(resourceFile);
    setIsRevealed(false);
    setIsInstance(true);
    repaint();

    return true;
  }

    public void removeGraphNode(EntityNode entityNode) {
        if (mInterior != null) {
            entityNode.clearConnections();
            mInterior.remove(entityNode);
        }
        mDirectory = (EntityNode[])Util.remove(mDirectory, Types.ENTITY_NODE, entityNode);
        repaint();
    }

    public void startConnectionBand(int x, int y) {
        mConnectionBand.x1 = x;
        mConnectionBand.y1 = y;
        mConnectionBand.x2 = x;
        mConnectionBand.y2 = y;
        mInterior.add(mConnectionBand, new Integer(CONNECTION_BAND_LAYER));
        mConnectionBand.setVisible(true);
        mConnectionBand.setBounds(0, 0, mInterior.getWidth(), mInterior.getHeight());
    }

    public void moveConnectionBand(int x, int y) {
        mConnectionBand.x2 = x;
        mConnectionBand.y2 = y;
        mConnectionBand.repaint();
    }

    public void stopConnectionBand() {
        mConnectionBand.setVisible(false);
        mInterior.remove(mConnectionBand);
    }

    private class ConnectionBand extends JPanel {
        public int x1 = 0;
        public int y1 = 0;
        public int x2 = 0;
        public int y2 = 0;
        private BasicStroke mConnectionStroke = new BasicStroke(2.0f,
                                                      BasicStroke.CAP_BUTT,
                                                      BasicStroke.JOIN_MITER);
        private Color mColor = new Color(0, 0, 255, 150);

        public ConnectionBand() {
            setOpaque(false);
        }

        public boolean contains(int x, int y) {
            return false;
        }

        public boolean contains(java.awt.Point p) {
            return false;
        }

        public void paintComponent(Graphics g) {
            Graphics2D g2 = (Graphics2D)g;
            g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2.setStroke(mConnectionStroke);
            g2.setPaint(mColor);
            g2.drawLine(x1, y1, x2, y2);
        }
    }

    protected boolean canDelete() {
        if (mEntity == g.scene)
            return false;
        return (JOptionPane.showInternalConfirmDialog(this,
                                            "Delete entire enclosure \"" + mEntity.mName + "\"",
                                            "Confirm delete",
                                            JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
    }

  protected boolean serializeNode(XMLWriter xmlOut) {
    EnclosureNode encNode;
    Entity[]      directory;
    EntityNode    nextNode;

    try {
      if (!mIsMaximized)
        return super.serializeNode(xmlOut);

      xmlOut.startElement("entityNode");
      Util.serializeElement(xmlOut, "left", 0);
      Util.serializeElement(xmlOut, "top", 0);
      Util.serializeElement(xmlOut, "width", getMinimumWidth());
      Util.serializeElement(xmlOut, "height", getMinimumWidth());
      Util.serializeElement(xmlOut, "revealed", isRevealed());
      xmlOut.endElement("entityNode");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }
}