package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.io.*;
import java.util.*;
import java.util.zip.*;

public class FileUtil {

  public static final int EOF = -1;

  public FileUtil() {
  }

  /**
   * Clean recursivly
   * @param file the directory to clean
   */
  public static boolean clean(File file) {
    if (file.isDirectory()) {
      String filen[] = file.list();
      for (int i = 0 ; i < filen.length ; i++) {
        File subfile = new File(file, filen[i]);
        if ((subfile.isDirectory()) && (! clean(subfile))) {
          return false;
        } else if (! subfile.delete()) {
          return false;
        }
      }
    }
    return true;
  }

  public static boolean readToStringBuffer(File directory, String fileName, StringBuffer contents) {
    File sourceFile;
    try {
      sourceFile = new File(directory.getPath() + g.FS + fileName);
      if (sourceFile == null) {
        System.err.println("Reading file \"" +  fileName + "\"failed.");
        return false;
      }
    }
    catch (Exception e) {
      return false;
    }
    return readToStringBuffer(sourceFile, contents);
  }

  public static boolean readToStringBuffer(File sourceFile, StringBuffer contents) {
    try {
      if (!sourceFile.exists() || !sourceFile.isFile() || !sourceFile.canRead()) {
        System.err.println("Cannot read from file \"" +  sourceFile.getName() + "\".");
        return false;
      }
      int filesize = (int) sourceFile.length();
      char[] data = new char[filesize];
      contents.setLength(0);

      BufferedReader fileReader = new BufferedReader(new FileReader(sourceFile));
      fileReader.read(data);
      fileReader.close();
      contents.insert(0, data);
    }
    catch (Exception e) {
      System.err.println("Reading file \"" +  sourceFile.getName() + "\"failed.");
      return false;
    }
    return true;
  }

  public static boolean deleteFile(File directory, String fileName) {
    try {
      File destinationFile = new File(directory.getPath() + g.FS + fileName);
      destinationFile.delete();
    }
    catch (Exception e) {
      System.err.println("Error deleting file.");
      return false;
    }
    return true;
  }

  public static boolean writeStringToFile(File directory, String fileName, String contents) {
    File destinationFile = null;
    try {
      destinationFile = new File(directory.getPath() + g.FS + fileName);

      if (destinationFile == null) {
        System.err.println("Output file is null.");
        return false;
      }
    }
    catch (Exception e) {
      System.err.println("Error opening output file.");
      return false;
    }
    return writeStringToFile(destinationFile, contents);
  }

  public static boolean writeStringToFile(File destinationFile, String contents) {
    if (destinationFile.exists()) {
      if (!destinationFile.isFile()) {
        System.err.println("Output file is not a file.");
        return false;
      }
      try {
        destinationFile.delete();
      }
      catch(Exception e) {
        System.err.println("Cannot overwrite output file.");
        return false;
      }
    }

    try {
      destinationFile.createNewFile();
      BufferedWriter fileWriter = new BufferedWriter(new FileWriter(destinationFile));
      fileWriter.write(contents);
      fileWriter.close();
    }
    catch(Exception e) {
      System.err.println("Error writing to file.");
      return false;
    }
    return true;
  }

  public static String stripPostfix(String fileName) {
    String postfix = getPostfix(fileName);
    if (postfix.equals(fileName)) {
      return fileName;
    }
    else {
      return fileName.substring(0, fileName.length() - postfix.length() - 1);
    }

    /*StringTokenizer tokenizer = new StringTokenizer(fileName, ".");
    return tokenizer.nextToken();*/
  }

  public static String getPostfix(String fileName) {
    StringTokenizer tokenizer = new StringTokenizer(fileName, ".");
    String postfix = "";
    while (tokenizer.hasMoreTokens()) {
      postfix = tokenizer.nextToken();
    }
    return postfix;
  }

  public static void zipUp(String fileNames[], String context, String outFile) throws IOException {
   byte b[] = new byte[512];
   String entry;
   FileOutputStream fileOut = new FileOutputStream(outFile);
   ZipOutputStream zout = new ZipOutputStream(fileOut);
   for(int i = 0; i < fileNames.length; i ++) {
     InputStream in = new FileInputStream(fileNames[i]);
     entry = fileNames[i];
     if (entry.startsWith(context)) {
          entry = entry.substring(context.length() + 1);
     }
     ZipEntry e = new ZipEntry(entry.replace(File.separatorChar,'/'));
     zout.putNextEntry(e);
     int len=0;
     while((len=in.read(b)) != -1) {
       zout.write(b,0,len);
      }
     in.close();
     zout.closeEntry();
        //print(e);
      }
    zout.close();
    fileOut.close();
  }

  public static File unzip( String file, String context ) {
    Enumeration enum;
    File lastFile = null;

    try {
      ZipFile zipFile = new ZipFile( file );
      enum = zipFile.entries();

      while( enum.hasMoreElements() ) {

        ZipEntry target
             = (ZipEntry)enum.nextElement();
//        System.out.print(
//                  target.getName() + " ." );
        lastFile = getEntry( zipFile, target, context );
//        System.out.println( ". unpacked" );
      }
    }
    catch( FileNotFoundException e ){
      System.err.println( "zipfile not found" );
    }
    catch( ZipException e ){
      System.err.println( "zip error..." );
    }
    catch( IOException e ){
      System.err.println( "IO error..." );
    }
    return lastFile;
 }

  private static File getEntry( ZipFile zipFile, ZipEntry target, String context )
                       throws ZipException,IOException {
    try {
      File file = new File( context + g.FS + target.getName() );

      if( target.isDirectory() ) {
        file.mkdirs();
      }
      else {
        BufferedInputStream bis = new BufferedInputStream(zipFile.getInputStream( target ) );

       String parentName;
       if( ( parentName = file.getParent() ) != null ) {
         File dir = new File( parentName );
         dir.mkdirs();
        }
        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream( file ) );
        int c;
        while( ( c = bis.read() ) != EOF ) {
          bos.write( (byte)c );
        }
        bos.close();
      }
      return file;
    }
    catch( ZipException e ){
      throw e;
    }
    catch( IOException e ){
      throw e;
    }
  }

  public static void copyDirectory(File sourceDir, File destDir) {
    destDir.mkdirs();
    if (!sourceDir.isDirectory())
      return;
    String[] fileStrings = sourceDir.list();
    File sourceFile, destFile;
    for (int i = 0; i < fileStrings.length; i++) {
      sourceFile = new File(sourceDir.getPath() + g.FS + fileStrings[i]);
      destFile = new File(destDir.getPath() + g.FS + fileStrings[i]);
      if (sourceFile.isDirectory()) {
        copyDirectory(sourceFile, destFile);
      }
      else {
        try {
          copy(sourceFile, destFile);
        }
        catch (Exception e) {
        }
      }
    }
  }

  public static void copy(File source, File dest)
    throws IOException
  {
    BufferedInputStream in = new BufferedInputStream( new FileInputStream(source) );
    BufferedOutputStream out = new BufferedOutputStream( new FileOutputStream(dest) );
    byte buffer[] = new byte[1024];
    int read = -1;
    while ((read = in.read(buffer, 0, 1024)) != -1)
      out.write(buffer, 0, read);
    out.flush();
    out.close();
    in.close();
  }
}