package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.net.*;
import java.io.*;
import java.util.*;

public class Function extends Entity {

  public static final int STANDARD_TRIGGER = 0;
  public static final int TIMER_TRIGGER = 1;
  public static final int SCENEGRAPH_TRIGGER = 2;
  public static int DEFAULT_TIMER_INTERVAL = 10;

  public String mBaseClassName;
  private Instanceable[] mInstances;
  public int mTriggerType = STANDARD_TRIGGER;
  public int mTimerInterval = DEFAULT_TIMER_INTERVAL;
  public boolean mRunOnce;

  public Function() {
  }

  public Function(String name, Enclosure enclosure) {
    super(name, enclosure);
  }

  public boolean isFunction() {
    return true;
  }

  public void delete() {
    g.timerQueue.removeFromTimerQueue(this);
    super.delete();
  }

  public String getCode() {  // The code of the run function
    return "";
  }

  public String getClassLevelCode() {     // Returns the code outside the run function
    return "";
  }



  protected boolean serializeEntity(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("entity");
      String newName = storeClassFile();

      Util.serializeElement(xmlOut, "type", "function");
      Util.serializeElement(xmlOut, "playerClass", newName);
//      Util.serializeElement(xmlOut, "triggerType", mTriggerType);
//      Util.serializeElement(xmlOut, "timerInterval", mTimerInterval);
      if (mIsInstance) {
        Util.serializeElement(xmlOut, "instanceOf", mRRL.relativePath());
      }
      else {
        Util.serializeElement(xmlOut, "class", newName);
      }

      if (! super.serializeEntity(xmlOut))
          return false;

      xmlOut.endElement("entity");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }

  protected String storeClassFile() {
    File classFile = null;
    Function function;

/*    URL url = ((URLClassLoader)getClass().getClassLoader()).findResource(getClass().getName());
    if (url != null) {
      File original = new File(url.getFile());
      File dest = new File(g.tempDirectory + g.FS + "classes" + g.FS +  "roebling" + g.FS +
                           original.getName());
      try {
        FileUtil.copy(original, dest);
      }
      catch (IOException e) {
        return "";
      }
    }
    return getClass().getName();
  }*/

    Class newClass = Compiler.makeNewFunctionType(this, mName, mProperties,
                                                  getCode(), getClassLevelCode());
    if (newClass == null)
      return "FUNCTION_ERROR in function:" + mName;

    classFile = new File(g.tempDirectory + g.FS + "roebling" + g.FS +
                           FileUtil.getPostfix(newClass.getName()) + ".class");
    String newName = g.tempDirectory + g.FS + "classes" + g.FS + "roebling" + g.FS +
                            FileUtil.getPostfix(newClass.getName()) + ".class";

    classFile.renameTo(new File(newName));
    return newClass.getName();
  }

  public void update() {
    if (mTriggerType == SCENEGRAPH_TRIGGER && !mRunOnce)
      return;
    super.update();
    mRunOnce = false;
  }

//------------- Utility functions
  private Enclosure mCreateContext = null;
  private String mEntityName = "";
  private String mPropertyName = "";

  private String firstWord(String name) {
    StringTokenizer nameSplitter = new StringTokenizer(name, " ");
    return nameSplitter.nextToken();
  }

  private boolean decodeName(Enclosure context, String fullName) {
    StringBuffer path = new StringBuffer(fullName);
    Util.replaceString(path, "..", "*");
    StringTokenizer propertySplitter = new StringTokenizer(path.toString(), ".");
    String entityPath = propertySplitter.nextToken();
    try {
      mPropertyName = propertySplitter.nextToken();
    }
    catch (Exception e) {
      mPropertyName = "";
    }

    StringTokenizer pathSplitter = new StringTokenizer(entityPath, "/\\");
    Entity drillContext = context;
    mEntityName = firstWord(pathSplitter.nextToken());

    while (pathSplitter.hasMoreTokens()) {
      if (mEntityName.equals("*")) {
        if (drillContext != null)
          drillContext = drillContext.mEnclosure;
      }
      else if (drillContext.isEnclosure())
        drillContext = ((Enclosure)drillContext).getEntity(mEntityName);
      mEntityName = firstWord(pathSplitter.nextToken());
    }
    if (drillContext == null)
      return false;

    if (mEntityName.equals("*")) {
      if (drillContext != null)
        mCreateContext = drillContext.mEnclosure;
      mEntityName = "";
      return true;
    }
    if (drillContext.isEnclosure()) {
      mCreateContext = (Enclosure)drillContext;
    }
    else
      return false;

    return true;
  }


  // Point creation
  protected Point newPoint() {
    return newPoint("p", mEnclosure);
  }
  protected Point newPoint(String name) {
    return newPoint(name, mEnclosure);
  }
  protected Point newPoint(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    roebling.Point p = new roebling.Point(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(p, mCreateContext);
    }
    return p;
  }

  //Line creation
  protected Line newLine() {
    return newLine("l", mEnclosure);
  }
  protected Line newLine(String name) {
    return newLine(name, mEnclosure);
  }
  protected Line newLine(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    roebling.Line l = new roebling.Line(mCreateContext.newName(mEntityName), mCreateContext);
    l.setStartPoint(-100, 0);
    l.setStartPoint(100, 0);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(l, mCreateContext);
    }
    return l;
  }

  //Polygon creation
/*  protected Polygon newPolygon() {
    return newPolygon("Polygon", mEnclosure);
  }
  protected Polygon newPolygon(String name) {
    return newPolygon(name, mEnclosure);
  }
  protected Polygon newPolygon(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    roebling.Polygon poly = new roebling.Polygon(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(poly, mCreateContext);
    }
    return poly;
  }*/

  //Text creation
  protected Text newText() {
    return newText("t", mEnclosure);
  }
  protected Text newText(String name) {
    Text t = newText(name, mEnclosure);
    t.setText(name);
    return t;
  }
  protected Text newText(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Text t = new Text(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(t, mCreateContext);
    }
    return t;
  }

  //Rotation creation
  protected Rotation newRotation(double angle) {
    Rotation rot = newRotation("Rotation", mEnclosure);
    rot.setAngle(angle);
    return rot;
  }
  protected Rotation newRotation() {
    return newRotation("Rotation", mEnclosure);
  }
  protected Rotation newRotation(String name) {
    return newRotation(name, mEnclosure);
  }
  protected Rotation newRotation(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Rotation rot = new Rotation(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(rot, mCreateContext);
    }
    return rot;
  }

  //Translation creation
  protected Translation newTranslation(double dx, double dy) {
    Translation trans = newTranslation("Translation", mEnclosure);
    trans.setX(dx);
    trans.setY(dy);
    return trans;
  }
  protected Translation newTranslation() {
    return newTranslation("Translation", mEnclosure);
  }
  protected Translation newTranslation(String name) {
    return newTranslation(name, mEnclosure);
  }
  protected Translation newTranslation(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Translation trans = new Translation(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(trans, mCreateContext);
    }
    return trans;
  }

  //Scale creation
  protected Scale newScale(double x, double y) {
    Scale scale = newScale("Scale", mEnclosure);
    scale.setX(x);
    scale.setY(y);
    return scale;
  }
  protected Scale newScale() {
    return newScale("Scale", mEnclosure);
  }
  protected Scale newScale(String name) {
    return newScale(name, mEnclosure);
  }
  protected Scale newScale(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Scale scale = new Scale(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(scale, mCreateContext);
    }
    return scale;
  }

  //Image creation
  protected Image newImage(String name, String imageFile) {
    Image im = newImage(name, mEnclosure);
    im.setImage(imageFile);
    return im;
  }
  protected Image newImage() {
    return newImage("Image", mEnclosure);
  }
  protected Image newImage(String name) {
    return newImage(name, mEnclosure);
  }
  protected Image newImage(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Image im = new Image(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(im, mCreateContext);
    }
    return im;
  }

  //Button creation

  //General access
  protected Enclosure getEnclosure(String name) {
    try {
      return (Enclosure)getEntity(name);
    }
    catch (Exception e) {
      return null;
    }
  }
  protected Entity getEntity(String name) {
    if (!decodeName(mEnclosure, name))
      return null;
    return mCreateContext.getEntity(mEntityName);
  }

  //Property manipulation (locating, getting, setting, connecting, adding to enclosures)
  protected Property prop(String name) {
    Entity entity = getEntity(name);
    if (entity == null)
      return null;
    return entity.getProperty(mPropertyName);
  }

  protected boolean connect(Entity parentEntity, String parentPropName,
                            Entity childEntity, String childPropName) {
    Property parent = parentEntity.getProperty(parentPropName);
    if (parent == null)
      return false;
    Property child = childEntity.getProperty(childPropName);
    if (child == null)
      return false;
    return connect(parent, child);
  }
  protected boolean connect(String parentName, String childName) {
    Property parent = prop(parentName);
    if (parent == null)
      return false;
    Property child = prop(childName);
    if (child == null)
      return false;
    return connect(parent, child);
  }
  protected boolean connect(Property parent, Property child) {
    if (!Property.newRelationship(parent, child))
      return false;
    if (g.HAS_SCENEGRAPH) {
      PropertyNode parentNode;
      PropertyNode childNode;
      parentNode = g.sceneGraphPanel.getOutputNode(parent);
      if (parentNode == null)
        return false;
      childNode = g.sceneGraphPanel.getInputNode(child);
      if (childNode == null)
        return false;
      g.sceneGraphPanel.newConnection(parentNode, childNode);
    }
    return true;
  }

  protected Property addLocationProp(String name, boolean input) {
    Enclosure enc = getEnclosure(name);
    if (enc == null)
      return null;
    return addLocationProp(enc, mPropertyName, input);
  }

  protected Property addLocationProp(Enclosure enc, String propName, boolean input) {
    if (enc.getProperty(propName) != null)
      return null;
    LocationProp prop = new LocationProp();
    prop.mName = propName;
    prop.x = 0.0;
    prop.y = 0.0;
    prop.setIsEnclosureIn(input);
    prop.setIsEnclosureOut(!input);
    enc.addProperty(prop);

    if (g.HAS_SCENEGRAPH) {
      EntityNode node = g.sceneGraphPanel.getEntityNode(enc);
      if (node != null)
        node.setEntity(enc);
    }
    return prop;
  }


  protected NumProp addNumProp(String name, boolean input) {
    Enclosure enc = getEnclosure(name);
    if (enc == null)
      return null;
    return addNumProp(enc, mPropertyName, input);
  }

  protected NumProp addNumProp(Enclosure enc, String propName, boolean input) {
    if (enc.getProperty(propName) != null)
      return null;
    NumProp prop = new NumProp();
    prop.mName = propName;
    prop.value = 0.0;
    prop.setIsEnclosureIn(input);
    prop.setIsEnclosureOut(!input);
    enc.addProperty(prop);

    if (g.HAS_SCENEGRAPH) {
      EntityNode node = g.sceneGraphPanel.getEntityNode(enc);
      if (node != null)
        node.setEntity(enc);
    }
    return prop;
  }


  //Enclosure creation
  protected Enclosure newEnclosure() {
    return newEnclosure("enc", mEnclosure);
  }
  protected Enclosure newEnclosure(String name) {
    return newEnclosure(name, mEnclosure);
  }
  protected Enclosure newEnclosure(String name, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    Enclosure e = new Enclosure(mCreateContext.newName(mEntityName), mCreateContext);
    if (g.HAS_SCENEGRAPH) {
      EnclosureNode encNode = (EnclosureNode)g.sceneGraphPanel.addNewEntityNode(e, mCreateContext);
      if (encNode != null)
        encNode.setIsRevealed(false);
    }
    return e;
  }

  //Function creation

  //Resource instantiation
  protected Entity instantiate(String resPath) {
    File tempFile = new File(resPath);
    return instantiate(tempFile.getName(), resPath, mEnclosure);
  }
  protected Entity instantiate(String name, String resPath) {
    return instantiate(name, resPath, mEnclosure);
  }
  protected Entity instantiate(String name, String resPath, Enclosure enclosure) {
    if (!decodeName(enclosure, name))
      return null;
    String newName = mCreateContext.newName(mEntityName);
    Entity e = mCreateContext.instantiateResource(resPath);
    if (e == null)
      return null;
    if (!e.mName.equals(newName))
      e.mName = newName;
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(e, mCreateContext);
    }
    return e;
  }

  //Entity deletion
  protected void delete(String name) {
    Entity entity = getEntity(name);
    if (entity == null)
      return;
    if (g.HAS_SCENEGRAPH) {
      EntityNode node = g.sceneGraphPanel.getEntityNode(entity);
      if (node == null)
        entity.delete();
      else
        node.delete();
    }
    else {
      entity.delete();
    }
  }
}