package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

public class FunctionPlus extends Function {

  private NumProp mAProp;
  private NumProp mBProp;
  private NumProp mCProp;
  private double a;
  private double b;
  private double c;

  public FunctionPlus() {
  }

  public FunctionPlus(String name, Enclosure enclosure) {
    super(name, enclosure);
    mBaseClassName = "FunctionPlus";
    addProperties();
  }


  protected void addProperties() {
    mAProp  = new NumProp();
    mAProp.mName = "a";
    mAProp.value = 1.0;
    mAProp.valueSet();
    mAProp.setIsInput(true);
    mAProp.setIsOutput(false);
    addProperty(mAProp);

    mBProp  = new NumProp();
    mBProp.mName = "b";
    mBProp.value = 2.0;
    mBProp.valueSet();
    mBProp.setIsInput(true);
    mBProp.setIsOutput(false);
    addProperty(mBProp);

    mCProp = new NumProp();
    mCProp.mName = "c";
    mCProp.value = 3.0;
    mCProp.valueSet();
    mCProp.setIsInput(false);
    mCProp.setIsOutput(true);
    addProperty(mCProp);
  }

  public String getCode() {
    return "c = a + b;";
  }

  public void run() {
    c = a + b;
  }

  protected void readInputs() {
    mAProp.validate();
    a = mAProp.value;
    mBProp.validate();
    b = mBProp.value;
  }

  protected void writeOutputs() {
    mCProp.value = c;
    mCProp.valueSet();
  }
}