package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

public class Image extends GraphicEntity {
  public final static int DEFAULT_SIZE = 2;

  private StringProp mImageNameProperty;
  private String mImageName;
  private String mSavedImageName = "";
  private LocationProp mTopLeftProperty;
//  private Point2D mTopLeft;
//  private NumProp mWidthProperty;
//  private NumProp mHeightProperty;
  private LocationProp mBottomRightProperty;
  private boolean mBottomRightChanged = false;

  private NumProp mAlphaProperty;
  private double mAlpha;
/*  private NumProp mRotateProperty;
  private double mRotate = 0.0;*/
  private NumProp mLayerProperty;

  AffineTransform mTransform = new AffineTransform();
  Composite mOriginalComposite;
  int mWidth = 0;
  int mHeight = 0;
  int mSavedWidth = 0;
  int mSavedHeight = 0;

  double mXScale = 0.0;
  double mYScale = 0.0;

//  Point2D mRotHit = new Point2D.Double();

  private BoolProp mVisibleProperty;
  private BoolProp mSelectableProperty;

  private BasicStroke mLineStroke;
  private Point2D mPreDragTopLeft = new Point2D.Double();
  private Point2D mPreDragBottomRight = new Point2D.Double();
  private Point2D mPostDragTopLeft = new Point2D.Double();
  private Point2D mPostDragBottomRight = new Point2D.Double();
  private Point2D.Double mPanelTopLeft = new Point2D.Double();
  private Point2D.Double mPanelBottomRight = new Point2D.Double();

  private ImageIcon mImageIcon;
  private java.awt.Image mDrawImage;

  public Image() {
    super();
    mEntityType = "image";
    addProperties();
  }

  public Image(String name, Enclosure enclosure) {
    super(name, enclosure);
    mEntityType = "image";
    addProperties();
  }

  protected void addProperties() {
    mImageNameProperty  = new StringProp();
    mImageNameProperty.mName = "ImageName";
    mImageNameProperty.value = "images/default.gif";
    mImageNameProperty.valueSet();
    addProperty(mImageNameProperty);

    mTopLeftProperty = new LocationProp();
    mTopLeftProperty.mName = "TopLeft";
    addProperty(mTopLeftProperty);

    mBottomRightProperty = new LocationProp();
    mBottomRightProperty.mName = "BottomRight";
    addProperty(mBottomRightProperty);

/*    mWidthProperty = new NumProp();
    mWidthProperty.mName = "Width";
    addProperty(mWidthProperty);

    mHeightProperty = new NumProp();
    mHeightProperty.mName = "Height";
    addProperty(mHeightProperty);

    mRotateProperty = new NumProp();
    mRotateProperty.mName = "Rotation";
    mRotateProperty.value = 0.0;
    addProperty(mRotateProperty);
*/
    mAlphaProperty = new NumProp();
    mAlphaProperty.mName = "Alpha";
    mAlphaProperty.value = 255.0;
    addProperty(mAlphaProperty);

    mLayerProperty = new NumProp();
    mLayerProperty.mName = "Layer";
    mLayerProperty.value = 0;
    addProperty(mLayerProperty);

    mVisibleProperty = new BoolProp();
    mVisibleProperty.mName = "Visible";
    mVisibleProperty.value = true;
    addProperty(mVisibleProperty);

    mSelectableProperty = new BoolProp();
    mSelectableProperty.mName = "Selectable";
    mSelectableProperty.value = true;
    addProperty(mSelectableProperty);
  }

  public void setImage(String imagePath) {
    mImageNameProperty.value = imagePath;
    mImageNameProperty.valueSet();
  }

  public void setTopLeft(Point2D TopLeft) {
    mTopLeftProperty.x = TopLeft.getX();
    mTopLeftProperty.y = TopLeft.getY();
    mTopLeftProperty.valueSet();
  }

  public void setBottomRight(Point2D BottomRight) {
    mBottomRightProperty.x = BottomRight.getX();
    mBottomRightProperty.y = BottomRight.getY();
    mBottomRightProperty.valueSet();
  }

  protected void readInputs() {
    mImageNameProperty.validate();
    mImageName = mImageNameProperty.value;

    mAlphaProperty.validate();
    mAlpha = Math.max(Math.min(mAlphaProperty.value / 255.0, 1.0), 0.0);
    mTopLeftProperty.validate();
    mPanelTopLeft.setLocation(g.transform.toPanelLocation(new Point2D.Double(mTopLeftProperty.x, mTopLeftProperty.y)));
//    mWidthProperty.validate();
//    mWidth = (int)Math.max(mWidthProperty.value, 0.0);
//    mHeightProperty.validate();
//    mHeight = (int)Math.max(mHeightProperty.value, 0.0);
//    mRotateProperty.validate();
//    mRotate = mRotateProperty.value;

    mBottomRightProperty.validate();
    mPanelBottomRight.setLocation(g.transform.toPanelLocation(new Point2D.Double(mBottomRightProperty.x, mBottomRightProperty.y)));
    mWidth = (int)Math.abs(mPanelBottomRight.getX() - mPanelTopLeft.getX());
    mHeight = (int)Math.abs(mPanelBottomRight.getY() - mPanelTopLeft.getY());

    mVisibleProperty.validate();
    mVisible = mVisibleProperty.value;
    mSelectableProperty.validate();
    mSelectable = mSelectableProperty.value;
    mLayerProperty.validate();
    mLayer = (int)mLayerProperty.value;
  }

  protected void writeOutputs() {
/*    if (mWidth != (int)mWidthProperty.value) {
      mWidthProperty.value = mWidth;
      mWidthProperty.valueSet();
    }
    if (mHeight != (int)mHeightProperty.value) {
      mHeightProperty.value = mHeight;
      mHeightProperty.valueSet();
    }*/
    if (mBottomRightChanged)
      mBottomRightProperty.valueSet();
  }

  public void draw(Graphics2D graphics) {
//    update();
    if (! mVisible)
        return;

    if (mDrawImage == null)
      return;

    mEnclosure.makeTransform();
    mSaveTrans = graphics.getTransform();
    graphics.transform(mEnclosure.mCompositeTrans);

    mTransform.setToIdentity();
    mTransform.translate(mPanelTopLeft.getX(), mPanelTopLeft.getY());
    mTransform.scale(mXScale, mYScale);
//    mTransform.rotate(mRotate * Math.PI / 180.0);

    if (mAlpha < 0.99) {
      mOriginalComposite = graphics.getComposite();
      graphics.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)mAlpha));
      graphics.drawImage(mDrawImage, mTransform, null);
      graphics.setComposite(mOriginalComposite);
    }
    else {
      graphics.drawImage(mDrawImage, mTransform, null);
    }
    graphics.setTransform(mSaveTrans);
  }

  public void savePreDragLocation() {
    mPreDragTopLeft.setLocation(mPanelTopLeft);
    mPreDragBottomRight.setLocation(mPanelBottomRight);
  }

  public  boolean hit(Point2D hitLocation)  {
    if (!mSelectable)
      return false;
    if (mImageIcon == null)
      return false;
    mEnclosure.inverseTransform(hitLocation,mTransHitPoint);

    if (mTransHitPoint.getX() < mPanelTopLeft.getX())
      return false;
    if (mTransHitPoint.getX() > mPanelTopLeft.getX() + mWidth)
      return false;
    if (mTransHitPoint.getY() < mPanelTopLeft.getY())
      return false;
    if (mTransHitPoint.getY() > mPanelTopLeft.getY() + mHeight)
      return false;
    return true;


/*    mRotHit.setLocation(hitLocation.getX() - mPanelTopLeft.getX(),
                        hitLocation.getY() - mPanelTopLeft.getY());
    double cos = Math.cos(-(mRotate * Math.PI / 180.0));
    double sin = Math.sin(-(mRotate * Math.PI / 180.0));
    mRotHit.setLocation((mRotHit.getX()*cos - mRotHit.getY()*sin) + mPanelTopLeft.getX(),
                        (mRotHit.getX()*sin + mRotHit.getY()*cos) + mPanelTopLeft.getY());


    if (mImageIcon == null)
      return false;
    if (mRotHit.getX() < mPanelTopLeft.getX())
      return false;
    if (mRotHit.getX() > mPanelTopLeft.getX() + mWidth)
      return false;
    if (mRotHit.getY() < mPanelTopLeft.getY())
      return false;
    if (mRotHit.getY() > mPanelTopLeft.getY() + mHeight)
      return false;
    return true;
    */
  }

  public void move(Point2D mouseDelta) {
    mPostDragTopLeft.setLocation(mPreDragTopLeft.getX() + mouseDelta.getX(),
                                   mPreDragTopLeft.getY() + mouseDelta.getY());
    mPostDragBottomRight.setLocation(mPreDragBottomRight.getX() + mouseDelta.getX(),
                                   mPreDragBottomRight.getY() + mouseDelta.getY());

    mTopLeftProperty.setLocation(g.transform.toGeomLocation(mPostDragTopLeft));
    mBottomRightProperty.setLocation(g.transform.toGeomLocation(mPostDragBottomRight));
    mTopLeftProperty.valueSet();
    mBottomRightProperty.valueSet();
  }

  public void run() {
    ImageIcon oldImage = mImageIcon;
    mBottomRightChanged = false;

    if (!mSavedImageName.equals(mImageName)) {
      try {
        mImageIcon = new ImageIcon(g.resourceDirectory.getPath() + g.FS + mImageName);
        mDrawImage = mImageIcon.getImage();
        mSavedHeight = mImageIcon.getIconHeight();
        mSavedWidth = mImageIcon.getIconWidth();
      }
      catch (Exception e) {
        mImageIcon = null;
      }
    }
    if (oldImage == null && mImageIcon != null) {
      mSavedWidth = mWidth = mImageIcon.getIconWidth();
      mSavedHeight = mHeight = mImageIcon.getIconHeight();
      mPanelBottomRight.setLocation(mPanelTopLeft.getX() + mWidth, mPanelTopLeft.getY() + mHeight);
      mBottomRightProperty.setLocation(g.transform.toGeomLocation(mPanelBottomRight));
      mBottomRightChanged = true;
    }
//    if (mSavedHeight != mHeight || mSavedWidth != mWidth) {
//      mDrawImage = mBaseImage.getScaledInstance(mWidth, mHeight, java.awt.Image.SCALE_FAST);
//    }

//    mSavedHeight = mHeight;
//    mSavedWidth = mWidth;
    if (mSavedWidth == 0)
      mXScale = 1.0;
    else
      mXScale = (double)mWidth / (double)mSavedWidth;

    if (mSavedHeight == 0)
      mYScale = 1.0;
    else
      mYScale = (double)mHeight / (double)mSavedHeight;

    mSavedImageName = mImageName;
  }
}