package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.geom.*;

public class LocationProp extends Property {

  public double x;
  public double y;

  public double xs[] = new double[0];
  public double ys[] = new double[0];

  public LocationProp() {
    super();
    mType = LOCATION;
    isCompound = true;
  }

  public void setLocation(Point2D point) {
    x = point.getX();
    y = point.getY();
  }

  public void setLocation(double x, double y) {
    this.x = x;
    this.y = y;
  }

  public double distance(LocationProp loc2) {
    return Point2D.distance(x, y, loc2.x, loc2.y);
  }

  public void copyValue(Property property) {
    if (isArray) {
      xs = (double[])((LocationProp)property).xs.clone();
      ys = (double[])((LocationProp)property).ys.clone();
    }
    else {
      x = ((LocationProp)property).x;
      y = ((LocationProp)property).y;
    }
  }

  public String getValueByName(String fieldName) {
    if (fieldName.equals("x")) {
      return String.valueOf(x);
    }
    else if (fieldName.equals("y")) {
      return String.valueOf(y);
    }
    else return "0.0";
  }

  public String getValueByName(String fieldName, int index) {
    if (fieldName.equals("x")) {
      return String.valueOf(xs[index]);
    }
    else if (fieldName.equals("y")) {
      return String.valueOf(ys[index]);
    }
    else return "0.0";
  }

  public void setValueByName(String fieldName, String aValue) {  // used for compound types
    if (fieldName.equals("x")) {
      x = Double.valueOf(aValue).doubleValue();
    }
    else if (fieldName.equals("y")) {
      y = Double.valueOf(aValue).doubleValue();
    }
  }

  public void setValueByName(String fieldName, int index, String aValue) {  // used for compound types
    if (fieldName.equals("x")) {
      xs[index] = Double.valueOf(aValue).doubleValue();
    }
    else if (fieldName.equals("y")) {
      ys[index] = Double.valueOf(aValue).doubleValue();
    }
  }

  public int arrayLength() {
    return xs.length;
  }

  public void setArrayLength(int length) {
    xs = new double[length];
    ys = new double[length];
  }

  public void insertAt(int index) {
    xs = Util.insertDouble(xs, 0, index);
    ys = Util.insertDouble(ys, 0, index);
  }

  public void deleteItem(int index) {
    xs = Util.removeDouble(xs, index);
    ys = Util.removeDouble(ys, index);
  }

  public boolean serializeProperty(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("property");

      super.serializeProperty(xmlOut);
      if (isArray) {
        Util.serializeElement(xmlOut, "valueName", "x");
        xmlOut.startElement("value");
        Util.serializeElement(xmlOut, "length", xs.length);
        for (int i = 0; i < xs.length; i++) {
          Util.serializeElement(xmlOut, "index", xs[i]);
        }
        xmlOut.endElement("value");

        Util.serializeElement(xmlOut, "valueName", "y");
        xmlOut.startElement("value");
        for (int i = 0; i < ys.length; i++) {
          Util.serializeElement(xmlOut, "index", ys[i]);
        }
        xmlOut.endElement("value");
      }
      else {
        Util.serializeElement(xmlOut, "valueName", "x");
        Util.serializeElement(xmlOut, "value", x);
        Util.serializeElement(xmlOut, "valueName", "y");
        Util.serializeElement(xmlOut, "value", y);
      }

      xmlOut.endElement("property");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }

}