package roebling;

import javax.swing.DefaultDesktopManager;

/**
 * Title:        ParaMour
 * Description:  Parametric design tool.
 * Copyright:    Copyright (c) 2001
 * Company:      NewBricks
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import java.beans.PropertyVetoException;

/**
 * MyDesktopManager.
 * The DefaultDesktopManager uses a static (!)
currentActiveFrame. I
 * don't want that, since I want to have more than one
DesktopManager.
 * This bug causes problems when DesktopManagers are nested.
 *
 * Also activateFrame() contains a workaround of Bug ID
4302764, the
 * focus does not switch properly between 2 JInternalFrames.
 *
 * @author Birgit Arkesteijn
 * @version $Revision: 1.3 $ $Date: 2001/01/16 16:33:34 $
 *

*/
public class MyDesktopManager extends DefaultDesktopManager
{
    private static final String     version_id = "@(#)$Id: MyDesktopManager.java,v 1.3 2001/01/16 16:33:34 birgit Exp $ Copyright Westhawk Ltd";

    protected JInternalFrame currentActiveFrame = null;
    protected Component whoHadTheFocus = null;


/** This will activate <b>f</b> moving it to the front. It
will
  * set the current active frame (if any)
IS_SELECTED_PROPERTY to false.
  * There can be only one active frame across all Layers.
  */
public void activateFrame(JInternalFrame f)
{
    Container p = f.getParent();
    Component[] c;

    // fix for bug: 4162443
    if(p == null)
    {
        // If the frame is not in parent, it's icon maybe, check it
        p = f.getDesktopIcon().getParent();
        if(p == null)
            return;
    }
    // we only need to keep track of the currentActive InternalFrame, if any
    if (currentActiveFrame == null)
    {
        currentActiveFrame = f;
    }
    else if (currentActiveFrame != f)
    {
        // Work around the focus bug. Bug ID 4302764
        // Before 'moveToFront', get the component with the focus
        // Move the focus to another component.
        // After the 'moveToFront' set the focus back

//        whoHadTheFocus = com.aspentech.janet.janetscreen.Mainscreen.whoHasTheFocus(f);
        try {
            whoHadTheFocus = ((FocusTrackingInternalFrame)f).whoHasTheFocus();
            currentActiveFrame.requestFocus();
        }
        catch (Exception e) {
        }


        // if not the same frame as the current active
        // we deactivate the current
        if (currentActiveFrame.isSelected())
        {
            try
            {
                currentActiveFrame.setSelected(false);
            }
            catch(PropertyVetoException e2) {}

}
currentActiveFrame = f;
    }

    f.moveToFront();
    if (whoHadTheFocus != null)
    {
        whoHadTheFocus.requestFocus();
    }
}


// implements javax.swing.DesktopManager
public void deactivateFrame(JInternalFrame f)
{
  if (currentActiveFrame == f)
    currentActiveFrame = null;
}

// workaround for bug 4326562
public void deiconifyFrame(JInternalFrame f)
{
    super.deiconifyFrame(f);
    f.toFront();
}

}