package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */


public class NumProp extends Property {

  public double value;
  public double values[] = new double[0];

  public NumProp() {
    super();
    mType = NUMBER;
  }

  public void copyValue(Property property) {
    if (isArray) {
      values = (double[])((NumProp)property).values.clone();
    }
    else {
      value = ((NumProp)property).value;
    }
  }

  public String valueAsString() {
    validate();
    return String.valueOf(value);
  }

  public int arrayLength() {
    return values.length;
  }

  public void setArrayLength(int length) {
    values = new double[length];
  }
  public void setValueByName(String fieldName, String aValue) {  // used for compound types
    value = Double.valueOf(aValue).doubleValue();
  }

  public void setValueByName(String fieldName, int index, String aValue) {  // used for compound types
    values[index] = Double.valueOf(aValue).doubleValue();
  }

  public void insertAt(int index) {
    values = Util.insertDouble(values, 0, index);
  }

  public void deleteItem(int index) {
    values = Util.removeDouble(values, index);
  }

  public boolean serializeProperty(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("property");

      super.serializeProperty(xmlOut);
      if (isArray) {
        xmlOut.startElement("value");
        Util.serializeElement(xmlOut, "length", values.length);
        for (int i = 0; i < values.length; i++) {
          Util.serializeElement(xmlOut, "index", values[i]);
        }
        xmlOut.endElement("value");
      }
      else {
        Util.serializeElement(xmlOut, "value", value);
      }

      xmlOut.endElement("property");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }
}