package roebling;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

public class PlayerFrame extends JFrame {
  JPanel contentPane;
  BorderLayout borderLayout1 = new BorderLayout();
  HashMap fileToRRLmap = new HashMap();

  /**Construct the frame*/
  public PlayerFrame() {
    g.playerFrame = this;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    loadRRLs();

    RRL resourceRRL = getRRL(Player.mRootResource);
    if (resourceRRL != null)
      g.scene.instantiateResource(resourceRRL);
  }
  /**Component initialization*/
  private void jbInit() throws Exception  {
    g.scene = new Scene();
    g.scenePanel =  new ScenePanel();

    //setIconImage(Toolkit.getDefaultToolkit().createImage(PlayerFrame.class.getResource("[Your Icon]")));
    contentPane = (JPanel) this.getContentPane();
    contentPane.setLayout(borderLayout1);
    this.setSize(new Dimension(400, 300));
    this.setTitle("Roebling");
    contentPane.add(g.scenePanel, BorderLayout.CENTER);
  }
  /**Overridden so we can exit when window is closed*/
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }

  private void loadRRLs() {
    loadResourceDirectory(g.resourceDirectory);
    loadTempDirectory(g.tempDirectory);
  }

  private void loadTempDirectory(File directory) {
    String[] files = directory.list();
    File newFile;
    ResourceNode newNode;

    for (int i = 0; i < files.length; i++) {
      newFile = new File(directory.getPath() + g.FS + files[i]);
      if (!newFile.isDirectory()) {
        String relDirPath = newFile.getParent().substring(g.tempDirectory.getPath().length() + 1, newFile.getParent().length());
//        g.tr("Relative path: " + relDirPath);
        newNode = new ResourceNode(FileUtil.stripPostfix(files[i]), new File(g.resourceDirectory.getPath() + g.FS + relDirPath + ".enc"));
        newNode.mRRL.expandedEnclosureDirectory = newFile.getParentFile();
//        g.tr("Adding to map: " + g.resourceDirectory.getPath() + g.FS + relDirPath + ".enc" + "\n" + newNode.mRRL.expandedEnclosureDirectory + "\n");
        fileToRRLmap.put(g.resourceDirectory.getPath() + g.FS + relDirPath + ".enc", newNode.mRRL);

      }
      if (newFile.isDirectory()) {
        loadTempDirectory(newFile);
      }
    }
  }

  private void loadResourceDirectory(File directory) {
    String[] files = directory.list();
    File newFile;
    ResourceNode newNode;

    for (int i = 0; i < files.length; i++) {
      newFile = new File(directory.getPath() + g.FS + files[i]);
      newNode = new ResourceNode(FileUtil.stripPostfix(files[i]), newFile);
      if (! files[i].substring(0,1).equals("$"))
      fileToRRLmap.put(newFile.getPath(), newNode.mRRL);
      if (newFile.isDirectory()) {
        loadResourceDirectory(newFile);
      }
    }
  }

  public RRL getRRL(File resourceFile) {
//    Set keys = fileToRRLmap.keySet();
//    for (Iterator i = keys.iterator(); i.hasNext();)
//      System.out.println("Key: " + i.next());
    return (RRL)fileToRRLmap.get(resourceFile.getPath());
  }

  public RRL getRRL(String relativePath) {
    RRL test = getRRL(new File(g.resourceDirectory.getPath() + g.FS + relativePath + ".fun"));
    if (test == null)
      test = getRRL(new File(g.resourceDirectory.getPath() + g.FS + relativePath + ".enc"));
    return test;
  }

  public class ResourceNode {
    RRL mRRL;

    public ResourceNode (String name, File resourceFile) {
      mRRL = new RRL();
      mRRL.resourceFile = resourceFile;
      mRRL.isFunction = FileUtil.getPostfix(resourceFile.getName()).equals("fun");
      mRRL.isEnclosure = FileUtil.getPostfix(resourceFile.getName()).equals("enc");
    }
  }
}