package roebling;

/**
 * Title:        ParaMour
 * Description:  Parametric design tool.
 * Copyright:    Copyright (c) 2001
 * Company:      NewBricks
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.Color;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;

public abstract class PropertyBound extends GraphNode{

    protected Property mProperty;
    protected boolean mIsInput = false;
    protected boolean mIsOutput = false;
    protected boolean mIsEnclosureIn = false;
    protected boolean mIsEnclosureOut = false;
    protected EntityNode mEntityNode;
    protected ConnectionNode[] mConnectionNodes;

    public PropertyBound() {
        super();
        mConnectionNodes = new ConnectionNode[0];
    }

    public boolean isOnWireNode() {
        return false;
    }

    public void setIsInput(boolean isInput) {
        mIsInput = isInput;
    }

    public boolean isInput() {
        return mIsInput;
    }

    public void setIsOutput(boolean isOutput) {
        mIsOutput = isOutput;
    }

    public boolean isOutput() {
        return mIsOutput;
    }

    public void setIsEnclosureIn(boolean isEnclosureIn) {
        mIsEnclosureIn = isEnclosureIn;
    }

    public boolean isEnclosureIn() {
        return mIsEnclosureIn;
    }

    public void setIsEnclosureOut(boolean isEnclosureOut) {
        mIsEnclosureOut = isEnclosureOut;
    }

    public boolean isEnclosureOut() {
        return mIsEnclosureOut;
    }

    public Entity getEntity() {
        return mEntityNode.mEntity;
    }

    public void setProperty(Property property) {
        mProperty = property;
    }

    public Property getProperty() {
        return mProperty;
    }

    public EntityNode getEntityNode() {
        return mEntityNode;
    }

    public EnclosureNode getEnclosureNode() {
        return mEntityNode.getEnclosureNode();
    }
    public void removeConnectionNode(ConnectionNode connectionNode) {
        mConnectionNodes = (ConnectionNode[])Util.remove(mConnectionNodes, Types.CONNECTION_NODE, connectionNode);
    }

    public void clearConnections() {
      ConnectionNode connection;
      for (int i = 0; i < mConnectionNodes.length; i++) {
        connection = mConnectionNodes[i];
        connection.getEnclosureNode().removeConnectionNode(mConnectionNodes[i]);
      }
      g.sceneGraphPanel.repaint();
    }

    // Removes the old property connection node. (Will only remove one. (There should only be one.))
    public void newParent(PropertyBound newParent) {
        for (int i = 0; i < mConnectionNodes.length; i++) {
            if (mConnectionNodes[i].getChildNode() == this && mConnectionNodes[i].getParentNode() != newParent) {
                mConnectionNodes[i].getParent().remove(mConnectionNodes[i]);
                mConnectionNodes = (ConnectionNode[])Util.remove(mConnectionNodes, Types.CONNECTION_NODE, i);

                return;
            }
        }
    }

    public void moveConnections() {
        for (int i = 0; i < mConnectionNodes.length; i++) {
            mConnectionNodes[i].connectionsMoved();
        }
    }

    public void addConnectionNode(ConnectionNode connection) {
        mConnectionNodes = (ConnectionNode[])Util.append(mConnectionNodes, Types.CONNECTION_NODE, connection);
    }

    public ConnectionNode[] getConnectionNodes() {
        return mConnectionNodes;
    }

    public abstract java.awt.Point getCenter(Container container);

    public abstract int getOwnerLayer(EnclosureNode enclosureNode);

}