package roebling;

import javax.swing.JScrollPane;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;

public class PropertyEditPanel extends JPanel {

  protected static final int FONT_SIZE = 10;
  protected static Font mFont = new Font("sansserif", Font.PLAIN, FONT_SIZE);

  private static final int PROP_FIELD_Y_START = 28;
  private static final int PROP_FIELD_X_START = 24;
  private static final int PROP_FIELD_HEIGHT = 20;
  private static final int COMPOUND_ARRAY_UNIT_SPACE = 5;

  public EntityNode mEntityNode;
  public PropertyNode mPropertyNode;
  private PropertyEditView mPropEditView;
  private PropertyEditPanel mPropEditPanel;
  private JScrollPane mScrollPane;
  private ButtonPanel mButtonPanel;
  public String mErrorString;

  private JTextField mFirstFocusField = null;
  public Component mLastFocus = null;
  private JTextField mDummyField = null;

  public PropertyEditPanel(EntityNode entityNode, PropertyNode propNode) {
    mEntityNode = entityNode;
    mPropertyNode = propNode;
    mPropEditPanel = this;
    setLayout(new BorderLayout());
    setOpaque(false);

    mPropEditView = new PropertyEditView();
    mScrollPane = new JScrollPane(mPropEditView);

    mButtonPanel = new ButtonPanel();


    add(mScrollPane, BorderLayout.CENTER);
    add(mButtonPanel, BorderLayout.SOUTH);
    mButtonPanel.setSize(100, 14);

    mDummyField = new JTextField("Dummy");
    mDummyField.setBounds(0,0,1,1);
    mButtonPanel.add(mDummyField);
    mDummyField.setVisible(false);
    resized();
  }

  public void resized() {
    setLocation(20, 21);
    setSize(mEntityNode.getSize().width - 40, mEntityNode.getSize().height - 39);
    validate();
    if (mLastFocus != null) {
      mLastFocus.requestFocus();
    }
  }

  public void setLocation(int x, int y) {
      mDummyField.requestFocus();
      super.setLocation(x, y);
  }

  public void setSize(int width, int height) {
      mDummyField.requestFocus();
      super.setSize(width, height);
  }

  public void start() {
    mPropEditView.start();
  }

  private boolean canEditName() {
    if (mEntityNode.mEntity.isGraphicEntity()) return false;
    if (mEntityNode.mEntity.mIsInstance) return false;
    if (mEntityNode.mEntity.isTransformation()) return false;
    return true;
  }

  public void nextProperty() {
    PropertyNode[] propNodes = mEntityNode.mPropertyNodes;
    int index = Util.find(propNodes, mPropertyNode);
    boolean found = false;
    while (!found) {
      index++;
      if (index >= propNodes.length)
        index = 0;
      found = (propNodes[index].mProperty != mPropertyNode.mProperty);
    }
    g.sceneGraphPanel.setActivePropertyNode(propNodes[index]);
    mEntityNode.editPropNode(propNodes[index]);
    mEntityNode.repaint();
  }

  public void previousProperty() {
    PropertyNode[] propNodes = mEntityNode.mPropertyNodes;
    int index = Util.find(propNodes, mPropertyNode);
    boolean found = false;
    while (!found) {
      index--;
      if (index < 0)
        index = propNodes.length - 1;
      found = (propNodes[index].mProperty != mPropertyNode.mProperty);
    }
    g.sceneGraphPanel.setActivePropertyNode(propNodes[index]);
    mEntityNode.editPropNode(propNodes[index]);
    mEntityNode.repaint();
  }

  public class ButtonPanel extends JPanel {
    JLabel mDone = new JLabelNoAntialias(" Done ");
    JLabel mNextProp = new JLabelNoAntialias(" Next Property ");

    public ButtonPanel() {
      setOpaque(false);
      setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

      mDone.setFont(mFont);
      mDone.setForeground(Color.black);
      mDone.setBackground(Color.white);
      mDone.setOpaque(true);
      mDone.setBorder(BorderFactory.createLineBorder(Color.black));
      mDone.addMouseListener(new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          mEntityNode.editPropNode(mPropertyNode);
          mEntityNode.repaint();
        }
      });

      mNextProp.setFont(mFont);
      mNextProp.setForeground(Color.black);
      mNextProp.setBackground(Color.white);
      mNextProp.setOpaque(true);
      mNextProp.setBorder(BorderFactory.createLineBorder(Color.black));
      mNextProp.addMouseListener(new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          nextProperty();
        }
      });
      //mEntityNode.dispatchEvent(new MouseEvent(mEntityNode, MouseEvent.MOUSE_EXITED, 0, 0, 0, 0, 0, false));

      add(mDone);
      add(Box.createRigidArea(new Dimension(5, 14)));
      if (mEntityNode.mEntity.mProperties.length > 1)
        add(mNextProp);
    }
  }

  public class PropertyEditView extends JPanel {// implements Scrollable {
    JLabel mNameLabel = new JLabelNoAntialias("Name ");
    JTextField mNameField = new JTextFieldNoAntialias();
    Dimension mSize = new Dimension();
    Vector mValueEditors = new Vector();
    int buttonIndex;

    public PropertyEditView() {
      setBackground(Color.white);
      setLayout(null);

      mNameLabel.setLocation(10, 10);
      mNameLabel.setSize(40, 12);
      mNameLabel.setFont(mFont);
      mNameLabel.setForeground(Color.black);
      mNameField.setLocation(44, 9);
      mNameField.setSize(60, 14);
      mNameField.setFont(mFont);
      if (! canEditName() )
        mNameField.setEnabled(false);
      mNameField.addFocusListener(new FocusListener()  {
        public void focusGained(FocusEvent e) {
          mNameField.selectAll();
          mLastFocus = mNameField;
        }
        public void focusLost(FocusEvent e) {
            if (!JNaming.isValidJavaIdentifier(mNameField.getText())) {
              mNameField.setBackground(Color.red);
              mNameField.requestFocus();
            }
            else {
              mNameField.setBackground(Color.white);
              mPropertyNode.mProperty.mName = mNameField.getText();
              mPropertyNode.nameChanged();


            }
        }
      });

      mNameField.addKeyListener(new KeyAdapter()  {
        public void keyPressed(KeyEvent e) {
          if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            if (!JNaming.isValidJavaIdentifier(mNameField.getText())) {
              mNameField.setBackground(Color.red);
              mNameField.requestFocus();
            }
            else {
              mNameField.setBackground(Color.white);
              mPropertyNode.mProperty.mName = mNameField.getText();
              mPropertyNode.nameChanged();
              mEntityNode.editPropNode(mPropertyNode);
              mEntityNode.repaint();
            }
          }
          else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
            nextProperty();
          }
          else if (e.getKeyCode() == KeyEvent.VK_UP) {
            previousProperty();
          }
        }
      });


      add(mNameLabel);
      add(mNameField);

      placeEditFields();
      for (int i = 0; i < mValueEditors.size(); i++) {
        add((ValueField)mValueEditors.get(i));
      }

      mSize.setSize(100, height());
      this.setSize(mSize);

      /*
      if (mPropertyNode.mProperty.mType == Property.NUMBER) {
        NumValueField valueField = new NumValueField(mPropertyNode.mProperty.mName,
                            ((NumProp)mPropertyNode.mProperty).value, 24, 28);
        add(valueField);
      }
      if (mPropertyNode.mProperty.mType == Property.NUMBER) {
        NumValueField valueField = new NumValueField(mPropertyNode.mProperty.mName,
                            ((NumProp)mPropertyNode.mProperty).value, 24, 28);
        add(valueField);
      }*/

    }

    private int height() {
      if (mPropertyNode.mProperty.isArray) {
        return (PROP_FIELD_Y_START + (mValueEditors.size() * PROP_FIELD_HEIGHT)) +
               (((mValueEditors.size() - 1 ) * (COMPOUND_ARRAY_UNIT_SPACE - 2)));
      }
      return PROP_FIELD_Y_START + (mValueEditors.size() * PROP_FIELD_HEIGHT);
    }

    private ValueField placeOneField(int type, String name, String value, int y) {
      ValueField valueField;
      if (type == Property.NUMBER) {
        valueField = new NumValueField(mPropertyNode.mProperty, name, Double.valueOf(value).doubleValue(), PROP_FIELD_X_START, y);
      }
      else if (type == Property.BOOLEAN) {
        valueField = new BoolValueField(mPropertyNode.mProperty, name, Boolean.valueOf(value).booleanValue(), PROP_FIELD_X_START, y);
      }
      else if (type == Property.STRING) {
        valueField = new StringValueField(mPropertyNode.mProperty, name, value, PROP_FIELD_X_START, y);
      }
      else return null;

      mValueEditors.add(valueField);
      return valueField;
    }

    private void insertAt(int index) {
      mPropertyNode.mProperty.insertAt(index);
      mPropertyNode.mProperty.valueSet();
      g.scene.refresh();
      mEntityNode.editPropNode(mPropertyNode);
      mEntityNode.editPropNode(mPropertyNode);
    }

    private void delete(int index) {
      mPropertyNode.mProperty.deleteItem(index);
      mPropertyNode.mProperty.valueSet();
      g.scene.refresh();
      mEntityNode.editPropNode(mPropertyNode);
      mEntityNode.editPropNode(mPropertyNode);
    }

    private void placeSimpleArray(Property prop, int y) {
      int type = prop.mType;
      String name = prop.mName;

      add(new InsertLabel(0, y - 10));

      if (type == Property.NUMBER) {
        NumProp numProp = (NumProp)prop;
        for (int i = 0; i < prop.arrayLength(); i++) {
          NumValueField valueField = new NumValueField(mPropertyNode.mProperty, name + "[" + i + "]", numProp.values[i], PROP_FIELD_X_START, y);
          valueField.mIndex = i;
          valueField.mBaseName = name;
          mValueEditors.add(valueField);
          y += PROP_FIELD_HEIGHT;

          add (new DeleteLabel(i, y - 22));
          add (new InsertLabel(i + 1, y - 10));
        }
      }
      else if (type == Property.BOOLEAN) {
        BoolProp boolProp = (BoolProp)prop;
        for (int i = 0; i < prop.arrayLength(); i++) {
          BoolValueField valueField = new BoolValueField(mPropertyNode.mProperty, name + "[" + i + "]", boolProp.values[i], PROP_FIELD_X_START, y);
          valueField.mIndex = i;
          valueField.mBaseName = name;
          mValueEditors.add(valueField);
          y += PROP_FIELD_HEIGHT;

          add (new DeleteLabel(i, y - 22));
          add (new InsertLabel(i + 1, y - 10));
        }
      }
      else if (type == Property.STRING) {
        StringProp stringProp = (StringProp)prop;
        for (int i = 0; i < prop.arrayLength(); i++) {
          StringValueField valueField = new StringValueField(mPropertyNode.mProperty, name + "[" + i + "]", stringProp.values[i], PROP_FIELD_X_START, y);
          valueField.mIndex = i;
          valueField.mBaseName = name;
          mValueEditors.add(valueField);
          y += PROP_FIELD_HEIGHT;

          add (new DeleteLabel(i, y - 22));
          add (new InsertLabel(i + 1, y - 10));
        }
      }
    }

    private void placeCompoundArray(Property prop, int y) {
      String name;
      int subType;
      String value;
      ValueField valueField;
      CompoundType type = (CompoundType)g.propertyTypes[mPropertyNode.mProperty.mType];

      add(new InsertLabel(0, y - 10));

      for (int i = 0; i < prop.arrayLength(); i++) {
        for (int j = 0; j < type.mFieldOrder.size(); j++) {
          name = (String)type.mFieldOrder.get(j);
          subType = ((Integer)type.mFields.get(name)).intValue();
          value = prop.getValueByName(name, i);
          valueField = placeOneField(subType, name + "[" + i + "]", value, y);
          valueField.mBaseName = name;
          valueField.mIndex = i;
          y += PROP_FIELD_HEIGHT;
        }
        add (new DeleteLabel(i, y - 25));
        add (new InsertLabel(i + 1, y - 10));
        y += COMPOUND_ARRAY_UNIT_SPACE;
      }
    }

    private void placeEditFields() {
      boolean isArray = mPropertyNode.mProperty.isArray;
      boolean isCompound = mPropertyNode.mProperty.isCompound;
      Property prop = mPropertyNode.mProperty;
      String name;
      int index;
      int subType;
      String value;

      int y = PROP_FIELD_Y_START;
      if (! isArray) {
        if (! isCompound) {   // Simplest case, Atomic
          placeOneField(prop.mType, prop.mName, prop.valueAsString(), y);
        }
        else {              // Compound
          CompoundType type = (CompoundType)g.propertyTypes[mPropertyNode.mProperty.mType];
          for (int i = 0; i < type.mFieldOrder.size(); i++) {
            name = (String)type.mFieldOrder.get(i);
            subType = ((Integer)type.mFields.get(name)).intValue();
            value = prop.getValueByName(name);
            placeOneField(subType, name, value, y);
            y += PROP_FIELD_HEIGHT;
          }
        }
      }
      else {      // Array
        if (! isCompound) {   // Array of simple types
          placeSimpleArray(prop, y);
        }
        else {              // Compound
          placeCompoundArray(prop, y);
        }
      }

    }

    public Dimension getPreferredSize(  ) {
      return mSize;
    }

    public void start() {
      mNameField.setText(mPropertyNode.mProperty.mName);
      if (canEditName()) {
        mNameField.requestFocus();
      }
      else if (mFirstFocusField != null ){
        mFirstFocusField.requestFocus();
      }
    }
/*
    public Dimension getPreferredScrollableViewportSize() {
      return mSize;
    }
    public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
      return 8;
    }

    public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
      return 32;
    }

    public boolean getScrollableTracksViewportWidth() {
      return false;
    }

    public boolean getScrollableTracksViewportHeight() {
      return false;
    }*/
  }

  public class InsertLabel extends JLabelNoAntialias {
    int mIndex;

    public InsertLabel(int index, int y) {
      super( " Insert ");
      mIndex = index;
      setForeground(Color.gray);
      setFont(mFont);
      setBackground(Color.white);
      this.setLocation(110, y);
      this.setSize(36,12);
      setBorder(BorderFactory.createLineBorder(Color.gray));
      addMouseListener(new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          mPropEditView.insertAt(mIndex);
        }
      });
    }
  }

  public class DeleteLabel extends JLabelNoAntialias {
    int mIndex;

    public DeleteLabel(int index, int y) {
      super( "Del");
      mIndex = index;
      setForeground(Color.gray);
      setFont(mFont);
      setBackground(Color.white);
      this.setLocation(4, y);
      this.setSize(18,12);
      setBorder(BorderFactory.createLineBorder(Color.gray));
      addMouseListener(new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          mPropEditView.delete(mIndex);
        }
      });
    }
  }

  public class ValueField extends JPanel {
    JTextField mField;
    JLabel     mName;
    Property   mProp;
    public String mBaseName;
    public int mIndex;

    ValueField(Property prop, int type, String name, String value, int x, int y) {
      setLayout(null);
      setBackground(Color.white);
      setOpaque(true);

      mProp = prop;

      mBaseName = name;
      mName = new JLabelNoAntialias(name);
      mName.setForeground(Color.black);
      mName.setBackground(Color.white);
      mName.setOpaque(true);
      mName.setFont(mFont);
      mName.setLocation(0, 0);
      mName.setSize(40, 14);

      mField = new JTextFieldNoAntialias();
      if (mFirstFocusField == null) {
        mFirstFocusField = mField;
      }
      mField.setText(value);
      mField.setFont(mFont);
      mField.setSize(40, 14);
      mField.setLocation(40, 0);
      if (! mProp.isSettable())
        mField.setEnabled(false);

      mField.addFocusListener(new FocusListener()  {
        public void focusGained(FocusEvent e) {
          mField.selectAll();
          mLastFocus = mField;
        }
        public void focusLost(FocusEvent e) {
          //if (!mSkipValueUpdate) {
            if (!updateValue()) {
              mField.requestFocus();
            }
          //}
        }
      });

      mField.addKeyListener(new KeyAdapter()  {
        public void keyPressed(KeyEvent e) {
          if (e.getKeyCode() == KeyEvent.VK_ENTER) {
              if (updateValue()) {
                mEntityNode.editPropNode(mPropertyNode);
                mEntityNode.repaint();
              }
          }
          else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
            nextProperty();
          }
          else if (e.getKeyCode() == KeyEvent.VK_UP) {
            previousProperty();
          }
        }
      });

      this.setLocation(x, y);
      this.setSize(80, 20);

      add(mName);
      add(mField);
    }

    public boolean updateValue() {
      if (mProp.isArray) {
        mProp.setValueByName(mBaseName, mIndex, mField.getText());
      }
      else {
        mProp.setValueByName(mBaseName, mField.getText());
      }
      mProp.valueSet();
      g.scene.refresh();
      return true;
    }
  }

  public class NumValueField extends ValueField {
    double mValue;

    public NumValueField(Property prop, String name, double value, int x, int y) {
      super(prop, Property.NUMBER, name, String.valueOf(value), x, y);
    }
    public boolean updateValue() {
      try {
        mValue = Double.parseDouble(mField.getText());
        mField.setBackground(Color.white);
        return super.updateValue();
      }
      catch (NumberFormatException e) {
        mField.requestFocus();
        mField.selectAll();
        mField.setBackground(Color.red);
        return false;
      }
    }
  }

  public class BoolValueField extends ValueField {
    boolean mValue;

    public BoolValueField(Property prop, String name, boolean value, int x, int y) {
      super(prop, Property.BOOLEAN, name, String.valueOf(value), x, y);
    }
    public boolean updateValue() {
      mValue = Util.parseBoolean(mField.getText());
      mField.setText(String.valueOf(mValue));
      return super.updateValue();
    }
  }

  public class StringValueField extends ValueField {
    String mValue;

    public StringValueField(Property prop, String name, String value, int x, int y) {
      super(prop, Property.STRING, name, value, x, y);
    }
    public boolean updateValue() {
      mValue = mField.getText();
      return super.updateValue();
    }
  }

  public class JavaTokenValueField extends ValueField {
    String mValue;

    public JavaTokenValueField(Property prop, String name, String value, int x, int y) {
      super(prop, Property.STRING, name, value, x, y);
    }
    public boolean updateValue() {
      mValue = mField.getText();

      if (! JNaming.isValidJavaIdentifier(mValue)) {
        mField.setBackground(Color.red);
        return false;
      }
      mField.setBackground(Color.white);
      return super.updateValue();
    }
  }

}