package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.io.*;
import javax.swing.tree.*;
import java.awt.event.*;

public class Publisher {
  private Vector mRRLList = new Vector();
  private final int CANCEL = 0;
  private final int OK = 1;
  private int mButtonPushed = 0;
  private HashSet mResToProcess = new HashSet();
  private HashSet mResProcessed = new HashSet();
  private File storeFile;   // The directory into which we're saving all of this
  private File resourceDest; // The new resource directory
  private File classDest; // Where all the classes go
  private int mWidth = 600;
  private int mHeight = 600;
  private boolean mIncludeJRE = true;

  static private String REPLACE_RESOURCE = "$$$RESOURCE$$$";
  static private String REPLACE_WIDTH = "$$$WIDTH$$$";
  static private String REPLACE_HEIGHT = "$$$HEIGHT$$$";

  public Publisher() {
  }

  public void publishEnclosures() {
    if (! selectEnclosures())
      return;

    makeConstantDirs();
    copyConstantFiles();
    fillFullResourceList();
    copyResources();
    zipUpClasses();
    makeBatchFiles();
  }

  private void zipUpClasses() {
    File classDir = new File(storeFile.getPath() + g.FS + "lib" + g.FS + "roebling");
    String[] files = classDir.list();
    for (int i = 0; i < files.length; i++) {
      files[i] = classDir.getPath() + g.FS + files[i];
    }
    try {
      FileUtil.zipUp(files, classDir.getParent(), storeFile.getPath() + g.FS + "lib" + g.FS + "roebling.jar");
      FileUtil.clean(classDir);
      FileUtil.deleteFile(classDir.getParentFile(), "roebling");
    }
    catch (Exception e) {
    }
  }

  private void fillFullResourceList() {
    mResProcessed.clear();
    mResToProcess.clear();

    for (Iterator i = mRRLList.iterator(); i.hasNext(); ) {
      RRL exteriorRRL = (RRL)i.next();
      mResToProcess.add(exteriorRRL.relativePath() + ".enc");
    }

    while (mResToProcess.size() > 0) {
      String relPathToProcess = (String)mResToProcess.iterator().next();
      addResourcesUsed(relPathToProcess);
      mResToProcess.remove(relPathToProcess);
      mResProcessed.add(relPathToProcess);
    }


//    System.out.println("Resources used: ");
    for (Iterator j = mResProcessed.iterator(); j.hasNext(); ) {
      System.out.println("  " + j.next());
    }

  }

  private void makeConstantDirs() {
    storeFile = storeAs();
    if (storeFile == null)
      return;

    storeFile.mkdirs();
    File tempDir = new File(storeFile.getPath() + g.FS + "temp");
    tempDir.mkdir();

    new File(storeFile.getPath() + g.FS + "lib").mkdirs();
  }

  private void copyConstantFiles() {

    if (mIncludeJRE) {
      FileUtil.copyDirectory(new File(g.USER_DIR + g.FS + "bin"),
                            new File(storeFile.getPath() + g.FS + "bin"));

      FileUtil.copyDirectory(new File(g.USER_DIR + g.FS + "lib"),
                            new File(storeFile.getPath() + g.FS + "lib"));
    }

    classDest = new File(storeFile.getPath() + g.FS + "lib" + g.FS + "roebling");
    FileUtil.copyDirectory(new File(g.classDirectory.getPath() + g.FS + "roebling"),  classDest);

    resourceDest = new File(storeFile.getPath() + g.FS + "resources");
    resourceDest.mkdir();
  }

  private void copyResources() {
    for (Iterator i = mResProcessed.iterator(); i.hasNext(); ) {
      String relPath = (String)i.next();
      String postfix = FileUtil.getPostfix(relPath);
      if (postfix.equals("fun")) {
        FileUtil.unzip(g.resourceDirectory.getPath() + g.FS + relPath,
                       storeFile.getPath() + g.FS + "lib");
      }
      else if (postfix.equals("enc")) {
        try {
          File tempDir = new File(storeFile.getPath() + g.FS + "temp" + g.FS + FileUtil.stripPostfix(relPath));
          tempDir.mkdirs();

          FileUtil.unzip(g.resourceDirectory.getPath() + g.FS + relPath,
                       tempDir.getPath());

          FileUtil.clean(new File(tempDir.getPath() + g.FS + "classes"));
          FileUtil.deleteFile(tempDir, "classes");
        }
        catch (Exception e) {
        }
      }
      else {      // Images
        try {

          File resCopy = new File(resourceDest.getPath() + g.FS + relPath);
          File resDir = resCopy.getParentFile();
          resDir.mkdirs();

          FileUtil.copy(new File(g.resourceDirectory.getPath() + g.FS + relPath), resCopy);
        }
        catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
  }

  private void makeBatchFiles() {
    StringBuffer buffer = new StringBuffer();
    RRL rrl;
    for (Iterator i = mRRLList.iterator(); i.hasNext();) {
      rrl = (RRL)i.next();
      buffer.setLength(0);
      if (mIncludeJRE)
        FileUtil.readToStringBuffer(g.templateDirectory, "deploy.txt", buffer);
      else
        FileUtil.readToStringBuffer(g.templateDirectory, "deployNoJRE.txt", buffer);
      Util.replaceString(buffer, REPLACE_RESOURCE, rrl.relativePath());
      Util.replaceString(buffer, REPLACE_WIDTH, String.valueOf(mWidth));
      Util.replaceString(buffer, REPLACE_HEIGHT, String.valueOf(mHeight));

      String batchName = new File(rrl.relativePath()).getName() + ".bat";
      FileUtil.writeStringToFile(new File(storeFile.getPath() + g.FS + batchName),
                                 buffer.toString());
    }
  }

  private File storeAs() {
    JFileChooser chooser = new JFileChooser();
   // Note: source for ExampleFileFilter can be found in FileChooserDemo,
   // under the demo/jfc directory in the Java 2 SDK, Standard Edition.
//    ExampleFileFilter filter = new ExampleFileFilter();
//    filter.addExtension("fun");
//    filter.addExtension("enc");
//    filter.setDescription("Roebling resources");
//    chooser.setFileFilter(filter);
//    chooser.setFileView(new ImageFileView());
//    chooser.setFileSystemView(new ResourceFileSystemView(chooser.getFileSystemView()));
    chooser.setAcceptAllFileFilterUsed(true);
    chooser.setCurrentDirectory(new File(g.USER_DIR));
    int returnVal = chooser.showSaveDialog(g.sceneGraphFrame);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      return chooser.getSelectedFile();
    }
    else
      return null;
  }


  private void addResourcesUsed(String relPath) {
    FileUtil.unzip(g.resourceDirectory.getPath() + g.FS + relPath,
                   g.tempDirectory.getPath() + g.FS + FileUtil.stripPostfix(relPath));

    File unnamedClassDir = new File(g.tempDirectory.getPath() + g.FS + FileUtil.stripPostfix(relPath) +
                                    g.FS + "classes" + g.FS + "roebling");
    if (unnamedClassDir.exists()) {
      FileUtil.copyDirectory(unnamedClassDir, classDest);
    }

    ResourceParser parser = new ResourceParser(relPath);
    parser.parseFile();
  }

  private boolean selectEnclosures() {
    EnclosureSelector selector = new EnclosureSelector();
    mRRLList.clear();
    selector.pack();
    selector.show();

    if (mButtonPushed != OK)
      return false;
    if (mRRLList.size() == 0)
      return false;
    return true;
  }

  public class EnclosureSelector extends JDialog {
    JScrollPane jScrollPane = new JScrollPane();
    ResourceNode mRootNode;
    JTree mResourceTree = new JTree();
    JButton mOK;
    JButton mCancel;
    EnclosureSelector mSelf;
    JTextField mWidthField;
    JTextField mHeightField;
    JCheckBox mIncludeJREBox;

    public EnclosureSelector () {
      super(g.devFrame, "Publish enclosures", true);
      mSelf = this;
      init();
    }

    private void init() {
      this.setLocation(800, 300);
      this.setSize(390, 400);
      this.setResizable(true);

      Container content = getContentPane();
      content.setLayout(new BorderLayout());

      content.add(new JLabel("  Select enclosures to publish..."), BorderLayout.NORTH);
      content.add(jScrollPane, BorderLayout.CENTER);
      jScrollPane.getViewport().add(mResourceTree, null);
      loadResourceTree();
      mResourceTree.setModel(new DefaultTreeModel(mRootNode));
      mResourceTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
      mOK = new JButton("OK");
      mOK.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          mButtonPushed = OK;
          mIncludeJRE = mIncludeJREBox.isSelected();
          try {
            mWidth = Integer.parseInt(mWidthField.getText().trim());
            mHeight = Integer.parseInt(mHeightField.getText().trim());
          }
          catch (Exception e2) {
          }

          TreePath[] selPaths = mResourceTree.getSelectionPaths();
          if (selPaths != null) {
            for (int i = 0; i < selPaths.length; i++) {
              RRL rrl = ((ResourceNode)selPaths[i].getLastPathComponent()).mRRL;
              if (rrl.isEnclosure)
                mRRLList.add(rrl);
            }
          }
          mSelf.hide();
          mSelf.dispose();
        }
      });

      mOK.setDefaultCapable(true);
      mOK.requestDefaultFocus();
      mCancel = new JButton ("Cancel");
      mCancel.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          mButtonPushed = CANCEL;
          mSelf.hide();
          mSelf.dispose();
        }
      });

      JPanel bottomPanel = new JPanel();
      bottomPanel.setLayout(new BorderLayout());


      JPanel propsPanel = new JPanel();
      propsPanel.setLayout(new FlowLayout());


      JLabel widthLabel = new JLabel("Player width:");
      mWidthField = new JTextField("  " + String.valueOf(mWidth));


      JLabel heightLabel = new JLabel("Player height:");
      mHeightField = new JTextField("  " + String.valueOf(mHeight));


      mIncludeJREBox = new JCheckBox("Include JRE:", true);
      mIncludeJREBox.setHorizontalTextPosition(AbstractButton.LEFT);
      mIncludeJREBox.setForeground(widthLabel.getForeground());

      propsPanel.add(widthLabel);
      propsPanel.add(mWidthField);
      propsPanel.add(heightLabel);
      propsPanel.add(mHeightField);
      propsPanel.add(mIncludeJREBox);

      JPanel okPanel = new JPanel();
      okPanel.setLayout(new FlowLayout());
      okPanel.add(mOK);
      okPanel.add(mCancel);

      bottomPanel.add(propsPanel, BorderLayout.CENTER);
      bottomPanel.add(okPanel, BorderLayout.SOUTH);

      content.add(bottomPanel, BorderLayout.SOUTH);
    }


    private void loadResourceTree() {
      mRootNode = new ResourceNode(g.resourceDirectory.getName(), g.resourceDirectory);
      loadDirectory(mRootNode, g.resourceDirectory);
    }

    public void reloadResourceTree() {
      mRootNode.removeAllChildren();
      loadDirectory(mRootNode, g.resourceDirectory);
      ((DefaultTreeModel)mResourceTree.getModel()).reload(mRootNode);
    }

    public void loadDirectory(ResourceNode parentNode, File directory) {
      String[] files = directory.list();
      File newFile;
      ResourceNode newNode;

      for (int i = 0; i < files.length; i++) {
        newFile = new File(directory.getPath() + g.FS + files[i]);
        newNode = new ResourceNode(FileUtil.stripPostfix(files[i]), newFile);
        if (! files[i].substring(0,1).equals("$") &&
            (newFile.isDirectory() || FileUtil.getPostfix(files[i]).equals("enc")))
          parentNode.add(newNode);
        if (newFile.isDirectory()) {
          loadDirectory(newNode, newFile);
        }
      }
    }

    public class ResourceNode extends DefaultMutableTreeNode {
      RRL mRRL;


      public ResourceNode (String name, File resourceFile) {
        super(name);
        mRRL = new RRL();
        mRRL.resourceFile = resourceFile;
        mRRL.isFunction = FileUtil.getPostfix(resourceFile.getName()).equals("fun");
        mRRL.isEnclosure = FileUtil.getPostfix(resourceFile.getName()).equals("enc");
      }
    }
  }

  class ResourceParser extends XMLParser {
    private File mFile;
    private XMLReader mReader;
    private String mContent;
    private String mTag;

    private final int NONE = 0;
    private int TAG = 1;
    private int CONTENT = 2;
    private int mLastRead = NONE;

    private int mMode = NONE;
    private int FUNCTION_MODE = 1;
    private int ENCLOSURE_MODE = 2;
    private int PROPERTY_MODE = 4;
    private int IMAGE_MODE = 3;

    public ResourceParser(String relativePath) {
      mFile = new File(g.tempDirectory + g.FS + FileUtil.stripPostfix(relativePath) +
                       g.FS + "enclosure.xml");
      mReader = new XMLReader(mFile, this);
      mContent = null;
    }

    void process(String tag, String content) {
      String relPath;

      if (tag.equals("instanceOf")) {
        if (mMode == FUNCTION_MODE) {
          relPath = content + ".fun";
          mResProcessed.add(relPath);
        }
        else if (mMode == ENCLOSURE_MODE) {
          relPath = content + ".enc";
          if (! mResProcessed.contains(relPath))
            mResToProcess.add(relPath);
        }
      }
      else if (tag.equals("type")) {
        if (content.equals("function"))
          mMode = FUNCTION_MODE;
        else if (content.equals("enclosure"))
          mMode = ENCLOSURE_MODE;
      }
      else if (tag.equals("property")) {
        mMode = PROPERTY_MODE;
      }
      else if (tag.equals("name")) {
        if (mMode == PROPERTY_MODE && content.equals("ImageName"))
          mMode = IMAGE_MODE;
      }
      else if (tag.equals("value")) {
        if (mMode == IMAGE_MODE) {
          mResProcessed.add(content);
          mMode = NONE;
        }
      }
    }

    void endTag(String tag) {}

    public void startElement(String tag) {
      if (mLastRead == TAG) {
        process(mTag, "");
      }
      mTag = tag;
      mLastRead = TAG;
    }

    public void endElement(String tag) {
      endTag(tag);
    }

/*    private String charsToString(char[] ch, int start, int length) {
      StringBuffer buffer = new StringBuffer();
      buffer.setLength(length);
      for (int i = 0; i < length; i++) {
        buffer.setCharAt(i, ch[i + start]);
      }
      return buffer.toString();
    }*/

    public void characters(String chars) {
      mContent = chars; //charsToString(ch, start, length);
      process(mTag, mContent);
      mLastRead = CONTENT;
    }

    public void parseFile() {
//      try {
        mReader.parse();
/*      }
      catch (IOException e) {
        e.printStackTrace();
      }
      catch (SAXException e) {
        e.printStackTrace();
      }*/
    }
  }

}