package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import javax.swing.tree.*;
import javax.swing.*;
import java.io.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;

public class ResourcesFrame extends FocusTrackingInternalFrame {

  JTree mResourceTree = new JTree();
  JScrollPane jScrollPane = new JScrollPane();
  ResourceNode mRootNode;
  HashMap fileToRRLmap = new HashMap();

  public ResourcesFrame() {
    super();
    putClientProperty("JInternalFrame.isPalette", Boolean.TRUE);
    try {
        jbInit();
    }
    catch(Exception e) {
        e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    this.setBounds(110, 510, 390, 400);
    this.setResizable(true);
    this.setIconifiable(true);
    this.setFrameIcon(null);
//    this.getContentPane().add(mNewFunctionFolderButton, BorderLayout.NORTH);
    this.getContentPane().add(jScrollPane); //, BorderLayout.CENTER);
    this.setTitle(" Stored Enclosures");

    jScrollPane.getViewport().add(mResourceTree, null);
    loadResourceTree();
    mResourceTree.setModel(new DefaultTreeModel(mRootNode));
    mResourceTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    mResourceTree.addMouseListener(new MouseAdapter() {
      public void mouseReleased(MouseEvent e)  {
          Point2D releasePoint = new Point2D.Double();
          releasePoint.setLocation(e.getX(), e.getY());
          Util.changeCoordinates(releasePoint, mResourceTree, g.sceneGraphPanel);

          ResourceNode selection = (ResourceNode)mResourceTree.getLastSelectedPathComponent();
          if (selection == null || selection.mRRL.resourceFile.isDirectory())
              return;
          g.sceneGraphPanel.releasedNewResource(releasePoint, selection.mRRL);
      }
    });
  }

  private void loadResourceTree() {
    mRootNode = new ResourceNode(g.resourceDirectory.getName(), g.resourceDirectory);
    loadDirectory(mRootNode, g.resourceDirectory);
  }

  public void reloadResourceTree() {
    fileToRRLmap.clear();
    mRootNode.removeAllChildren();
    loadDirectory(mRootNode, g.resourceDirectory);
    ((DefaultTreeModel)mResourceTree.getModel()).reload(mRootNode);
  }

  public void loadDirectory(ResourceNode parentNode, File directory) {
    String[] files = directory.list();
    File newFile;
    ResourceNode newNode;

    for (int i = 0; i < files.length; i++) {
      newFile = new File(directory.getPath() + g.FS + files[i]);
      newNode = new ResourceNode(FileUtil.stripPostfix(files[i]), newFile);
      if (! files[i].substring(0,1).equals("$"))
        parentNode.add(newNode);
      fileToRRLmap.put(newFile.getPath(), newNode.mRRL);
      if (newFile.isDirectory()) {
        loadDirectory(newNode, newFile);
      }
    }
  }

  public RRL getRRL(String relativePath) {
    RRL test = getRRL(new File(g.resourceDirectory.getPath() + g.FS + relativePath + ".fun"));
    if (test == null)
      test = getRRL(new File(g.resourceDirectory.getPath() + g.FS + relativePath + ".enc"));
    return test;
  }

  public RRL getRRL(File resourceFile) {
//    Set keys = fileToRRLmap.keySet();
//    for (Iterator i = keys.iterator(); i.hasNext();)
//      System.out.println("Key: " + i.next());
    return (RRL)fileToRRLmap.get(resourceFile.getPath());
  }

  public class ResourceNode extends DefaultMutableTreeNode {
    RRL mRRL;


    public ResourceNode (String name, File resourceFile) {
      super(name);
      mRRL = new RRL();
      mRRL.resourceFile = resourceFile;
      mRRL.isFunction = FileUtil.getPostfix(resourceFile.getName()).equals("fun");
      mRRL.isEnclosure = FileUtil.getPostfix(resourceFile.getName()).equals("enc");
    }
  }
}