package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.geom.*;

public class Rotation extends Transformation {

  private NumProp mAngleProperty;
  private double mAngle;
  private LocationProp mPivotProperty;
  private double mPivotX;
  private double mPivotY;

  public Rotation() {
  }

  public Rotation(String name, Enclosure context) {
    super(name, context);
    mEntityType = "rotation";
    addProperties();
  }

  public void setAngle(double angle) {
    mAngleProperty.value = angle;
    mAngleProperty.valueSet();
  }

  public void setPivot(double x, double y) {
    mPivotProperty.x = x;
    mPivotProperty.y = y;
    mPivotProperty.valueSet();
  }

  public void setPivot(Point2D point) {
    mPivotProperty.x = point.getX();
    mPivotProperty.y = point.getY();
    mPivotProperty.valueSet();
  }

  protected void addProperties() {
      mAngleProperty  = new NumProp();
      mAngleProperty.mName = "Angle";
      mAngleProperty.value = 0.0;
      addProperty(mAngleProperty);

      mPivotProperty = new LocationProp();
      mPivotProperty.mName = "Pivot";
      addProperty(mPivotProperty);
  }

  protected void readInputs() {
    mAngleProperty.validate();
    mAngle = mAngleProperty.value;
    mPivotProperty.validate();
    mPivotX = g.transform.xToPanel(mPivotProperty.x);
    mPivotY = g.transform.yToPanel(mPivotProperty.y);
  }

  public void run() {
    mAffineTrans.setToRotation(mAngle * Math.PI / 180.0, mPivotX, mPivotY);
  }

}