package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

public class Scale extends Transformation {

  private static final double MIN_SCALE = .0001;
  private NumProp mXScaleProperty;
  private double mXScale;
  private NumProp mYScaleProperty;
  private double mYScale;

  public Scale() {
  }

  public Scale(String name, Enclosure context) {
    super(name, context);
    mEntityType = "scale";
    addProperties();
  }


  public void setX(double x) {
    mXScaleProperty.value = x;
    mXScaleProperty.valueSet();
  }

  public void setY(double y) {
    mYScaleProperty.value = y;
    mYScaleProperty.valueSet();
  }

  protected void addProperties() {
    mXScaleProperty  = new NumProp();
    mXScaleProperty.mName = "xScale";
    mXScaleProperty.value = 1.0;
    addProperty(mXScaleProperty);

    mYScaleProperty  = new NumProp();
    mYScaleProperty.mName = "yScale";
    mYScaleProperty.value = 1.0;
    addProperty(mYScaleProperty);
  }

  protected void readInputs() {
    mXScaleProperty.validate();
    mXScale = mXScaleProperty.value;
    mYScaleProperty.validate();
    mYScale = mYScaleProperty.value;

    if (Math.abs(mXScale) < MIN_SCALE) {
      if (mXScale < 0.0)
        mXScale = -MIN_SCALE;
      else
        mXScale = MIN_SCALE;
    }

    if (Math.abs(mYScale) < MIN_SCALE) {
      if (mYScale < 0.0)
        mYScale = -MIN_SCALE;
      else
        mYScale = MIN_SCALE;
    }
  }

  public void run() {
    mAffineTrans.setToScale(mXScale, mYScale);
  }
}