package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.lang.reflect.*;

public class Scene extends Enclosure {

  private boolean mInUpdate = false;
  public Entity[] mEntityList = new Entity[0];
  private TreeSet mDrawList = new TreeSet(new LayerComparator());
  private TreeSet mHitList = new TreeSet(new LayerComparator());

  public Scene() {
    super("scene", null);
    if (g.HAS_SCENEGRAPH) {
      g.sceneGraphPanel.addNewEntityNode(this, null);
    }
  }

  public boolean isScene() {
    return true;
  }

  public void updateScene() {
    if (mInUpdate)
      return;
    mInUpdate = true;
    mDrawList.clear();
    for (int i = 0; i < mEntityList.length; i++) {
      mEntityList[i].update();
    }
    mInUpdate = false;
  }

  public void refresh() {
    g.scenePanel.repaint();
  }

  public void draw(Graphics2D graphics) {     // Scene has been updated.
    if (g.drawCycle == Integer.MAX_VALUE)
      g.drawCycle = 0;
    else
      g.drawCycle++;

    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    for (Iterator i = mDrawList.iterator(); i.hasNext();) {
      ((GraphicEntity)i.next()).draw(graphics);
    }
  }

  public void allHits(Point2D hitLocation) {
      allHits(hitLocation, null);             // Perform allHits without a filter
  }

  public void allHits(Point2D hitLocation, Class typeFilter) {
      GraphicEntity current;
      mHitList.clear();

      for (Iterator i = mDrawList.iterator(); i.hasNext();) {
          current = ((GraphicEntity)i.next());
          if (current.isVisible())
              if (typeFilter == null || typeFilter.isAssignableFrom(current.getClass()))
                  if (current.hit(hitLocation))
                      mHitList.add(current);
      }
  }

  public GraphicEntity hit(Point2D hitLocation) {
      allHits(hitLocation, Types.POINT);
      if (mHitList.size() > 0) return (GraphicEntity)mHitList.last();

      allHits(hitLocation, Types.LINE);
      if (mHitList.size() > 0) return (GraphicEntity)mHitList.last();

      allHits(hitLocation);
      if (mHitList.size() > 0) return (GraphicEntity)mHitList.last();

      return null;
  }

  public void forceButtonRepaint(Graphics2D graphics) {
  }

  public Function newFunction(Enclosure enclosure) {
    return newFunction(Types.FUNCTION, "Function", enclosure);
  }

  Function newFunction(Class functionType, String name, Enclosure enclosure) {
    String newName = enclosure.newName(name);
    Function newFunc = null;
    Object[] args = new Object[] {newName, enclosure};
    Class[]  argTypes = new Class[] {"".getClass(), Types.ENCLOSURE};
    try {
      newFunc = (Function)functionType.getConstructor(argTypes).newInstance(args);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    return newFunc;
  }

  public void addToDrawList(GraphicEntity graphicEntity) {
    mDrawList.add(graphicEntity);
  }

  public void removeFromDrawList(GraphicEntity graphicEntity) {
    mDrawList.remove(graphicEntity);
  }

  public class LayerComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      GraphicEntity g1 = (GraphicEntity)o1;
      GraphicEntity g2 = (GraphicEntity)o2;
      int layer1 = g1.getLayer();
      int layer2 = g2.getLayer();

      if (layer1 > layer2)
        return 1;
      else if (layer1 < layer2)
        return -1;
      else if (g1.mID > g2.mID)
        return 1;
      else if (g1.mID < g2.mID)
        return -1;

      return 0;
    }
  }

}