package roebling;


/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.util.*;
import java.net.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.*;

public class SceneFrame extends FocusTrackingInternalFrame {

  BorderLayout borderLayout1 = new BorderLayout();
  JToolBar mButtonBar = new JToolBar();
  JButton mArrowButton;
  JButton mPointButton;
  JButton mLineButton;
  JButton mTextButton;
  JButton mImageButton;
  JButton mButtonButton;
  JButton mPolygonButton;

  public SceneFrame() {
    g.scenePanel =  new ScenePanel();
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    setSize(490,500);
    setLocation(10, 10);
  }

  private void jbInit() throws Exception {
    this.setResizable(true);
    this.setClosable(false);
    this.setMaximizable(true);
    this.setIconifiable(true);
    this.setTitle("Scene");
    this.getContentPane().setLayout(borderLayout1);
    mButtonBar.setOrientation(JToolBar.VERTICAL);
    mButtonBar.setFloatable(false);
    this.getContentPane().add(g.scenePanel, BorderLayout.CENTER);
    this.getContentPane().add(mButtonBar, BorderLayout.WEST);
    addGraphicButtons(mButtonBar);
  }

  private void addGraphicButtons(JToolBar toolBar) {

    String jarPath = "";
    String tempStr;
    StringTokenizer pathTokens = new StringTokenizer(System.getProperty("java.class.path"), ";");
    while (pathTokens.hasMoreTokens()) {
      tempStr = pathTokens.nextToken();
      if (tempStr.indexOf("paramour.jar") != -1) jarPath = tempStr;
    }

    if (!jarPath.equals("")) {
      jarPath = "jar:file:" + jarPath + "!/";
    }
    else
      jarPath = "file:";

    try {
      mArrowButton = new JButton(new ImageIcon(new URL(jarPath + "images/arrow.gif")));
      mArrowButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/arrowHilite.gif")));
      mArrowButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/arrowHilite.gif")));
      mArrowButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
          resetButtons();
          mArrowButton.setSelected(true);
          g.scenePanel.arrowButtonHit();
        }
    });
    mArrowButton.setSelected(true);
    toolBar.add(mArrowButton);

    mPointButton = new JButton(new ImageIcon(new URL(jarPath + "images/point.gif")));
    mPointButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/pointHilite.gif")));
    mPointButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/pointHilite.gif")));
    mPointButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mPointButton.setSelected(true);
        g.scenePanel.pointButtonHit();
      }
    });
    toolBar.add(mPointButton);

    mLineButton = new JButton(new ImageIcon(new URL(jarPath + "images/line.gif")));
    mLineButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/lineHilite.gif")));
    mLineButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/lineHilite.gif")));
    mLineButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mLineButton.setSelected(true);
        g.scenePanel.lineButtonHit();
      }
    });
    toolBar.add(mLineButton);

    mPolygonButton = new JButton(new ImageIcon(new URL(jarPath + "images/polygon.gif")));
    mPolygonButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/polygonHilite.gif")));
    mPolygonButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/polygonHilite.gif")));
    mPolygonButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mPolygonButton.setSelected(true);
        g.scenePanel.polygonButtonHit();
      }
    });
    toolBar.add(mPolygonButton);

    mTextButton = new JButton(new ImageIcon(new URL(jarPath + "images/text.gif")));
    mTextButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/textHilite.gif")));
    mTextButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/textHilite.gif")));
    mTextButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mTextButton.setSelected(true);
        g.scenePanel.textButtonHit();
      }
    });
    toolBar.add(mTextButton);

    mImageButton = new JButton(new ImageIcon(new URL(jarPath + "images/image.gif")));
    mImageButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/imageHilite.gif")));
    mImageButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/imageHilite.gif")));
    mImageButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mImageButton.setSelected(true);
        g.scenePanel.imageButtonHit();
      }
    });
    toolBar.add(mImageButton);

    mButtonButton = new JButton(new ImageIcon(new URL(jarPath + "images/button.gif")));
    mButtonButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/buttonHilite.gif")));
    mButtonButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/buttonHilite.gif")));
    mButtonButton.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        resetButtons();
        mButtonButton.setSelected(true);
        g.scenePanel.buttonButtonHit();
      }
    });
    toolBar.add(mButtonButton);

/*
    segmentButton = new JButton(new ImageIcon(new URL(jarPath + "images/segment.gif")));
    segmentButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/segmentHilite.gif")));
    segmentButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/segmentHilite.gif")));
    segmentButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
            segmentButton_actionPerformed(e);
        }
    });
    toolBar.add(segmentButton);

    polygonButton = new JButton(new ImageIcon(new URL(jarPath + "images/polygon.gif")));
    polygonButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/polygonHilite.gif")));
    polygonButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/polygonHilite.gif")));
    polygonButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
            polygonButton_actionPerformed(e);
        }
    });
    toolBar.add(polygonButton);

    textButton = new JButton(new ImageIcon(new URL(jarPath + "images/text.gif")));
    textButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/textHilite.gif")));
    textButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/textHilite.gif")));
    textButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
            textButton_actionPerformed(e);
        }
    });
    toolBar.add(textButton);

    relationshipButton = new JButton(new ImageIcon(new URL(jarPath + "images/relationship.gif")));
    relationshipButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/relationshipHilite.gif")));
    relationshipButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/relationshipHilite.gif")));
    relationshipButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
            relationshipButton_actionPerformed(e);
        }
    });
    toolBar.add(relationshipButton);

    eraseButton = new JButton(new ImageIcon(new URL(jarPath + "images/erase.gif")));
    eraseButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/eraseHilite.gif")));
    eraseButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/eraseHilite.gif")));
    eraseButton.addActionListener(new ActionListener()  {
        public void actionPerformed(ActionEvent e) {
            eraseButton_actionPerformed(e);
        }
      });
      toolBar.add(eraseButton);
*/
    }
    catch (MalformedURLException e) {
      System.out.println("Cannot locate button images in file \"ParaMour.jar\"");
    }
  }

  public void selectArrow() {
    mArrowButton.setSelected(true);
  }

  public void resetButtons() {
    mArrowButton.setSelected(false);
    mPointButton.setSelected(false);
    mLineButton.setSelected(false);
    mPolygonButton.setSelected(false);
    mTextButton.setSelected(false);
    mButtonButton.setSelected(false);
  }
}