package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

public class SceneGraphFrame extends FocusTrackingInternalFrame {
    BorderLayout borderLayout1 = new BorderLayout();
    private boolean mFirstPaint = true;
    private SceneGraphFrame mSelf;

    public boolean mArrayProps = false;

    JPanel ToolPanel = new JPanel();
    BorderLayout borderLayout2 = new BorderLayout();
    JMenuBar jMenuBar1 = new JMenuBar();
    JMenu jMenu1 = new JMenu();
    JMenuItem mEmptyEnclosureMenuItem = new JMenuItem();
    JSplitPane jSplitPane1 = new JSplitPane();
    JMenuItem mEmptyFunctionMenuItem = new JMenuItem();
    JMenu jMenu2 = new JMenu();
    JMenuItem jMenuItem1 = new JMenuItem();
    JMenuItem mNewRotationMenuItem = new JMenuItem();
    JMenuItem mNewTranslationMenuItem = new JMenuItem();
    JMenuItem mNewScaleMenuItem = new JMenuItem();
    JMenuItem mEraseSceneMenuItem = new JMenuItem();

    JToolBar mButtonBar = new JToolBar();
    JButton mNumberPropertyButton;
    JButton mColorPropertyButton;
    JButton mBooleanPropertyButton;
    JButton mStringPropertyButton;
    JButton mPointPropertyButton;
    JButton mArrowToolButton;
    JButton mEraseToolButton;
    JButton mToggleArrayButton;
    JButton mCustomPropertyButton;

    public SceneGraphFrame() {
      mSelf = this;
      initialize("SceneGraph");
    }

    private void initialize(String name) {
        try {
            jbInit();
            setupCustomCursors();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        setSize(860,900);
        setLocation(500, 10);
    }
/*
    public void paintComponent(Graphics graphics) {
//        mToggleArrayButton.setSize(39, 20);
      super.paintComponent(graphics);
    }
*/

  void setupCustomCursors() {
    g.locCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/locCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "LOC_CURSOR");
    g.numCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/numCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "NUM_CURSOR");
    g.boolCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/boolCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "BOOL_CURSOR");
    g.stringCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/stringCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "STIRNG_CURSOR");
    g.colorCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/colorCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "COLOR_CURSOR");
    g.eraseCursor = Toolkit.getDefaultToolkit().createCustomCursor(new ImageIcon("images/eraseCursor.gif").getImage(),
                                                                  new java.awt.Point(5,5), "ERASE_CURSOR");
  }

    private void jbInit() throws Exception {
        this.setResizable(true);
        this.setClosable(false);
        this.setMaximizable(true);
        this.setIconifiable(true);
        this.setTitle("Scene Graph");
        this.getContentPane().setLayout(borderLayout1);

        mButtonBar.setOrientation(JToolBar.VERTICAL);
        mButtonBar.setFloatable(false);
        this.getContentPane().add(mButtonBar, BorderLayout.WEST);
        addGraphicButtons(mButtonBar);

//        ToolPanel.setLayout(borderLayout2);
        this.setJMenuBar(jMenuBar1);

        jMenu1.setText("Entities");
        mEmptyEnclosureMenuItem.setText("New Enclosure");
        mEmptyEnclosureMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                Enclosure enc = g.activeEnclosure;
                Enclosure newEnc = new Enclosure(enc.newName("Enclosure"), enc);
                EntityNode node = g.sceneGraphPanel.addNewEntityNode(newEnc, enc);
                g.sceneGraphPanel.setSelected(node);
            }
        });

        mEmptyFunctionMenuItem.setText("New Function");
        mEmptyFunctionMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                Enclosure enc = g.activeEnclosure;
                Function newFunc = g.scene.newFunction(enc);
                EntityNode node = g.sceneGraphPanel.addNewEntityNode(newFunc, enc);
                g.sceneGraphPanel.setSelected(node);
            }
        });

        mNewRotationMenuItem.setText("New Rotation");
        mNewRotationMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                Enclosure enc = g.activeEnclosure;
                Rotation newRot = new Rotation(enc.newName("Rotation"), enc);
                //enc.addTransformation(newRot);
                EntityNode node = g.sceneGraphPanel.addNewEntityNode(newRot, enc);
                g.sceneGraphPanel.setSelected(node);
            }
        });

        mNewTranslationMenuItem.setText("New Translation");
        mNewTranslationMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                Enclosure enc = g.activeEnclosure;
                Translation newTrans = new Translation(enc.newName("Translation"), enc);
                //enc.addTransformation(newTrans);
                EntityNode node = g.sceneGraphPanel.addNewEntityNode(newTrans, enc);
                g.sceneGraphPanel.setSelected(node);
            }
        });

        mNewScaleMenuItem.setText("New Scale");
        mNewScaleMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
                Enclosure enc = g.activeEnclosure;
                Scale newScale = new Scale(enc.newName("Scale"), enc);
                //enc.addTransformation(newScale);
                EntityNode node = g.sceneGraphPanel.addNewEntityNode(newScale, enc);
                g.sceneGraphPanel.setSelected(node);
            }
        });

        jMenu2.setText("Tools");
        jMenuItem1.setText("Publish stored enclosures...");
        jMenuItem1.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
              if (g.publisher == null)
                g.publisher = new Publisher();
              g.publisher.publishEnclosures();
            }
        });

        mEraseSceneMenuItem.setText("Erase scene...");
        mEraseSceneMenuItem.addActionListener(new ActionListener()  {
            public void actionPerformed(ActionEvent e) {
              if (JOptionPane.showInternalConfirmDialog(mSelf,
                   "Erase entire scene?",
                   "Confirm erase",
                   JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
                return;
              g.sceneGraphPanel.eraseScene();
            }
        });

        this.getContentPane().add(g.sceneGraphPanel, BorderLayout.CENTER);

        jMenuBar1.add(jMenu1);
        jMenuBar1.add(jMenu2);
        jMenu1.add(mEmptyFunctionMenuItem);
        jMenu1.add(mEmptyEnclosureMenuItem);
        jMenu1.add(mNewRotationMenuItem);
        jMenu1.add(mNewTranslationMenuItem);
        jMenu1.add(mNewScaleMenuItem);
        jMenu2.add(jMenuItem1);
        jMenu2.add(mEraseSceneMenuItem);
    }

    private void addGraphicButtons(JToolBar toolBar) {
        String jarPath = "";
        String tempStr;
        StringTokenizer pathTokens = new StringTokenizer(System.getProperty("java.class.path"), ";");
        while (pathTokens.hasMoreTokens()) {
            tempStr = pathTokens.nextToken();
            if (tempStr.indexOf("paramour.jar") != -1) jarPath = tempStr;
        }

        if (!jarPath.equals("")) {
            jarPath = "jar:file:" + jarPath + "!/";
        }
        else
            jarPath = "file:";

        try {
            mArrowToolButton = new JButton(new ImageIcon(new URL(jarPath + "images/arrow.gif")));
            mArrowToolButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/arrowHilite.gif")));
            mArrowToolButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/arrowHilite.gif")));
            mArrowToolButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.ARROW_TOOL);
                }
            });
            mArrowToolButton.setSelected(true);
            toolBar.add(mArrowToolButton);

            mEraseToolButton = new JButton(new ImageIcon(new URL(jarPath + "images/erase.gif")));
            mEraseToolButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/eraseHilite.gif")));
            mEraseToolButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/eraseHilite.gif")));
            mEraseToolButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.ERASE_TOOL);
                }
            });
            mEraseToolButton.setSelected(false);
            toolBar.add(mEraseToolButton);

            mToggleArrayButton = new JButton(new ImageIcon(new URL(jarPath + "images/toArray.gif")));
            mToggleArrayButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/toArrayHilite.gif")));
            mToggleArrayButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/toArrayHilite.gif")));
            mToggleArrayButton.addMouseListener(new MouseAdapter()  {
              public void mousePressed(MouseEvent e) {
                    mToggleArrayButton.setSelected(!mToggleArrayButton.isSelected());
                }
              public void mouseReleased(MouseEvent e)  {
                mArrayProps = mToggleArrayButton.isSelected();
              }
            });

            mToggleArrayButton.setSelected(false);

            toolBar.add(mToggleArrayButton);
            //mToggleArrayButton.setSize(34, 16);

            mNumberPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/numberProperty.gif")));
            mNumberPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/numberPropertyHilite.gif")));
            mNumberPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/numberPropertyHilite.gif")));
            mNumberPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.NUMBER_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mNumberPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mNumberPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.NUMBER);
                }
            });
            mNumberPropertyButton.setSelected(false);
            toolBar.add(mNumberPropertyButton);


            mBooleanPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/booleanProperty.gif")));
            mBooleanPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/booleanPropertyHilite.gif")));
            mBooleanPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/booleanPropertyHilite.gif")));
            mBooleanPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.BOOLEAN_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mBooleanPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mBooleanPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.BOOLEAN);
                }
            });
            mBooleanPropertyButton.setSelected(false);
            toolBar.add(mBooleanPropertyButton);


            mPointPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/pointProperty.gif")));
            mPointPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/pointPropertyHilite.gif")));
            mPointPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/pointPropertyHilite.gif")));
            mPointPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.POINT_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mPointPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mPointPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.PAIR);
                }
            });
            mPointPropertyButton.setSelected(false);
            toolBar.add(mPointPropertyButton);


            mColorPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/colorProperty.gif")));
            mColorPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/colorPropertyHilite.gif")));
            mColorPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/colorPropertyHilite.gif")));
            mColorPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.COLOR_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mColorPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mColorPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.COLOR);
                }
            });
            mColorPropertyButton.setSelected(false);
            toolBar.add(mColorPropertyButton);


            mStringPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/stringProperty.gif")));
            mStringPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/stringPropertyHilite.gif")));
            mStringPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/stringPropertyHilite.gif")));
            mStringPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.STRING_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mStringPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mStringPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.STRING);
                }
            });
            mStringPropertyButton.setSelected(false);
            toolBar.add(mStringPropertyButton);


            mCustomPropertyButton = new JButton(new ImageIcon(new URL(jarPath + "images/customProperty.gif")));
            mCustomPropertyButton.setPressedIcon(new ImageIcon(new URL(jarPath + "images/customPropertyHilite.gif")));
            mCustomPropertyButton.setSelectedIcon(new ImageIcon(new URL(jarPath + "images/customPropertyHilite.gif")));
            mCustomPropertyButton.addMouseListener(new MouseAdapter()  {
                public void mousePressed(MouseEvent e) {
                    g.sceneGraphPanel.setTool(SceneGraphPanel.CUSTOM_PROPERTY_TOOL);
                }
                public void mouseReleased(MouseEvent e)  {
//                    mStringPropertyButton.setSelected(false);
//                    Point2D releasePoint = new Point2D.Double();
//                    releasePoint.setLocation(e.getX(), e.getY());
//                    Util.changeCoordinates(releasePoint, mStringPropertyButton, mSceneGraph.getTopMaximizedEnclosureNode());
//                    mSceneGraph.releasedNewProperty(releasePoint, Types.STRING);
                }
            });
            mCustomPropertyButton.setSelected(false);
//            toolBar.add(mCustomPropertyButton);
        }
        catch (MalformedURLException e) {
            System.out.println("Cannot locate button images in file \"ParaMour.jar\"");
        }
    }

    private void resetTools() {
        mArrowToolButton.setSelected(false);
        mEraseToolButton.setSelected(false);
        mNumberPropertyButton.setSelected(false);
        mBooleanPropertyButton.setSelected(false);
        mPointPropertyButton.setSelected(false);
        mColorPropertyButton.setSelected(false);
        mStringPropertyButton.setSelected(false);
        mCustomPropertyButton.setSelected(false);
    }

    public void setTool(int tool) {
        resetTools();
        if (tool == SceneGraphPanel.ERASE_TOOL) {
            mEraseToolButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.ARROW_TOOL) {
            mArrowToolButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.NUMBER_PROPERTY_TOOL) {
            mNumberPropertyButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.BOOLEAN_PROPERTY_TOOL) {
            mBooleanPropertyButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.POINT_PROPERTY_TOOL) {
            mPointPropertyButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.COLOR_PROPERTY_TOOL) {
            mColorPropertyButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.STRING_PROPERTY_TOOL) {
            mStringPropertyButton.setSelected(true);
        }
        else if (tool == SceneGraphPanel.CUSTOM_PROPERTY_TOOL) {
            mCustomPropertyButton.setSelected(true);
        }    }

}