package roebling;

import javax.swing.JPanel;

/**
 * Title:        Roebling
 * Description:  Parametric design tool.
 * Copyright:    Copyright (c) 2001
 * Company:      NewBricks
 * @author Simon Greenwold
 * @version 1.0
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.io.*;
import javax.swing.*;

public class SceneGraphPanel extends JPanel {

  HashMap mEntityIDMap = new HashMap();
  HashMap mPropertyIDtoInputNode = new HashMap();
  HashMap mPropertyIDtoOutputNode = new HashMap();

  private Stack mMaximizedEnclosures = new Stack();

  public static final int ARROW_TOOL = 0;
  public static final int ERASE_TOOL = 1;
  public static final int NUMBER_PROPERTY_TOOL = 2;
  public static final int BOOLEAN_PROPERTY_TOOL = 3;
  public static final int POINT_PROPERTY_TOOL = 4;
  public static final int COLOR_PROPERTY_TOOL = 5;
  public static final int STRING_PROPERTY_TOOL = 6;
  public static final int CUSTOM_PROPERTY_TOOL = 7;
  private int mTool = ARROW_TOOL;
  public int mPropertyToolType = 0;


  BorderLayout borderLayout1 = new BorderLayout();
  private EntityNode mSelectedEntity = null;
  private boolean mConnectionsSelectable = false;
  private PropertyNode mActivePropertyNode;

  public SceneGraphPanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

    /*
    public boolean executeCommand(Command command) {
        if (command.getCode() == Command.ADD_ENTITY) {
            addNewEntityNode((Entity)command.param1(), (Enclosure)command.param2());
        }
        else if (command.getCode() == Command.SET_ACTIVE_ENTITY_NODE) {
            selectNode((EntityNode)command.param1());
        }
        else if (command.getCode() == Command.ADD_PROPERTY) {
            addNewPropertyNode(getEntityNode((Entity)command.param1()),
                               (Property)command.param2(),
                               ((Integer)command.param3()).intValue());
        }
        else if (command.getCode() == Command.DELETE_ENTITY) {
            deleteEntity((Entity)command.param1());
        }
        else if (command.getCode() == Command.SET_ENTITY_NAME) {
            getEntityNode((Entity)command.param1()).setName((String)command.param2());
        }
        else if (command.getCode() == Command.CLEAR_CONNECTIONS_PROPERTY) {
            clearConnectionsProperty((Property)command.param1());
        }
        else if (command.getCode() == Command.REPLACE_PROPERTY) {
            replaceProperty((Property)command.param1(), (Property)command.param2());
        }
        return true;
    }
    */

  private void replaceProperty(Property oldProperty, Property newProperty) {
    PropertyNode node;
    node = getInputNode(oldProperty);
    if (node != null) {
      mPropertyIDtoInputNode.remove(new Integer(oldProperty.mID));
      setInputPropertyNode(newProperty, node);
      node.setProperty(newProperty);
    }

    node = getOutputNode(oldProperty);
    if (node != null) {
      mPropertyIDtoOutputNode.remove(new Integer(oldProperty.mID));
      setOutputPropertyNode(newProperty, node);
      node.setProperty(newProperty);
    }
  }

  private void deleteEntity(Entity entity) {
    EntityNode graphNode = getEntityNode(entity);
    graphNode.getEnclosureNode().removeGraphNode(graphNode);
  }

  private void clearConnectionsProperty(Property property) {
    PropertyNode testNode;
    testNode = getInputNode(property);
    if (testNode != null)
      testNode.clearConnections();
    testNode = getOutputNode(property);
    if (testNode != null)
      testNode.clearConnections();
  }
/*
  public void addNewPropertyNode(EntityNode entityNode, Property property, int index) {
    entityNode.addProperty(property, index);
  }*/

  public EntityNode getEntityNode(Entity entity) {
    return (EntityNode)mEntityIDMap.get(new Integer(entity.mID));
  }

  public PropertyNode getInputNode(Property property) {
    return (PropertyNode)mPropertyIDtoInputNode.get(new Integer(property.mID));
  }

  public void setInputPropertyNode(Property property, PropertyNode node) {
    Integer ID = new Integer(property.mID);
    mPropertyIDtoInputNode.remove(ID);
    mPropertyIDtoInputNode.put(ID, node);
  }

  public void setOutputPropertyNode(Property property, PropertyNode node) {
    Integer ID = new Integer(property.mID);
    mPropertyIDtoOutputNode.remove(ID);
    mPropertyIDtoOutputNode.put(ID, node);
  }

  public PropertyNode getOutputNode(Property property) {
    return (PropertyNode)mPropertyIDtoOutputNode.get(new Integer(property.mID));
  }

  public void eraseScene() {
    EnclosureNode sceneNode = (EnclosureNode)getEntityNode(g.scene);
    EntityNode[] nodeList = (EntityNode[])sceneNode.mDirectory.clone();
    for (int i = 0; i < nodeList.length; i++) {
      nodeList[i].delete();
    }
  }

  public EntityNode addNewEntityNode(Entity entity, Enclosure context) {
    EntityNode entityNode;
    EnclosureNode enclosureNode = null;

    if (context != null) {
      enclosureNode = (EnclosureNode)getEntityNode(context);
      if (enclosureNode == null || !enclosureNode.isRevealed())
        return null;
    }

    if (entity.isEnclosure()) {
      entityNode = new EnclosureNode();
    }
    else if(entity.isFunction()) {
      entityNode = new FunctionNode();
    }
    else if (entity.isTransformation()) {
      entityNode = new TransformationNode();
    }
    else {
      entityNode = new EntityNode();
    }
    mEntityIDMap.put(new Integer(entity.mID), entityNode);
    entityNode.setEntity(entity);
    if (context == null) {
      setScene((EnclosureNode)entityNode);
    }
    else {
      entityNode.setEnclosureNode(enclosureNode);
      enclosureNode.addGraphNode(entityNode);

      if (entity.mNodeHints != null) {
        entityNode.setLocation(entity.mNodeHints.left, entity.mNodeHints.top);
        entityNode.setSize(entity.mNodeHints.width, entity.mNodeHints.height);
        if (entity.isEnclosure()) {
          ((EnclosureNode)entityNode).setIsRevealed(entity.mNodeHints.revealed);
        }
      }
    }

    entityNode.resized();
    if (entityNode.isTransformationNode()) {
      entityNode.mEnclosureNode.transformationListChanged();
    }
    return entityNode;
  }

  public void setScene(EnclosureNode sceneNode) {
    mMaximizedEnclosures.clear();
    maximizeEnclosureNode(sceneNode);
  }

  public void setActivePropertyNode(PropertyNode propertyNode) {
    mActivePropertyNode = propertyNode;
  }

  public PropertyNode getActivePropertyNode() {
    return mActivePropertyNode;
  }

  public void setConnectionsSelectable(boolean connectionsSelectable) {
    mConnectionsSelectable = connectionsSelectable;
  }

  public boolean connectionsSelectable() {
    return mConnectionsSelectable;
  }

  public void setSelected(EntityNode newSelected) {
//    if (newSelected == mSelectedEntity)
//      return;
    if (mSelectedEntity != null) {
      mSelectedEntity.setSelected(false);
    }
    mSelectedEntity = newSelected;
    g.selectedEntity = mSelectedEntity.mEntity;
    if (g.selectedEntity.isEnclosure()) {
      Enclosure enclosureAbove = (Enclosure)g.selectedEntity;
      while ( !((EnclosureNode)getEntityNode(enclosureAbove)).isRevealed() &&
              enclosureAbove.mEnclosure != null) {
        enclosureAbove = enclosureAbove.mEnclosure;
      }
      g.activeEnclosure = enclosureAbove;
    }
    else {
      g.activeEnclosure = g.selectedEntity.mEnclosure;
    }

    if (newSelected != null) {// && mEditingPanel != null) {
      bringToFront(newSelected);
      newSelected.setSelected(true);
//            mEditingPanel.setEntityName(newSelected.getEntity().getName());
    }
  }

  // Modify this to reveal the hidden (maybe)
  public void bringToFront(EntityNode entityNode) {
    if (entityNode.getEnclosureNode() == null)
      return;

    EntityNode toFront = entityNode;
    EnclosureNode context = toFront.getEnclosureNode();
    while (context != null && toFront != getTopMaximizedEnclosureNode()) {
      context.bringToFront(toFront);
      toFront = context;
      context = context.getEnclosureNode();
    }
  }

  public Enclosure getActiveEnclosure() {
    if (mSelectedEntity == null)
      return null;

    if (! mSelectedEntity.isEnclosureNode() || !((EnclosureNode)mSelectedEntity).isRevealed() )
      return (Enclosure)mSelectedEntity.getEnclosureNode().mEntity;
    else
      return (Enclosure)mSelectedEntity.mEntity;
  }

  public Entity getActiveEntity() {
    if (mSelectedEntity == null)
      return null;
    return mSelectedEntity.mEntity;
  }

  public void setTool(int tool) {
    mTool = tool;
    g.sceneGraphFrame.setTool(mTool);

    if (mTool == ERASE_TOOL) {
      setConnectionsSelectable(true);
    }
    else {
      setConnectionsSelectable(false);
    }

    if (mTool == NUMBER_PROPERTY_TOOL)
      mPropertyToolType = Property.NUMBER;
    else if (mTool == BOOLEAN_PROPERTY_TOOL)
      mPropertyToolType = Property.BOOLEAN;
    else if (mTool == STRING_PROPERTY_TOOL)
      mPropertyToolType = Property.STRING;
    else if (mTool == POINT_PROPERTY_TOOL)
      mPropertyToolType = Property.LOCATION;
    else if (mTool == COLOR_PROPERTY_TOOL)
      mPropertyToolType = Property.COLOR;
    else if (mTool == CUSTOM_PROPERTY_TOOL)
      mPropertyToolType = Property.CUSTOM;

    cursorChange();
  }

  private void cursorChange() {
    if (mTool == NUMBER_PROPERTY_TOOL) {

      setCursor(g.numCursor);
    }
    else if (mTool == POINT_PROPERTY_TOOL) {
      setCursor(g.locCursor);
    }
    else if (mTool == STRING_PROPERTY_TOOL) {
      setCursor(g.stringCursor);
    }
    else if (mTool == BOOLEAN_PROPERTY_TOOL) {
      setCursor(g.boolCursor);
    }
    else if (mTool == COLOR_PROPERTY_TOOL) {
      setCursor(g.colorCursor);
    }
    else if (mTool == ERASE_TOOL) {
      setCursor(g.eraseCursor);
    }
    else
      setCursor(g.defaultCursor);
  }

  public int getTool() {
      return mTool;
  }

    public void unMaximizeTopNode() {
        if (mMaximizedEnclosures == null || mMaximizedEnclosures.size() == 1)
            return;

        EnclosureNode currentMax = (EnclosureNode)mMaximizedEnclosures.pop();
        EnclosureNode newMax = getTopMaximizedEnclosureNode();
        remove(currentMax);

        currentMax.getEnclosureNode().getInterior().add(currentMax);
        currentMax.setBounds(currentMax.getUnMaximizedBounds());
        setSelected(currentMax);


        add(newMax, BorderLayout.CENTER);
        newMax.setBounds(getBounds());
        newMax.resized();

        revalidate();

        currentMax.setIsMaximized(false);
        currentMax.resized();

        newMax.invalidate();
        newMax.repaint();
    }

    public void maximizeEnclosureNode(EnclosureNode enclosureNode) {
        EnclosureNode currentMax = getTopMaximizedEnclosureNode();
        if (currentMax == enclosureNode)
            return;

        if (currentMax != null) {
            remove(currentMax);
        }

        enclosureNode.setIsMaximized(true);
        mMaximizedEnclosures.push(enclosureNode);
        add(enclosureNode, BorderLayout.CENTER);
        enclosureNode.setBounds(getBounds());
        enclosureNode.resized();
        revalidate();
        setSelected(enclosureNode);
    }


    public EnclosureNode getTopMaximizedEnclosureNode() {
        if (mMaximizedEnclosures.empty())
            return null;
        return (EnclosureNode)mMaximizedEnclosures.peek();
    }

    private void jbInit() throws Exception {
        this.setLayout(borderLayout1);

        this.addMouseListener(new MouseAdapter() {
          public void mouseEntered(MouseEvent e) {
            cursorChange();
          }
        });
    }

    // Tells you if they're connectable and if so, puts them in the proper parent-child position
    // -1: Parent -> Child = prop2 -> prop1
    // 0: NOT CONNECTABLE
    // 1: Parent -> Child = prop1 -> prop2
    public int connectable(PropertyNode prop1, PropertyNode prop2) {
      PropertyNode freeNode1 = null;
      PropertyNode freeNode2 = null;
      int multiplier = 1;

        if (! prop1.getProperty().isCompatable(prop2.getProperty()))
            return 0;

        if (prop1.mProperty.isFreeFloating()) {
            freeNode1 = prop1;
            freeNode2 = prop2;
        }
        else if (prop2.mProperty.isFreeFloating()) {
          if (! prop1.mProperty.isFreeFloating()) {
            freeNode1 = prop2;
            freeNode2 = prop1;
            multiplier = -1;
          }
        }


        if (freeNode1 != null) {
          if (freeNode1.mEntityNode.equals(freeNode2.mEntityNode)) {
            if (freeNode2.mProperty.isFreeFloating()) {   // Hard to determine which one is parent, which child
              if (freeNode2.mProperty.mParent != null) {
                return -1 * multiplier;
              }
              else {
                return 1 * multiplier;
              }
            }
            else {    // Not free-floating, on the same entityNode
              if (freeNode2.isEnclosureIn()) {
                return -1 * multiplier;
              }
              else if (freeNode2.isEnclosureOut()) {
                return 1 * multiplier;
              }
              else
                return 0;
            }
          }
          else if (freeNode2.mEntityNode.mEnclosureNode.equals(freeNode1.mEntityNode)) {
            if (freeNode2.isEnclosureIn()) {
              return 1 * multiplier;
            }
            else if (freeNode2.isEnclosureOut()) {
              return -1 * multiplier;
            }
            else if (freeNode2.isInput()) {
              return 1 * multiplier;
            }
            else {
              return -1 * multiplier;
            }
          }
          else {
            return 0;
          }
        }

        if (prop1.getEntityNode() == prop2.getEntityNode())
            return 0;

        if (prop1.isEnclosureIn()) {
            if (prop2.getEnclosureNode() == prop1.getEntityNode()) {    // Connecting from inside self
                if (prop2.isOutput() && !prop2.isEnclosureIn())
                    return 0;
                return 1;
            }
            else {          // Connecting from outside self
                if (prop2.getEnclosureNode() != prop1.getEnclosureNode()) // Different enclosures
                    return 0;

                if (prop2.isInput() && !prop2.isEnclosureOut())
                    return 0;
                return -1;
            }
        }
        else if (prop1.isEnclosureOut()) {
            if (prop2.getEnclosureNode() == prop1.getEntityNode()) {    // Connecting from inside self
                if (prop2.isInput() && !prop2.isEnclosureOut())
                    return 0;
                return -1;
            }
            else {          // Connecting to outside self
                if (prop2.isEnclosureOut() && prop1.getEnclosureNode() == prop2.getEntityNode())
                    return 1;

                if (prop2.getEnclosureNode() != prop1.getEnclosureNode()) // Different enclosures
                    return 0;

                if (prop2.isOutput() && !prop2.isEnclosureIn())
                    return 0;
                return 1;
            }
        }
                                                        // Prop1 is not an enclosure property
        else if (prop2.isEnclosureIn()) {
            if (prop1.getEnclosureNode() == prop2.getEntityNode()) {    // Connecting from inside self
                if (prop1.isOutput())
                    return 0;
                return -1;
            }
            else {          // Connecting from outside self
                if (prop1.getEnclosureNode() != prop2.getEnclosureNode()) // Different enclosures
                    return 0;

                if (prop1.isInput())
                    return 0;
                return 1;
            }
        }
        else if (prop2.isEnclosureOut()) {
            if (prop1.getEnclosureNode() == prop2.getEntityNode()) {    // Connecting from inside self
                if (prop1.isInput())
                    return 0;
                return 1;
            }
            else {          // Connecting from outside self
                if (prop1.getEnclosureNode() != prop2.getEnclosureNode()) // Different enclosures
                    return 0;

                if (prop1.isOutput())
                    return 0;
                return -1;
            }
        }


        else {                                       // Neither one is an enclosure property
            if (prop1.getEnclosureNode() != prop2.getEnclosureNode())   // Different enclosures
                return 0;

            if (prop1.isInput()) {
                if (prop2.isInput())
                    return 0;
                return -1;
            }
            else if (prop1.isOutput()) {
                if (prop2.isOutput())
                    return 0;
                return 1;
            }
        }
        return 0;
    }

    public void propertyNodeDragReleased(MouseEvent e) {
        Point2D releasedPoint = new Point2D.Double(e.getX(), e.getY());
        EnclosureNode maximizedEnclosure = getTopMaximizedEnclosureNode();
        GraphNode hitComponent;

        Util.changeCoordinates(releasedPoint, e.getComponent(), maximizedEnclosure);
        hitComponent = graphNodeHit((int)releasedPoint.getX(), (int)releasedPoint.getY());
        PropertyNode startNode = (PropertyNode)e.getComponent();

        if (hitComponent == null || !Types.PROPERTY_NODE.isInstance(hitComponent)) {
          EntityNode ownerNode = g.sceneGraphPanel.getEntityNode(startNode.mProperty.mOwner);
          if (ownerNode == null || ownerNode.mEditingProperty)
            return;

          if (startNode.mProperty.isFreeFloating()) {
            Point2D moveToPoint = new Point2D.Double(e.getX(), e.getY());
            Util.changeCoordinates(moveToPoint, e.getComponent(), ownerNode);
            startNode.mFreeFloatingLocation.setLocation(moveToPoint.getX(), moveToPoint.getY());
            startNode.setLocation((int)moveToPoint.getX(), (int)moveToPoint.getY());
          }
        }
        else {
            if (Types.PROPERTY_NODE.isInstance(hitComponent)) {     // Hit another property node
                PropertyNode releasedNode = (PropertyNode)hitComponent;
                int connectable = connectable(startNode, releasedNode);
                if (connectable != 0) {
                    PropertyNode parent = null;
                    PropertyNode child = null;

                    if (connectable == 1) {
                        parent = startNode;
                        child = releasedNode;
                    }
                    else {
                        parent = releasedNode;
                        child = startNode;
                    }
                    if (Property.newRelationship(parent.getProperty(), child.getProperty())) {
                      newConnection(parent, child);
                      g.scene.refresh();
                    }
                }
            }
        }
    }

    public void newConnection(PropertyNode parent, PropertyNode child) {
      child.newParent(parent);
      if (parent.mProperty.isFreeFloating()) {
        parent.mEnclosureNode.newConnection(parent, child);
        parent.mEnclosureNode.repaint();
      }
      else if (child.mProperty.isFreeFloating()) {
        child.mEnclosureNode.newConnection(parent, child);
        child.mEnclosureNode.repaint();
      }
      else if (child.isEnclosureOut()) {
          parent.getEnclosureNode().newConnection(parent, child);
          parent.getEnclosureNode().repaint();
      }
      else {
          child.getEnclosureNode().newConnection(parent, child);
          child.getEnclosureNode().repaint();
      }
//                        EditorFrame.getInstance().setUpEditor(mActivePropertyNode);


    }

    public GraphNode graphNodeHit(int x, int y) {
        Component base = getTopMaximizedEnclosureNode().findComponentAt(x, y);
        while (base != null && !Types.GRAPH_NODE.isInstance(base)) {
            try {
                base = base.getParent();
            }
            catch (Exception e) {
                return null;
            }
        }
        try {
            return (GraphNode)base;
        }
        catch (Exception e) {
            return null;
        }
    }

    public EntityNode entityNodeHit(int x, int y) {
        Component base = getTopMaximizedEnclosureNode().findComponentAt(x, y);
        while (base != null && !Types.ENTITY_NODE.isInstance(base)) {
            try {
                base = base.getParent();
            }
            catch (Exception e) {
                return null;
            }
        }
        try {
            return (EntityNode)base;
        }
        catch (Exception e) {
            return null;
        }
    }
/*
    public EntityNode instantiateResource(RRL rrl, Enclosure enclosure) {
      if (rrl.isFunction) {                 // InstantiateFunction
        if (rrl.functionClass == null) {
          File classFile = FileUtil.unzip(rrl.resourceFile.getPath(), g.tempDirectory.getPath());
//          g.tr("Class name: " + classFile.getName());
          rrl.functionClass = Compiler.loadFunction(FileUtil.stripPostfix(classFile.getName()));
        }
        Function newFunc = g.scene.newFunction(rrl.functionClass,
                                               FileUtil.stripPostfix(rrl.resourceFile.getName()),
                                               enclosure);
        newFunc.mIsInstance = true;
        newFunc.mRRL = rrl;
        EntityNode node = g.sceneGraphPanel.addNewEntityNode(newFunc, enclosure);
        node.setIsInstance(true);
        g.sceneGraphPanel.setSelected(node);
        return node;
      }
      else if (rrl.isEnclosure) {
        if (rrl.expandedEnclosureDirectory == null) {
          rrl.expandedEnclosureDirectory = new File(g.tempDirectory.getPath() + g.FS + rrl.relativePath());
          FileUtil.unzip(rrl.resourceFile.getPath(), rrl.expandedEnclosureDirectory.getPath());
        }
        Enclosure restored = enclosure.restore(rrl.expandedEnclosureDirectory);
        if (restored != null) {
          restored.mIsInstance = true;
          restored.mRRL = rrl;
          EnclosureNode restoredNode = (EnclosureNode)getEntityNode(restored);
          restoredNode.setIsRevealed(false);
          restoredNode.setIsInstance(true);
        }
      }

      return null;
    }
*/
    public void releasedNewResource(Point2D releasePoint, RRL rrl) {
        EntityNode hitNode = entityNodeHit((int)releasePoint.getX(), (int)releasePoint.getY());
        if (hitNode == null)
            return;

        Enclosure enclosure;
        Point2D localPoint = new Point2D.Double();
        localPoint.setLocation(releasePoint);


        if (hitNode.isEnclosureNode()) {
            enclosure = (Enclosure)hitNode.mEntity;
        }
        else {
            enclosure = hitNode.mEntity.mEnclosure;
        }

        Util.changeCoordinates(localPoint, getTopMaximizedEnclosureNode(), ((EnclosureNode)getEntityNode(enclosure)).getInterior());

        String newName = enclosure.newName(rrl.resourceFile.getName());

        Entity newEntity = enclosure.instantiateResource(rrl);

        if (newEntity == null)
          return;

        newEntity.mIsInstance = true;
        newEntity.mRRL = rrl;
        EntityNode newNode = g.sceneGraphPanel.addNewEntityNode(newEntity, enclosure);
        newNode.setIsInstance(true);
        //if (newNode.isEnclosureNode()) {
        //  ((EnclosureNode)newNode).setIsRevealed(false);
        //}

        //if (newNode != null) {
          setSelected(newNode);
        //  newNode.setIsInstance(true);
          if (newNode.isEnclosureNode()) {
            ((EnclosureNode)newNode).setIsRevealed(false);
          }
          newNode.setLocation((int)localPoint.getX(), (int)localPoint.getY());
        //}

        g.scene.refresh();

/*
        Function newFunction;
        try {
            newFunction = (Function)rrl.functionClass.newInstance();
        }
        catch (Exception e) {
            return;
        }
        newFunction.mName = newName;
        newFunction.mEnclosure = enclosure;

//        newFunction.fixPropertyIDs();
//        newFunction.getSceneManager().processCommand(Command.ADD_ENTITY, newFunction, enclosure);
        //newFunction.createPropertyNodes();
        enclosure.addEntity(newFunction);
*/


    }


/*    public void releasedNewFunction(Point2D releasePoint, Class functionClass, String type) {
        EntityNode hitNode = entityNodeHit((int)releasePoint.getX(), (int)releasePoint.getY());
        if (hitNode == null)
            return;

        Enclosure enclosure;
        Point2D localPoint = new Point2D.Double();
        localPoint.setLocation(releasePoint);


        if (hitNode.isEnclosureNode()) {
            enclosure = (Enclosure)hitNode.mEntity;
        }
        else {
            enclosure = hitNode.mEntity.mEnclosure;
        }

        Util.changeCoordinates(localPoint, getTopMaximizedEnclosureNode(), ((EnclosureNode)getEntityNode(enclosure)).getInterior());

        String newName = enclosure.newName(type);
        Function newFunction;
        try {
            newFunction = (Function)functionClass.newInstance();
        }
        catch (Exception e) {
            return;
        }
        newFunction.mName = newName;
        newFunction.mEnclosure = enclosure;

//        newFunction.fixPropertyIDs();
//        newFunction.getSceneManager().processCommand(Command.ADD_ENTITY, newFunction, enclosure);
        //newFunction.createPropertyNodes();
        enclosure.addEntity(newFunction);


        getEntityNode(newFunction).setLocation((int)localPoint.getX(), (int)localPoint.getY());
    }
/*
    //User dragged a new property onto the scene
    public void releasedNewProperty(Point2D releasePoint) {
        EntityNode hitNode = entityNodeHit((int)releasePoint.getX(), (int)releasePoint.getY());

        if (hitNode == null)
            return;
        Entity hitEntity = hitNode.getEntity();
        Property newProp;
        String propName = "";
        Point2D localPoint = new Point2D.Double();
        localPoint.setLocation(releasePoint);

        propName = hitEntity.getNewPropertyName(g.propertyTypes[mPropertyToolType].mName);
        newProp = Property.newProperty(propName, mPropertyToolType);

        Util.changeCoordinates(localPoint, getTopMaximizedEnclosureNode(), hitNode);
        int width = hitNode.getWidth();


        if (hitEntity.isEnclosure()) {
            newProp.setIsOutput(true);
            newProp.setIsInput(true);

            if (Util.near(0.0, localPoint.getX(), width / 2)) {
                newProp.setIsEnclosureIn(true);
            }
            else {
                newProp.setIsEnclosureOut(true);
            }
        }
        else {
            if (Util.near(0.0, localPoint.getX(), PROPERTY_FUNCTION_SIDE_AMOUNT)) {
                newProp.setIsOutput(false);
            }
            else if (Util.near(width, localPoint.getX(), PROPERTY_FUNCTION_SIDE_AMOUNT)) {
                newProp.setIsInput(false);
            }
        }

        hitEntity.addProperty(newProp);
    }*/

    /*
    public void releasedNewEnclosure(Point2D releasePoint, String enclosureName) {
        EntityNode hitNode = entityNodeHit((int)releasePoint.getX(), (int)releasePoint.getY());
        if (hitNode == null)
            return;

        Enclosure enclosure;
        Point2D localPoint = new Point2D.Double();
        localPoint.setLocation(releasePoint);


        if (hitNode.isEnclosureNode()) {
            enclosure = (Enclosure)hitNode.getEntity();
        }
        else {
            enclosure = hitNode.getEntity().mEnclosure;
        }

        Util.changeCoordinates(localPoint, getTopMaximizedEnclosureNode(), ((EnclosureNode)getEntityNode(enclosure)).getInterior());

        Enclosure enc = new Enclosure(new File(FileManager.getDefaultEnclosureDirectory().getPath() + FileManager.FILE_SEPARATOR +
                                       enclosureName), enclosure, false);
    }*/

}