package roebling;

import javax.swing.JPanel;

import java.awt.Color;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class ScenePanel extends JPanel implements MouseListener, MouseMotionListener {

    private boolean mFirstDraw = true;
    private Point2D mMousePressedPoint = new Point2D.Double();
    private Point2D mMouseDelta = new Point2D.Double();
    private Point2D mMouseDraggedTo = new Point2D.Double();
    private boolean mMouseDown = false;
    private Point2D mTransDrag = new Point2D.Double();

    private Point2D mPointDrawingTrackPoint = new Point2D.Double();
    private Ellipse2D mPointDrawingPaintPoint = new Ellipse2D.Double();

    private Line2D mLineDrawingTrackingLine = new  Line2D.Double();
    private Point2D mLineStartPoint = new Point2D.Double();
    private Point2D mLineEndPoint = new Point2D.Double();
    private BasicStroke mLineDrawingStroke = new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
//    private java.awt.Polygon mPolygonDrawingTrackingPoly = new java.awt.Polygon();
    private Vector  mPolygonPoints = new Vector();

    private GraphicEntity mSelection;
    private GraphicEntity mLastSelection;

    private final static int ARROW_MODE = 0;
    private int POINT_MODE = 1;
    private int LINE_MODE = 2;
    private int TEXT_MODE = 3;
    private int BUTTON_MODE = 4;
    private int POLYGON_MODE = 5;
    private int IMAGE_MODE = 6;
    private final static int NONE = 0;

    private int mMode = ARROW_MODE;
    private int mSubMode = NONE;
    private final static int POINT_BEGUN_SUBMODE = 1;
    private final static int LINE_BEGUN_SUBMODE = 2;
    private final static int TEXT_BEGUN_SUBMODE = 3;
    private final static int POLYGON_BEGUN_SUBMODE = 4;

    private boolean mInRepaint = false;

    public ScenePanel() {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        g.transform = new ScenePanelTransform(this);
        addMouseListener(this);
        addMouseMotionListener(this);
    }

    private void jbInit() throws Exception {
        this.setLayout(null);
        this.setBackground(Color.white);
    }

    public void paintComponent(Graphics graphics) {
      if (mInRepaint)
        return;
      mInRepaint = true;
        Graphics2D g2 = (Graphics2D)graphics;
        super.paintComponent(graphics); //paint background
        int pointSize = roebling.Point.DEFAULT_SIZE;

        if (mFirstDraw) {
            g.transform.centerOrigin();
            mFirstDraw = false;
        }

        g.scene.updateScene();
        g.scene.draw(g2);
        if (mMode == POINT_MODE && mSubMode == POINT_BEGUN_SUBMODE) {
            g2.setPaint(Color.gray);
            mPointDrawingPaintPoint.setFrame((int)mPointDrawingTrackPoint.getX() - (pointSize / 2),(int)mPointDrawingTrackPoint.getY() - (pointSize / 2), pointSize, pointSize);
            g2.fill(mPointDrawingPaintPoint);
        }
        else if (mMode == TEXT_MODE && mSubMode == TEXT_BEGUN_SUBMODE) {  // Draw text begun icon here...
            g2.setPaint(Color.gray);
            g2.setFont(Text.getDefaultFont());
            g2.drawString("Text", (int)mPointDrawingTrackPoint.getX(), (int)mPointDrawingTrackPoint.getY() - 1);
        }
        else if (mMode == LINE_MODE && mSubMode == LINE_BEGUN_SUBMODE) {
            g2.setStroke(mLineDrawingStroke);
            g2.setPaint(Color.gray);
            mLineDrawingTrackingLine.setLine(mLineStartPoint, mLineEndPoint);
            g2.draw(mLineDrawingTrackingLine);
        }
        else if (mMode == POLYGON_MODE && mSubMode == POLYGON_BEGUN_SUBMODE) {
            g2.setStroke(mLineDrawingStroke);
            g2.setPaint(Color.gray);
            if (mPolygonPoints.size() > 1) {
              for (int i = 1; i < mPolygonPoints.size(); i++) {
                Point2D.Double from = (Point2D.Double)mPolygonPoints.get(i-1);
                Point2D.Double to = (Point2D.Double)mPolygonPoints.get(i);
                mLineDrawingTrackingLine.setLine(from, to);
                g2.draw(mLineDrawingTrackingLine);
              }
            }
            if (mPolygonPoints.size() > 0) {
              Point2D.Double from = (Point2D.Double)mPolygonPoints.get(mPolygonPoints.size()-1);
              mLineDrawingTrackingLine.setLine(from, mLineEndPoint);
              g2.draw(mLineDrawingTrackingLine);
            }
        }
      mInRepaint = false;
    }

    public void mouseClicked(MouseEvent e) {
      if (mMode == POLYGON_MODE && mSubMode == POLYGON_BEGUN_SUBMODE) {
        if (e.getClickCount() == 2) {
          finishPolygon(g.activeEnclosure);
        }
      }
    }

    private void finishPolygon(Enclosure currentEnclosure) {
      mSubMode = NONE;
      if (mPolygonPoints.size() > 2) {
        String newName = currentEnclosure.newName("poly");
        Polygon poly = new Polygon(newName, currentEnclosure);
        Point2D placePoint = new Point2D.Double();


        for (int i = 0; i < mPolygonPoints.size(); i++) {
          placePoint.setLocation((Point2D)mPolygonPoints.get(i));
          currentEnclosure.inverseTransform(placePoint);
          poly.addPoint(g.transform.toGeomLocation(placePoint));
        }
        if (g.HAS_SCENEGRAPH) {
          g.sceneGraphPanel.addNewEntityNode(poly, currentEnclosure);
        }
      }
      repaint();
      mPolygonPoints.clear();
    }

    public void mousePressed(MouseEvent e) {
        g.mouseDown = true;
        mMouseDown = true;

        if (mMode == POLYGON_MODE)
          return;

        if (mMode == POINT_MODE) {
            mSubMode = POINT_BEGUN_SUBMODE;
            mPointDrawingTrackPoint.setLocation(e.getX(), e.getY());
            repaint();
        }

        if (mMode == TEXT_MODE) {
            mSubMode = TEXT_BEGUN_SUBMODE;
            mPointDrawingTrackPoint.setLocation(e.getX(), e.getY());
            repaint();
        }

        if (mMode == LINE_MODE) {
          mSubMode = LINE_BEGUN_SUBMODE;
          mLineStartPoint.setLocation(e.getX(), e.getY());
          mLineEndPoint.setLocation(mLineStartPoint);
          repaint();
        }

        mMousePressedPoint.setLocation(e.getX(), e.getY());
        mSelection = g.scene.hit(mMousePressedPoint);

        if (mLastSelection != null) {
            mLastSelection.setSelected(false);
//            mLastSelection.sendToBack();
        }

        mLastSelection = mSelection;
        if (mSelection == null)
            return;

        mSelection.setSelected(true);

        mSelection.mEnclosure.inverseTransform(mMousePressedPoint);
//        mSelection.bringToFront();
        mSelection.savePreDragLocation();
        mMouseDraggedTo.setLocation(mMousePressedPoint.getX(), mMousePressedPoint.getY());

    }

    public void mouseReleased(MouseEvent e) {
        g.mouseDown = false;
        mMouseDown = false;
//        SceneGraphPanel graph = scene().getSceneGraph();
        Enclosure currentEnclosure;

        if (mMode == ARROW_MODE) {
            if (mSelection == null)
                return;
//            if (graph != null)
//                graph.update();
        }

        currentEnclosure = g.activeEnclosure;

        if (currentEnclosure == null)
            currentEnclosure = g.scene;

        if (mMode == POINT_MODE) {
            mSubMode = NONE;
            String newName = currentEnclosure.newName("point");
            roebling.Point p = new roebling.Point(newName, currentEnclosure);
            Point2D placePoint = new Point2D.Double(e.getX(), e.getY());
            currentEnclosure.inverseTransform(placePoint);
            p.setLocation(g.transform.toGeomLocation(placePoint));
            if (g.HAS_SCENEGRAPH) {
              g.sceneGraphPanel.addNewEntityNode(p, currentEnclosure);
            }
            repaint();
//            if (graph != null) {
//                graph.update();
//            }
        }
        if (mMode == IMAGE_MODE) {
            mSubMode = NONE;

//            String newName = currentEnclosure.newName("button");
//            roebling.Button b = new roebling.Button(newName, currentEnclosure);
//            Point2D geomPoint = g.transform.toGeomLocation(new Point2D.Double(e.getX(), e.getY()));
//            b.setTopLeft(geomPoint);
//            b.setBottomRight(new Point2D.Double(geomPoint.getX() + 120, geomPoint.getY() - 30));
//            repaint();

            String newName = currentEnclosure.newName("image");
            roebling.Image image = new roebling.Image(newName, currentEnclosure);
            Point2D placePoint = new Point2D.Double(e.getX(), e.getY());
            currentEnclosure.inverseTransform(placePoint);
//            Point2D geomPoint = g.transform.toGeomLocation(new Point2D.Double(e.getX(), e.getY()));
            image.setTopLeft(g.transform.toGeomLocation(placePoint));
            //image.setBottomRight(new Point2D.Double(geomPoint.getX() + 120, geomPoint.getY() - 30));
            if (g.HAS_SCENEGRAPH) {
              g.sceneGraphPanel.addNewEntityNode(image, currentEnclosure);
            }
            repaint();
        }
        if (mMode == BUTTON_MODE) {   // Buttons don't transform nicely yet
            mSubMode = NONE;

//            String newName = currentEnclosure.newName("button");
//            roebling.Button b = new roebling.Button(newName, currentEnclosure);
//            Point2D geomPoint = g.transform.toGeomLocation(new Point2D.Double(e.getX(), e.getY()));
//            b.setTopLeft(geomPoint);
//            b.setBottomRight(new Point2D.Double(geomPoint.getX() + 120, geomPoint.getY() - 30));
//            repaint();

            String newName = currentEnclosure.newName("button");
            roebling.Button button = new roebling.Button(newName, currentEnclosure);
            Point2D geomPoint = g.transform.toGeomLocation(new Point2D.Double(e.getX(), e.getY()));
            button.setTopLeft(geomPoint);
            button.setBottomRight(new Point2D.Double(geomPoint.getX() + 120, geomPoint.getY() - 30));
            if (g.HAS_SCENEGRAPH) {
              g.sceneGraphPanel.addNewEntityNode(button, currentEnclosure);
            }
            repaint();
        }
        if (mMode == TEXT_MODE) {
            mSubMode = NONE;
            String newName = currentEnclosure.newName("text");
            roebling.Text t = new roebling.Text(newName, currentEnclosure);
            Point2D placePoint = new Point2D.Double(e.getX(), e.getY());
            currentEnclosure.inverseTransform(placePoint);
            t.setLocation(g.transform.toGeomLocation(placePoint));
            if (g.HAS_SCENEGRAPH) {
              g.sceneGraphPanel.addNewEntityNode(t, currentEnclosure);
            }
            repaint();
//            if (graph != null) {
//                graph.update();
//            }
        }

        if (mMode == LINE_MODE) {
           if (mSubMode == LINE_BEGUN_SUBMODE) {
                mSubMode = NONE;
                if (mLineStartPoint.distance(e.getX(), e.getY()) >= 2.0) {

                  String newName = currentEnclosure.newName("line");
                  roebling.Line l = new roebling.Line(newName, currentEnclosure);
                  Point2D placePoint = new Point2D.Double(e.getX(), e.getY());
                  currentEnclosure.inverseTransform(placePoint);
                  currentEnclosure.inverseTransform(mLineStartPoint);
                  l.setStartPoint(g.transform.toGeomLocation(mLineStartPoint));
                  l.setEndPoint(g.transform.toGeomLocation(placePoint));
                  if (g.HAS_SCENEGRAPH) {
                    g.sceneGraphPanel.addNewEntityNode(l, currentEnclosure);
                  }
                }
                repaint();
//                if (graph != null) {
//                    graph.update();
//                }
            }
        }

        if (mMode == POLYGON_MODE) {
          if (mSubMode != POLYGON_BEGUN_SUBMODE) {    // Polygon not begun yet.
            mSubMode = POLYGON_BEGUN_SUBMODE;
            mPolygonPoints.add(new Point2D.Double(e.getX(), e.getY()));
          }
          else {      // Polygon under way.
            if (((Point2D)mPolygonPoints.get(0)).distance(e.getX(), e.getY()) <= 3.0) {
                finishPolygon(currentEnclosure);
            }
            else {
              if (((Point2D)mPolygonPoints.lastElement()).distance(e.getX(), e.getY()) > 1.0) {
                mPolygonPoints.add(new Point2D.Double(e.getX(), e.getY()));
              }
            }
          }
        }

        System.gc();

    }
    public void mouseEntered(MouseEvent e) {
      if (! g.HAS_SCENEGRAPH)
        return;

      if (mMode == POINT_MODE) {
        setCursor(g.pointCursor);
      }
      else if (mMode == LINE_MODE) {
        setCursor(g.lineCursor);
      }
      else if (mMode == POLYGON_MODE) {
        setCursor(g.polyCursor);
      }
      else if (mMode == IMAGE_MODE) {
        setCursor(g.imageCursor);
      }
      else if (mMode == BUTTON_MODE) {
        setCursor(g.buttonCursor);
      }
      else if (mMode == TEXT_MODE) {
        setCursor(g.textCursor);
      }
      else
        setCursor(g.defaultCursor);

    }
    public void mouseExited(MouseEvent e) {
    }
    public void mouseDragged(MouseEvent e) {
      g.mouseX = g.transform.xToGeom(e.getX());
      g.mouseY = g.transform.yToGeom(e.getY());
      if (mMode == POINT_MODE && mSubMode == POINT_BEGUN_SUBMODE) {
        mPointDrawingTrackPoint.setLocation(e.getX(), e.getY());
        repaint();
        return;
      }
      if (mMode == TEXT_MODE && mSubMode == TEXT_BEGUN_SUBMODE) {
        mPointDrawingTrackPoint.setLocation(e.getX(), e.getY());
        repaint();
        return;
      }
      if (mMode == LINE_MODE) {
        if (mSubMode == LINE_BEGUN_SUBMODE) {
          mLineEndPoint.setLocation(e.getX(), e.getY());
          repaint();
          return;
        }
      }
      if (mMode == POLYGON_MODE) {
        return;
      }

      if (mSelection == null)
        return;

      mSelection.mEnclosure.inverseTransform(new Point2D.Double(e.getX(), e.getY()), mTransDrag);
//      mMouseDraggedTo.setLocation(e.getX(), e.getY());
//      mMouseDelta.setLocation(e.getX() - mMousePressedPoint.getX(), e.getY() - mMousePressedPoint.getY());

      mMouseDraggedTo.setLocation(mTransDrag.getX(), mTransDrag.getY());
      mMouseDelta.setLocation(mTransDrag.getX() - mMousePressedPoint.getX(), mTransDrag.getY() - mMousePressedPoint.getY());

      mSelection.move(mMouseDelta);
      repaint();
  }

    public void mouseMoved(MouseEvent e) {
      g.mouseX = g.transform.xToGeom(e.getX());
      g.mouseY = g.transform.yToGeom(e.getY());
        if (mMode == LINE_MODE) {
            if (mSubMode == LINE_BEGUN_SUBMODE) {
                mLineEndPoint.setLocation(e.getX(), e.getY());
                repaint();
            }
        }
        else if (mMode == POLYGON_MODE && mSubMode == POLYGON_BEGUN_SUBMODE) {
          mLineEndPoint.setLocation(e.getX(), e.getY());
          repaint();
        }
    }

    public void arrowButtonHit() {
      mMode = ARROW_MODE;
    }

    public void pointButtonHit() {
      mMode = POINT_MODE;
    }

    public void buttonButtonHit() {
      mMode = BUTTON_MODE;
    }

    public void imageButtonHit() {
      mMode = IMAGE_MODE;
    }

    public void lineButtonHit() {
      mMode = LINE_MODE;
    }

    public void polygonButtonHit() {
      mMode = POLYGON_MODE;
    }

    public void textButtonHit() {
      mMode = TEXT_MODE;
    }

    public void resetToArrowMode() {
        if (mMode != ARROW_MODE) {
            if (g.sceneFrame != null) {
                g.sceneFrame.resetButtons();
                g.sceneFrame.selectArrow();
            }
            arrowButtonHit();
        }
    }

    public void forceButtonRepaint() {
        paintComponent(getGraphics());
        g.scene.forceButtonRepaint((Graphics2D)getGraphics());
    }
}