package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

import java.awt.*;
import java.awt.geom.*;

public class ScenePanelTransform {

    public ScenePanelTransform() {
    }

    public ScenePanelTransform(Container container) {
        _container = container;
        centerOrigin();
    }

    public double getXMin() {
        if (_xScale >= 0.0) return xToGeom(0.0);
        else return xToGeom(_container.getWidth());
    }

    public double getXMax() {
        if (_xScale >= 0.0) return xToGeom(_container.getWidth());
        else return xToGeom(0.0);
    }

    public double getYMin() {
        if (_yScale >= 0.0) return yToGeom(0.0);
        else return yToGeom(_container.getHeight());
    }

    public double getYMax() {
        if (_yScale >= 0.0) return yToGeom(_container.getHeight());
        else return yToGeom(0.0);
    }

    public double getPanelXMin() {
        return 0.0;
    }

    public double getPanelXMax() {
        return _container.getWidth();
    }

    public double getPanelYMin() {
        return 0.0;
    }

    public double getPanelYMax() {
        return _container.getHeight();
    }

    public double xToPanel(double xGeom) {
        return (xGeom / _xScale) + _xOrigin;
    }

    public double yToPanel(double yGeom) {
        return (yGeom / _yScale) + _yOrigin;
    }

    public Point2D toPanelLocation(Point2D geometricLocation) {
        return new Point2D.Double((geometricLocation.getX() / _xScale) + _xOrigin, (geometricLocation.getY() / _yScale) + _yOrigin);
    }

    public double xToGeom(double xPanel) {
        return (xPanel - _xOrigin) * _xScale;
    }

    public double yToGeom(double yPanel) {
        return (yPanel - _yOrigin) * _yScale;
    }

    public Point2D toGeomLocation(Point2D panelLocation) {
        return new Point2D.Double((panelLocation.getX() - _xOrigin) * _xScale, (panelLocation.getY() - _yOrigin) * _yScale);
    }

    public Point2D toGeomDelta(Point2D panelDelta) {
        return new Point2D.Double(panelDelta.getX() * _xScale, panelDelta.getY() * _yScale);
    }

    public Point2D toPanelDelta(Point2D geomDelta) {
        return new Point2D.Double(geomDelta.getX() / _xScale, geomDelta.getY() / _yScale);
    }

    public void centerOrigin() {
        setXOrigin(_container.getWidth() / 2.0);
        setYOrigin(_container.getHeight() / 2.0);
    }

    public double getXScale() {return _xScale;}
    public double getYScale() {return _yScale;}
    public double getXOrigin() {return _xOrigin;}
    public double getYOrigin() {return _yOrigin;}

    public void   setXScale(double xScale) {_xScale = xScale;}
    public void   setYScale(double yScale) {_yScale = yScale;}
    public void   setXOrigin(double xOrigin) {_xOrigin = xOrigin;}
    public void   setYOrigin(double yOrigin) {_yOrigin = yOrigin;}

    private double _xOrigin, _yOrigin;  // Origin is in panel coordinates
    private double _xScale = 1.0;
    private double _yScale = -1.0;
    private Container _container;
}