package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */

public class StringProp extends Property {

  public String value = "";
  public String values[] = new String[0];

  public StringProp() {
    super();
    mType = STRING;
  }

  public void copyValue(Property property) {
    if (isArray) {
      values = (String[])((StringProp)property).values.clone();
    }
    else {
      value = new String(((StringProp)property).value);
    }
  }

  public String valueAsString() {
    validate();
    return value;
  }

  public int arrayLength() {
    return values.length;
  }

  public void setArrayLength(int length) {
    values = new String[length];
    for (int i = 0; i < length; i++) {
      values[i] = "";
    }
  }
  public void setValueByName(String fieldName, String aValue) {  // used for compound types
    value = new String(aValue);
  }

  public void setValueByName(String fieldName, int index, String aValue) {  // used for compound types
    values[index] = new String(aValue);
  }

  public void insertAt(int index) {
    values = (String[])Util.insert(values, ("").getClass(), "", index);
  }

  public void deleteItem(int index) {
    values = (String[])Util.remove(values, ("").getClass(), index);
  }

  public boolean serializeProperty(XMLWriter xmlOut) {
    try {
      xmlOut.startElement("property");

      super.serializeProperty(xmlOut);
      if (isArray) {
        xmlOut.startElement("value");
        Util.serializeElement(xmlOut, "length", values.length);
        for (int i = 0; i < values.length; i++) {
          if (values[i].equals("")) {
            Util.serializeElement(xmlOut, "index", "$$$EMPTY$$$");
          }
          else
            Util.serializeElement(xmlOut, "index", values[i]);
        }
        xmlOut.endElement("value");
      }
      else {
        if (value.equals(""))
          Util.serializeElement(xmlOut, "value", "$$$EMPTY$$$");
        else
          Util.serializeElement(xmlOut, "value", value);
      }

      xmlOut.endElement("property");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }

}