package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.util.*;

public class TimerQueue {

  private TimerQueueEntry[] mTimerQueue;
  private boolean mStarted = false;
  private Timer mTimer;
  private RefreshTask mRefreshTask;
  private final long TIMER_DELAY = 20L;

  public TimerQueue() {
    mTimerQueue = new TimerQueueEntry[0];
//    mRefreshTask = new RefreshTask();
//    mTimer = new Timer();
  }

  public void addToTimerQueue(Function function) {
    TimerQueueEntry entry = new TimerQueueEntry(function);
    mTimerQueue = (TimerQueueEntry[])Util.append(mTimerQueue, entry.getClass(), entry);
    if (!mStarted) {
      start();
      mStarted = true;
    }
  }

  public void removeFromTimerQueue(Function function) {
    TimerQueueEntry entry;
    TimerQueueEntry[] tempQueue = (TimerQueueEntry[])mTimerQueue.clone();
    for (int i = 0; i < tempQueue.length; i++) {
      entry = tempQueue[i];
      if (entry.function.equals(function)) {
        mTimerQueue = (TimerQueueEntry[])Util.remove(mTimerQueue, entry.getClass(), entry);
      }
    }
    if (mTimerQueue.length == 0) {
      if (mTimer != null)
        mTimer.cancel();
        mStarted = false;
    }
  }

  private void start() {
    mTimer = new Timer();
    mRefreshTask = new RefreshTask();
    mTimer.schedule(mRefreshTask, 0L, TIMER_DELAY);
  }

  public class RefreshTask extends TimerTask {
    private boolean mRefresh = false;
/*    private TaskThread mTaskThread = new TaskThread();
    private Thread mLastThread;

    public void run() {

      //System.out.println("TaskThread alive: " + mTaskThread.isAlive());
      if (mLastThread == null || !mLastThread.isAlive()) {
        mLastThread = new Thread(mTaskThread);
        mLastThread.start();
      }
    }

    public class TaskThread implements Runnable {*/
      public void run() {

        try {
          mRefresh = false;
          for (int i = 0; i < mTimerQueue.length; i++) {
            TimerQueueEntry entry = mTimerQueue[i];
            entry.delayCount--;
            if (entry.delayCount <= 0) {
              entry.function.needsRefresh();      // HERE MAY NEED WORK
              entry.delayCount = entry.function.mTimerInterval;
              mRefresh = true;
            }
          }
          if (mRefresh)
            g.scene.refresh();
        }
        catch (Exception e) {
        }
      }
    //}
  }

  public class TimerQueueEntry {

    public TimerQueueEntry(Function aFunction) {
      function = aFunction;
      delayCount = aFunction.mTimerInterval;
    }

    public Function function;
    public int delayCount;
  }
}