package roebling;


/**
* UndoableTextArea.java
* This is the TextArea with Undo and Redo Capabilities
*
* @author Rahul Sapkal(rahul@javareference.com)
*/

/*
* This example is from javareference.com
* for more information visit,
* http://www.javareference.com
*/

//import statements
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.undo.*;

public class UndoableTextArea extends JTextAreaNoAntialias implements UndoableEditListener, FocusListener, KeyListener
{
    //Setting Undo Limit to 1500 edits
    public static final int UNDO_LIMIT = 1500;

    //UndoManager
    private UndoManager m_undoManager;

    //constructor
    public UndoableTextArea()
    {
        this(new String());
    }

    //constructor
    public UndoableTextArea(String text)
    {
        super(text);

        //add the UndoableEditListener to the TextArea
        getDocument().addUndoableEditListener(this);
        //add the KeyListener
        this.addKeyListener(this);
        //add the FocusListener
        this.addFocusListener(this);
    }

    //createUndoMananger creating undo manager
    private void createUndoMananger()
    {
        m_undoManager = new UndoManager();
        m_undoManager.setLimit(UNDO_LIMIT);
    }

    //removeUndoMananger removing undo manager
    private void removeUndoMananger()
    {
        m_undoManager.end();
    }

    public void focusGained(FocusEvent fe)
    {
        //creating undo manager on focus gained
        createUndoMananger();
    }

    public void focusLost(FocusEvent fe)
    {
        //removing undo manager on focus lose
        removeUndoMananger();
    }

    //undoableEditHappened called when edit happened
    public void undoableEditHappened(UndoableEditEvent e)
    {
        //add the edits to the undo manager
        if (m_undoManager == null)
          createUndoMananger();
        m_undoManager.addEdit(e.getEdit());
    }

    public void keyPressed(KeyEvent e)
    {
        //identify the key pressed is the combination of
        //Controlkey and Z key
        if((e.getKeyCode() == KeyEvent.VK_Z) && (e.isControlDown()))
        {
            try
            {
                //Undo changes
                m_undoManager.undo();
            }
            catch(CannotUndoException cue)
            {
                Toolkit.getDefaultToolkit().beep();
            }
        }

        //identify the key pressed is the combination of
        //Controlkey and Y key
        if((e.getKeyCode() == KeyEvent.VK_Y) && (e.isControlDown()))
        {
            try
            {
                //Redo changes
                m_undoManager.redo();
            }
            catch(CannotRedoException cue)
            {
                Toolkit.getDefaultToolkit().beep();
            }
        }
    }

    public void keyReleased(KeyEvent e)
    {
    }

    public void keyTyped(KeyEvent e)
    {
    }
}