package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.lang.reflect.*;
import java.awt.Color;
import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.lang.Math.*;

public class Util {

    private static final Util instance = new Util(); // Util is a singleton

    public Util() {
    }

    public static Util getInstance() {
        return instance;
    }

/*    public static double round(double value, int decimals) {
        double mult = (double)(10 ^ decimals);
        return ((int)(value * mult)) / mult;
    }*/

    public static double clamp(double value, double low, double high) {
      return Math.max(Math.min(value, high ), low);
    }

    public static String encodeStringValue(String unencoded) {  //Makes spaces into ~
        StringBuffer buffer = new StringBuffer(unencoded);
        replaceString(buffer, " ", "~");
        return buffer.toString();
    }

    public static String decodeStringValue(String encoded) {  //Makes spaces into ~
        StringBuffer buffer = new StringBuffer(encoded);
        replaceString(buffer, "~", " ");
        return buffer.toString();
    }

    public static void replaceString(StringBuffer text, String preString, String postString) {
        int position = text.toString().indexOf(preString);
        int replaceLength = preString.length();
        String replaceString = postString;
        while (position >= 0) {
            text.replace(position, position + replaceLength, replaceString);
            position = text.toString().indexOf(preString);
        }
    }

    public static String round(double d, int place)
    {
      if (place <= 0)
        return ""+(int)(d+((d > 0)? 0.5 : -0.5));
      String s = "";
      if (d < 0)
        {
  	s += "-";
  	d = -d;
        }
      d += 0.5*Math.pow(10,-place);
      if (d > 1)
        {
  	int i = (int)d;
  	s += i;
  	d -= i;
        }
      else
        s += "0";
      if (d > 0)
        {
  	d += 1.0;
  	String f = ""+(int)(d*Math.pow(10,place));
  	s += "."+f.substring(1);
        }
      return s;
    }


    public void shiftRight(Object[] array, int index) {
        for (int i = index; i > 0; i--)
            array[i] = array[i-1];
    }

    public void shiftLeft(Object[] array, int index) {
        for (int i = index; i < array.length - 1; i++)
            array[i] = array[i+1];
    }

    public static boolean near (double first, double second, double tolerance) {
        return (java.lang.Math.abs(first - second) <= tolerance);
    }

    public static int[] append(int[] array, int newInt) {
      int[] newArray = new int[array.length + 1];
      System.arraycopy(array, 0, newArray, 0, array.length);
      newArray[array.length] = newInt;
      return newArray;
    }

    public static double[] append(double[] array, double newDouble) {
      double[] newArray = new double[array.length + 1];
      System.arraycopy(array, 0, newArray, 0, array.length);
      newArray[array.length] = newDouble;
      return newArray;
    }

    public static boolean[] removeBoolean(boolean[] array, int index) {
      boolean[] newArray = new boolean[array.length - 1];
      System.arraycopy(array, 0, newArray, 0, index);
      System.arraycopy(array, index + 1, newArray, index, newArray.length - index);
      return newArray;
    }

    public static double[] removeDouble(double[] array, int index) {
      double[] newArray = new double[array.length - 1];
      System.arraycopy(array, 0, newArray, 0, index);
      System.arraycopy(array, index + 1, newArray, index, newArray.length - index);
      return newArray;
    }

    public static int[] removeIndex(int[] array, int index) {
      int[] newArray = new int[array.length - 1];
      System.arraycopy(array, 0, newArray, 0, index);
      System.arraycopy(array, index + 1, newArray, index, newArray.length - index);
      return newArray;
    }

    public static int[] removeInt(int[] array, int oldItem) {
        int index = find(array, oldItem);
        if (index == -1)
            return array;
        return removeIndex(array, index);
    }

    public static int find(int[] array, int value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value)
                return i;
        }
        return -1;
    }

    public static Object[] append(Object[] array, Class type, Object newItem) {
        Object[] newArray = (Object[])Array.newInstance(type, array.length + 1);
        System.arraycopy(array, 0, newArray, 0, array.length);
        newArray[array.length] = newItem;
        return newArray;
    }

    public static Object[] insert(Object[] array, Class type, Object newItem, int index) {
      Object[] newArray = (Object[])Array.newInstance(type, array.length + 1);
      System.arraycopy(array, 0, newArray, 0, index);
      newArray[index] = newItem;
      System.arraycopy(array, index, newArray, index + 1, array.length - index);
      return newArray;
    }

    public static boolean[] insertBoolean(boolean[] array, boolean newItem, int index) {
      boolean[] newArray = new boolean[array.length + 1];
      System.arraycopy(array, 0, newArray, 0, index);
      newArray[index] = newItem;
      System.arraycopy(array, index, newArray, index + 1, array.length - index);
      return newArray;
    }

    public static double[] insertDouble(double[] array, double newItem, int index) {
      double[] newArray = new double[array.length + 1];
      System.arraycopy(array, 0, newArray, 0, index);
      newArray[index] = newItem;
      System.arraycopy(array, index, newArray, index + 1, array.length - index);
      return newArray;
    }

    public static Object[] addFront(Object[] array, Class type, Object newItem) {
        Object[] newArray = (Object[])Array.newInstance(type, array.length + 1);
        System.arraycopy(array, 0, newArray, 1, array.length);
        newArray[0] = newItem;
        return newArray;
    }

    public static Object[] remove(Object[] array, Class type, int index) {
        Object[] newArray = (Object[])Array.newInstance(type, array.length - 1);
        System.arraycopy(array, 0, newArray, 0, index);
        System.arraycopy(array, index + 1, newArray, index, newArray.length - index);
        return newArray;
    }

    public static Object[] remove(Object[] array, Class type, Object oldItem) {
        int index = find(array, oldItem);
        if (index == -1)
            return array;
        return remove(array, type, index);
    }

    public static int find(Object[] array, Object item) {
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(item))
                return i;
        }
        return -1;
    }

    public static boolean contains(Object[] array, Object item) {
        return find(array, item) != -1;
    }

    public static String generateGUID() {
        long ip; //the 0-63th bit
        long t;  //the 64-127 bit
        try {
            byte[] ipa=java.net.InetAddress.getLocalHost().getAddress();
            ip=(long)(""+ipa[0]+ipa[1]).hashCode();
            ip=ip<<32 | ((long)(""+ipa[2]+ipa[3]).hashCode());
        }
        catch (java.net.UnknownHostException e) {
            ip=(long)(Math.random()*(Long.MAX_VALUE));
        }
        t=System.currentTimeMillis();
        String sip=Long.toHexString(ip);
        while (sip.length()<16)
            sip="0"+sip;
        String st=Long.toHexString(t);
        while (st.length()<16)
            st="0"+st;
        String ret=sip+st;
        ret=ret.substring(0,8)+"_"+ret.substring(8,12)+"_"+ret.substring(12,16)+"_"+ret.substring(16,20)+"_"+ret.substring(20);
        return ret;
    }

    public static boolean parseBoolean(String s) {
        String testString = s.toLowerCase();
        if (testString.equals("t") ||
            testString.equals("true") ||
            testString.equals("y") ||
            testString.equals("yes") ||
            testString.equals("1"))
            return true;
        return false;
    }

    public static Color copyColor(Color from) {
        Color to = new Color(from.getRed(),
                             from.getGreen(),
                             from.getBlue(),
                             from.getAlpha());
        return to;
    }

    public static void tr(String trace) {
        System.out.println(trace);
    }

    public static double bound (double value, double lower, double upper) {
        return java.lang.Math.min(java.lang.Math.max(value, lower), upper);
    }

/*    public static void changeCoordinates(Point2D location, Component fromComponent, Component toComponent) {
        try {
            java.awt.Point fromComponentOrigin = fromComponent.getLocationOnScreen();
            java.awt.Point toComponentOrigin = toComponent.getLocationOnScreen();

            location.setLocation(fromComponentOrigin.getX() + location.getX() - toComponentOrigin.getX(),
                             fromComponentOrigin.getY() + location.getY() - toComponentOrigin.getY());
        }
        catch (Exception e) {
            //System.out.println("Bad coordinate change.");
            location.setLocation(0,0);
        }

    }*/

    public static void changeCoordinates(Point2D location, Component fromComponent, Component toComponent) {
        try {
            location.setLocation(SwingUtilities.convertPoint(fromComponent,(int)location.getX(), (int)location.getY(), toComponent));
        }
        catch (Exception e) {
            location.setLocation(0,0);
        }
    }


    public static void scalePoint(Point2D scaleCenter, Point2D preScaled, double scaleFactor, Point2D postScaled) {
        double newDistance = scaleCenter.distance(preScaled) * scaleFactor;
        double theta = Math.atan2((preScaled.getY() - scaleCenter.getY()) , (preScaled.getX() - scaleCenter.getX()));

        postScaled.setLocation(scaleCenter.getX() + (newDistance * Math.cos(theta)),
                               scaleCenter.getY() + (newDistance * Math.sin(theta)));
    }

    public static boolean serializeElement(XMLWriter xmlOut, String tagName, String value) {
        try {
            xmlOut.startElement(tagName);
            xmlOut.characters(value);
            xmlOut.endElement(tagName);
        }
        catch (Exception e) {
            return false;
        }
        return true;
    }

    public static boolean serializeElement(XMLWriter xmlOut, String tagName, double value) {
        try {
            String outString;
            xmlOut.startElement(tagName);
            outString = String.valueOf(value);
            xmlOut.characters(outString);
            xmlOut.endElement(tagName);
        }
        catch (Exception e) {
            return false;
        }
        return true;
    }

    public static boolean serializeElement(XMLWriter xmlOut, String tagName, int value) {
        try {
            String outString;
            xmlOut.startElement(tagName);
            outString = String.valueOf(value);
            xmlOut.characters(outString);
            xmlOut.endElement(tagName);
        }
        catch (Exception e) {
            return false;
        }
        return true;
    }

    public static boolean serializeElement(XMLWriter xmlOut, String tagName, boolean value) {
        try {
            String outString;
            xmlOut.startElement(tagName);
            outString = String.valueOf(value);
            xmlOut.characters(outString);
            xmlOut.endElement(tagName);
        }
        catch (Exception e) {
            return false;
        }
        return true;
    }
}