package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.io.*;
import java.util.*;

public class XMLReader {

  private File mFile;
  private Reader mReader;
  private XMLParser mParser;

  private StringBuffer mAccumulator = new StringBuffer(255);
  private int mCur;
  private boolean mIgnore;

  public static final int CHARS = 0;
  public static final int TAG = 1;
  public static final int END_TAG = 2;


  int mMode;

  public XMLReader(File xmlFile, XMLParser parser) {
    mFile = xmlFile;
    try {
      mReader = new BufferedReader(new FileReader(xmlFile));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    mParser = parser;
    mAccumulator.setLength(0);
  }

  private void resetAccum() {
    mAccumulator.setLength(0);
    mIgnore = false;
  }

  private String decode(StringBuffer buffer) {
    Util.replaceString(buffer, "&GT", ">");
    Util.replaceString(buffer, "&LT", "<");
    return buffer.toString();
  }

  public void parse() {
    mMode = CHARS;
    mIgnore = false;
    try {
      while ((mCur = mReader.read()) > 0) {
        if (mCur == '\n' || mCur == '\r')
          continue;
        switch (mMode) {
          case CHARS:
            if (mCur == '<') {
              mMode = TAG;
              if (mAccumulator.length() > 0) {
                mParser.characters(decode(mAccumulator));
              }
              resetAccum();
            }
            else
              mAccumulator.append((char)mCur);
            break;
          case TAG:
            if (mCur == '>') {
              mMode = CHARS;
              if (mAccumulator.length() > 0) {
                mParser.startElement(mAccumulator.toString());
              }
              resetAccum();
            }
            else if (mCur == '?') {
              mIgnore = true;
            }
            else if (mCur == '/') {
              mMode = END_TAG;
            }
            else
              if (!mIgnore)
                mAccumulator.append((char)mCur);
            break;
          case END_TAG:
            if (mCur == '>') {
              mMode = CHARS;
              if (mAccumulator.length() > 0) {
                mParser.endElement(mAccumulator.toString());
              }
              resetAccum();
            }
            else
              mAccumulator.append((char)mCur);
            break;
        }
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

}