package roebling;

/**
 * Title:        Roebling
 * Description:  Parametric dataflow programming.
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Simon Greenwold
 * @version 1.0
 */
import java.io.*;
import java.util.*;

public class XMLWriter {

  private File mFile;
  private Writer mWriter;
  private StringBuffer encoded;

  public XMLWriter(File file) {
    mFile = file;
    encoded = new StringBuffer();
  }

  public void startElement(String tag) {
    try {
      mWriter.write("<" + tag + ">\n");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void endElement(String tag) {
    try {
      mWriter.write("</" + tag + ">\n");
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void characters(String characters) {
    try {
      mWriter.write(encode(characters));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private String encode(String string) {
    char cur;

    encoded.setLength(0);
    for (int i = 0; i < string.length(); i++) {
      cur = string.charAt(i);
      if (cur == '<') {
        encoded.append("&LT");
      }
      else if (cur == '>') {
        encoded.append("&GT");
      }
      else
        encoded.append(cur);
    }
    return encoded.toString();
  }

  public void startDocument() {
    try {
      if (mFile.exists()) {
        mFile.delete();
      }
      mFile.createNewFile();
      mWriter = new BufferedWriter(new FileWriter(mFile));
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void endDocument() {
    try {
      mWriter.close();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
}